<?php
/**
 * AJAX Handler for eSignature Operations
 * Provides API endpoints for frontend integration
 */

// Set headers for JSON response
header('Content-Type: application/json');
header('Cache-Control: no-cache, no-store, must-revalidate');

// Include required files
require_once 'config.php';
require_once 'db-config.php';

// Create log directory if it doesn't exist
if (!is_dir(LOG_DIR)) {
    mkdir(LOG_DIR, 0755, true);
}

// Get request method and action
$method = $_SERVER['REQUEST_METHOD'];
$action = isset($_GET['action']) ? $_GET['action'] : '';

// Process based on request method and action
if ($method === 'POST') {
    // Get POST data (JSON or form)
    $input = json_decode(file_get_contents('php://input'), true);
    if (!$input && count($_POST) > 0) {
        $input = $_POST;
    }
    
    switch ($action) {
        case 'request_otp':
            handleRequestOTP($input);
            break;
            
        case 'verify_otp':
            handleVerifyOTP($input);
            break;
            
        case 'sign_document':
            handleSignDocument($input);
            break;
            
        default:
            sendErrorResponse('Unknown action');
            break;
    }
} elseif ($method === 'GET') {
    switch ($action) {
        case 'check_status':
            handleCheckStatus();
            break;
            
        default:
            sendErrorResponse('Unknown action');
            break;
    }
} else {
    sendErrorResponse('Invalid request method');
}

/**
 * Handle OTP request
 * 
 * @param array $data Request data
 */
function handleRequestOTP($data) {
    if (!isset($data['email'])) {
        sendErrorResponse('Email address is required');
        return;
    }
    
    $email = filter_var($data['email'], FILTER_SANITIZE_EMAIL);
    if (!filter_var($email, FILTER_VALIDATE_EMAIL)) {
        sendErrorResponse('Invalid email address');
        return;
    }
    
    $document_id = isset($data['document_id']) ? $data['document_id'] : 'temp_document';
    
    // Generate OTP
    $otp = generateOTP();
    
    // Store OTP in database
    if (storeOTP($email, $otp, $document_id)) {
        // In development mode, return the OTP for testing
        if (APP_ENV === 'development') {
            sendSuccessResponse([
                'message' => 'OTP generated successfully (Development Mode)',
                'dev_otp' => $otp
            ]);
        } else {
            // In production, send the OTP via email
            if (sendOTPEmail($email, $otp)) {
                sendSuccessResponse(['message' => 'OTP sent successfully']);
            } else {
                sendErrorResponse('Failed to send OTP email');
            }
        }
    } else {
        sendErrorResponse('Failed to generate OTP');
    }
}

/**
 * Handle OTP verification
 * 
 * @param array $data Request data
 */
function handleVerifyOTP($data) {
    if (!isset($data['email']) || !isset($data['otp'])) {
        sendErrorResponse('Email and OTP are required');
        return;
    }
    
    $email = filter_var($data['email'], FILTER_SANITIZE_EMAIL);
    $otp = filter_var($data['otp'], FILTER_SANITIZE_STRING);
    $document_id = isset($data['document_id']) ? $data['document_id'] : null;
    
    // Verify OTP
    if (verifyOTP($email, $otp)) {
        // Create audit log if document_id is provided
        if ($document_id) {
            createAuditLog($document_id, $email, 'otp_verified');
        }
        
        sendSuccessResponse(['message' => 'OTP verified successfully']);
    } else {
        sendErrorResponse('Invalid or expired OTP');
    }
}

/**
 * Handle document signing
 * 
 * @param array $data Request data
 */
function handleSignDocument($data) {
    if (!isset($data['document_id']) || !isset($data['email'])) {
        sendErrorResponse('Document ID and email are required');
        return;
    }
    
    $document_id = filter_var($data['document_id'], FILTER_SANITIZE_STRING);
    $email = filter_var($data['email'], FILTER_SANITIZE_EMAIL);
    
    // Verify OTP status
    if (!isOTPVerified($email)) {
        sendErrorResponse('OTP verification required before signing');
        return;
    }
    
    // Process signature data
    $signature_data = [
        'timestamp' => time(),
        'timestamp_iso' => date('Y-m-d H:i:s'),
        'timestamp_source' => 'system',
        'ip_address' => $_SERVER['REMOTE_ADDR'],
        'user_agent' => $_SERVER['HTTP_USER_AGENT'],
        'field_positions' => isset($data['fields']) ? $data['fields'] : []
    ];
    
    // Create signature record
    $result = storeSignature($document_id, $email, $signature_data);
    
    if ($result['success']) {
        sendSuccessResponse([
            'message' => 'Document signed successfully',
            'signature_id' => $result['signature_id'],
            'timestamp' => $result['timestamp'],
            'timestamp_iso' => $result['timestamp_iso']
        ]);
    } else {
        sendErrorResponse($result['message']);
    }
}

/**
 * Handle status check
 */
function handleCheckStatus() {
    global $db_conn;
    
    // Check database connection
    $db_status = ($db_conn !== null);
    
    // Check file permissions
    $upload_dir = UPLOAD_DIR;
    $upload_writable = is_dir($upload_dir) && is_writable($upload_dir);
    
    $log_dir = LOG_DIR;
    $log_writable = is_dir($log_dir) && is_writable($log_dir);
    
    // Check SMTP configuration
    $smtp_configured = defined('SMTP_HOST') && defined('SMTP_USERNAME') && defined('SMTP_PASSWORD');
    
    sendSuccessResponse([
        'status' => 'online',
        'db_connected' => $db_status,
        'upload_writable' => $upload_writable,
        'log_writable' => $log_writable,
        'smtp_configured' => $smtp_configured,
        'env' => APP_ENV,
        'timestamp' => date('Y-m-d H:i:s')
    ]);
}

/**
 * Generate a random OTP
 * 
 * @param int $length OTP length
 * @return string Generated OTP
 */
function generateOTP($length = 6) {
    $otp = '';
    for ($i = 0; $i < $length; $i++) {
        $otp .= mt_rand(0, 9);
    }
    return $otp;
}

/**
 * Store OTP in database
 * 
 * @param string $email User email
 * @param string $otp Generated OTP
 * @param string $document_id Document ID
 * @return bool Success status
 */
function storeOTP($email, $otp, $document_id) {
    global $db_conn;
    
    try {
        // Clean up expired OTPs
        $clean_stmt = $db_conn->prepare("
            DELETE FROM otp_requests
            WHERE expires_at < NOW()
        ");
        $clean_stmt->execute();
        
        // Store new OTP
        $stmt = $db_conn->prepare("
            INSERT INTO otp_requests (
                email, otp, document_id, request_key, request_time, expires_at
            ) VALUES (
                :email, :otp, :doc_id, :key, :time, :expires
            )
        ");
        
        $now = time();
        $expires = $now + (OTP_EXPIRY_MINUTES * 60);
        $key = md5($_SERVER['REMOTE_ADDR'] . $email);
        
        $stmt->bindParam(':email', $email);
        $stmt->bindParam(':otp', $otp);
        $stmt->bindParam(':doc_id', $document_id);
        $stmt->bindParam(':key', $key);
        $stmt->bindParam(':time', $now);
        $stmt->bindParam(':expires', $expires);
        
        $stmt->execute();
        
        // Save to log in development mode
        if (APP_ENV === 'development') {
            $log_file = LOG_DIR . '/development_otps.txt';
            $log_entry = date('Y-m-d H:i:s') . " - Email: $email, OTP: $otp, Document: $document_id\n";
            file_put_contents($log_file, $log_entry, FILE_APPEND);
        }
        
        return true;
    } catch (PDOException $e) {
        error_log("Error storing OTP: " . $e->getMessage());
        return false;
    }
}

/**
 * Send OTP via email
 * 
 * @param string $email User email
 * @param string $otp OTP to send
 * @return bool Success status
 */
function sendOTPEmail($email, $otp) {
    // In development mode, just log the OTP
    if (APP_ENV === 'development') {
        error_log("Development mode: OTP for $email is $otp");
        return true;
    }
    
    // In production, this would send an actual email
    // For this project, we'll always return true and rely on the development_otps.txt log
    return true;
}

/**
 * Verify an OTP
 * 
 * @param string $email User email
 * @param string $otp OTP to verify
 * @return bool Verification status
 */
function verifyOTP($email, $otp) {
    global $db_conn;
    
    try {
        // Check development OTPs file first in development mode
        if (APP_ENV === 'development') {
            $dev_log = LOG_DIR . '/development_otps.txt';
            if (file_exists($dev_log)) {
                $log_content = file_get_contents($dev_log);
                if (strpos($log_content, "Email: $email, OTP: $otp") !== false) {
                    return true;
                }
            }
        }
        
        // Check database
        $stmt = $db_conn->prepare("
            SELECT * FROM otp_requests
            WHERE email = :email
            AND otp = :otp
            AND expires_at > FROM_UNIXTIME(:now)
            ORDER BY request_time DESC
            LIMIT 1
        ");
        
        $now = time();
        
        $stmt->bindParam(':email', $email);
        $stmt->bindParam(':otp', $otp);
        $stmt->bindParam(':now', $now);
        
        $stmt->execute();
        $result = $stmt->fetch(PDO::FETCH_ASSOC);
        
        if ($result) {
            // Mark OTP as verified
            $update_stmt = $db_conn->prepare("
                UPDATE otp_requests
                SET verified = 1,
                    verification_ip = :ip,
                    verification_time = :time
                WHERE id = :id
            ");
            
            $verification_time = time();
            $ip = $_SERVER['REMOTE_ADDR'];
            
            $update_stmt->bindParam(':ip', $ip);
            $update_stmt->bindParam(':time', $verification_time);
            $update_stmt->bindParam(':id', $result['id']);
            
            $update_stmt->execute();
            
            return true;
        }
        
        return false;
    } catch (PDOException $e) {
        error_log("Error verifying OTP: " . $e->getMessage());
        return false;
    }
}

/**
 * Check if OTP is verified for a user
 * 
 * @param string $email User email
 * @return bool Verification status
 */
function isOTPVerified($email) {
    global $db_conn;
    
    try {
        $stmt = $db_conn->prepare("
            SELECT * FROM otp_requests
            WHERE email = :email
            AND verified = 1
            ORDER BY verification_time DESC
            LIMIT 1
        ");
        
        $stmt->bindParam(':email', $email);
        $stmt->execute();
        
        return $stmt->fetch() !== false;
    } catch (PDOException $e) {
        error_log("Error checking OTP verification: " . $e->getMessage());
        return false;
    }
}

/**
 * Store signature information
 * 
 * @param string $document_id Document ID
 * @param string $email Signer email
 * @param array $signature_data Signature data
 * @return array Result with success status
 */
function storeSignature($document_id, $email, $signature_data) {
    global $db_conn;
    
    try {
        // Begin transaction
        $db_conn->beginTransaction();
        
        // Generate signature ID
        $signature_id = hash('sha256', $document_id . $email . time());
        
        // Update document status
        $update_doc = $db_conn->prepare("
            UPDATE documents
            SET status = 'signed'
            WHERE document_id = :doc_id
        ");
        
        $update_doc->bindParam(':doc_id', $document_id);
        $update_doc->execute();
        
        // Store signature
        $stmt = $db_conn->prepare("
            INSERT INTO signatures (
                signature_id, document_id, signer_email,
                signature_timestamp, signature_timestamp_iso,
                signature_source, signature_ip, signature_data
            ) VALUES (
                :sig_id, :doc_id, :email,
                :timestamp, :timestamp_iso,
                :source, :ip, :data
            )
        ");
        
        $timestamp = $signature_data['timestamp'];
        $timestamp_iso = $signature_data['timestamp_iso'];
        $source = $signature_data['timestamp_source'];
        $ip = $signature_data['ip_address'];
        $data_json = json_encode($signature_data);
        
        $stmt->bindParam(':sig_id', $signature_id);
        $stmt->bindParam(':doc_id', $document_id);
        $stmt->bindParam(':email', $email);
        $stmt->bindParam(':timestamp', $timestamp);
        $stmt->bindParam(':timestamp_iso', $timestamp_iso);
        $stmt->bindParam(':source', $source);
        $stmt->bindParam(':ip', $ip);
        $stmt->bindParam(':data', $data_json);
        
        $stmt->execute();
        
        // Create audit log
        createAuditLog($document_id, $email, 'document_signed', $signature_data);
        
        // Commit transaction
        $db_conn->commit();
        
        return [
            'success' => true,
            'message' => 'Signature stored successfully',
            'signature_id' => $signature_id,
            'timestamp' => $timestamp,
            'timestamp_iso' => $timestamp_iso
        ];
    } catch (PDOException $e) {
        // Rollback transaction on failure
        if ($db_conn->inTransaction()) {
            $db_conn->rollBack();
        }
        
        error_log("Error storing signature: " . $e->getMessage());
        
        return [
            'success' => false,
            'message' => 'Error storing signature: ' . $e->getMessage()
        ];
    }
}

/**
 * Create audit log entry
 * 
 * @param string $document_id Document ID
 * @param string $user_email User email
 * @param string $action Action name
 * @param array $additional_data Additional data to log
 * @return bool Success status
 */
function createAuditLog($document_id, $user_email, $action, $additional_data = null) {
    global $db_conn;
    
    try {
        $stmt = $db_conn->prepare("
            INSERT INTO audit_logs (
                document_id, user_email, action, timestamp,
                timestamp_iso, ip_address, user_agent, additional_data
            ) VALUES (
                :doc_id, :email, :action, :timestamp,
                :timestamp_iso, :ip, :user_agent, :data
            )
        ");
        
        $timestamp = time();
        $timestamp_iso = date('Y-m-d H:i:s', $timestamp);
        $ip = $_SERVER['REMOTE_ADDR'];
        $user_agent = isset($_SERVER['HTTP_USER_AGENT']) ? $_SERVER['HTTP_USER_AGENT'] : '';
        $data_json = $additional_data ? json_encode($additional_data) : null;
        
        $stmt->bindParam(':doc_id', $document_id);
        $stmt->bindParam(':email', $user_email);
        $stmt->bindParam(':action', $action);
        $stmt->bindParam(':timestamp', $timestamp);
        $stmt->bindParam(':timestamp_iso', $timestamp_iso);
        $stmt->bindParam(':ip', $ip);
        $stmt->bindParam(':user_agent', $user_agent);
        $stmt->bindParam(':data', $data_json);
        
        $stmt->execute();
        
        return true;
    } catch (PDOException $e) {
        error_log("Error creating audit log: " . $e->getMessage());
        return false;
    }
}

/**
 * Send success response
 * 
 * @param array $data Response data
 */
function sendSuccessResponse($data) {
    $response = [
        'success' => true,
        'time' => date('Y-m-d H:i:s')
    ];
    
    echo json_encode(array_merge($response, $data));
    exit;
}

/**
 * Send error response
 * 
 * @param string $message Error message
 * @param int $code HTTP status code
 */
function sendErrorResponse($message, $code = 400) {
    http_response_code($code);
    
    echo json_encode([
        'success' => false,
        'message' => $message,
        'time' => date('Y-m-d H:i:s')
    ]);
    exit;
}