<?php
/**
 * Application Configuration
 * Centralizes all configuration settings
 */

// Error reporting
error_reporting(E_ALL);
ini_set('display_errors', 0); // Disable in production
ini_set('log_errors', 1);
ini_set('error_log', __DIR__ . '/logs/error.log');

// File upload settings
ini_set('upload_max_filesize', '10M');
ini_set('post_max_size', '10M');
ini_set('max_execution_time', 300); // 5 minutes
ini_set('memory_limit', '256M');

// Security settings
ini_set('session.cookie_httponly', 1);
ini_set('session.use_only_cookies', 1);
ini_set('session.cookie_secure', 1);
ini_set('session.cookie_samesite', 'Strict');

// Check environment - detect development env automatically
$is_dev = (isset($_SERVER['SERVER_NAME']) && (
    strpos($_SERVER['SERVER_NAME'], 'localhost') !== false ||
    strpos($_SERVER['SERVER_NAME'], '127.0.0.1') !== false ||
    strpos($_SERVER['SERVER_NAME'], '.test') !== false ||
    strpos($_SERVER['SERVER_NAME'], '.local') !== false ||
    strpos($_SERVER['SERVER_NAME'], 'server345.web-hosting.com') !== false
));

// Application constants
define('APP_NAME', 'Inkless-Flow');
define('APP_VERSION', '3.0.0');
define('APP_ENV', $is_dev ? 'development' : 'production');

// Show errors in development
if (APP_ENV === 'development') {
    ini_set('display_errors', 1);
}

// File paths
define('UPLOAD_DIR', __DIR__ . '/uploads');
define('TEMP_DIR', __DIR__ . '/temp');
define('LOG_DIR', __DIR__ . '/logs');

// Create necessary directories with proper permissions
$directories = [UPLOAD_DIR, TEMP_DIR, LOG_DIR];
foreach ($directories as $dir) {
    if (!is_dir($dir)) {
        mkdir($dir, 0755, true);
    } else {
        // Ensure proper permissions on existing directories
        chmod($dir, 0755);
    }
}

// Security headers
header('X-Content-Type-Options: nosniff');
header('X-Frame-Options: DENY');
header('X-XSS-Protection: 1; mode=block');
header('Strict-Transport-Security: max-age=31536000; includeSubDomains');
header('Content-Security-Policy: default-src \'self\'; script-src \'self\' \'unsafe-inline\' \'unsafe-eval\' https://cdnjs.cloudflare.com; style-src \'self\' \'unsafe-inline\' https://cdnjs.cloudflare.com; font-src \'self\' https://cdnjs.cloudflare.com; img-src \'self\' data:;');

// Database settings - Use environment variables if available
// Fixed: Removed port from hostname to avoid connection issues
define('DB_HOST', getenv('DB_HOST') ? getenv('DB_HOST') : 'localhost');
define('DB_NAME', getenv('DB_NAME') ? getenv('DB_NAME') : 'inklssbz_inkless-flow');
define('DB_USER', getenv('DB_USER') ? getenv('DB_USER') : 'inklssbz_inkless-flow');
define('DB_PASS', getenv('DB_PASS') ? getenv('DB_PASS') : '+3yjtNs[$J=X');

// OTP settings
define('OTP_LENGTH', 6);
define('OTP_EXPIRY_MINUTES', 5);
define('OTP_RATE_LIMIT', 5); // Max requests per hour
define('OTP_RATE_WINDOW', 3600); // 1 hour in seconds

// Signature settings
define('SIGNATURE_KEY_NAME', 'signature_hmac_key');
define('TRUSTED_TIMESTAMP_SOURCE', 'worldtimeapi.org');
define('TRUSTED_TIMESTAMP_URL', 'https://worldtimeapi.org/api/timezone/UTC'); // Changed to HTTPS

// Session settings
ini_set('session.gc_maxlifetime', 3600); // 1 hour
ini_set('session.cookie_lifetime', 3600); // 1 hour

// Email settings for cPanel hosted email
define('SMTP_HOST', 'mail.inkless-flow.com');  // Your mail server hostname
define('SMTP_PORT', 587);                      // Change to 465 if using SSL instead of TLS
define('SMTP_USERNAME', 'otp@inkless-flow.com');
define('SMTP_PASSWORD', 'zXqf8R#E$54x$s8%5&L#'); // Your provided password
define('SMTP_FROM_EMAIL', 'otp@inkless-flow.com');
define('SMTP_FROM_NAME', 'Inkless-Flow OTP Service');
define('SMTP_SECURE', 'tls');                  // Change to 'ssl' if using port 465

// Development mode settings - Only apply if explicitly set to development
if (APP_ENV === 'development') {
    ini_set('display_errors', 1);
    error_reporting(E_ALL);
    header('X-Robots-Tag: noindex, nofollow');
    
    // Development email settings (optional)
    define('DEV_EMAIL', 'otp@inkless-flow.com');
}

// Function to get client IP address
function getClientIP() {
    $ipaddress = '';
    if (isset($_SERVER['HTTP_CLIENT_IP']))
        $ipaddress = $_SERVER['HTTP_CLIENT_IP'];
    else if(isset($_SERVER['HTTP_X_FORWARDED_FOR']))
        $ipaddress = $_SERVER['HTTP_X_FORWARDED_FOR'];
    else if(isset($_SERVER['HTTP_X_FORWARDED']))
        $ipaddress = $_SERVER['HTTP_X_FORWARDED'];
    else if(isset($_SERVER['HTTP_FORWARDED_FOR']))
        $ipaddress = $_SERVER['HTTP_FORWARDED_FOR'];
    else if(isset($_SERVER['HTTP_FORWARDED']))
        $ipaddress = $_SERVER['HTTP_FORWARDED'];
    else if(isset($_SERVER['REMOTE_ADDR']))
        $ipaddress = $_SERVER['REMOTE_ADDR'];
    else
        $ipaddress = 'UNKNOWN';
    return $ipaddress;
}

// Function to sanitize input
function sanitizeInput($data) {
    $data = trim($data);
    $data = stripslashes($data);
    $data = htmlspecialchars($data);
    return $data;
}

// Function to validate email
function validateEmail($email) {
    return filter_var($email, FILTER_VALIDATE_EMAIL);
}

// Function to generate CSRF token
function generateCSRFToken() {
    if (empty($_SESSION['csrf_token'])) {
        $_SESSION['csrf_token'] = bin2hex(random_bytes(32));
    }
    return $_SESSION['csrf_token'];
}

// Function to validate CSRF token
function validateCSRFToken($token) {
    if (!isset($_SESSION['csrf_token']) || $token !== $_SESSION['csrf_token']) {
        return false;
    }
    return true;
}

// Function to log activity
function logActivity($action, $details = null) {
    $log_file = LOG_DIR . '/activity.log';
    $timestamp = date('Y-m-d H:i:s');
    $ip = getClientIP();
    $user_agent = isset($_SERVER['HTTP_USER_AGENT']) ? $_SERVER['HTTP_USER_AGENT'] : 'Unknown';
    
    $log_entry = sprintf(
        "[%s] %s - IP: %s - User Agent: %s%s\n",
        $timestamp,
        $action,
        $ip,
        $user_agent,
        $details ? " - Details: " . json_encode($details) : ""
    );
    
    file_put_contents($log_file, $log_entry, FILE_APPEND);
}