-- Create database (if not already created)
-- CREATE DATABASE IF NOT EXISTS inklssbz_inklessflow;
-- USE inklssbz_inklessflow;

-- Table for storing documents
CREATE TABLE IF NOT EXISTS documents (
    document_id VARCHAR(64) PRIMARY KEY,  -- SHA-256 hash of the document serves as the ID
    file_name VARCHAR(255) NOT NULL,
    upload_timestamp BIGINT NOT NULL,
    upload_timestamp_iso VARCHAR(50) NOT NULL,
    original_file_size BIGINT NOT NULL,
    mime_type VARCHAR(100) NOT NULL,
    status ENUM('uploaded', 'signed', 'deleted') DEFAULT 'uploaded',
    uploaded_by VARCHAR(255) NOT NULL,   -- Email address of uploader
    created_at TIMESTAMP DEFAULT CURRENT_TIMESTAMP,
    INDEX idx_status (status),
    INDEX idx_uploaded_by (uploaded_by),
    INDEX idx_created_at (created_at)
);

-- Table for storing OTP verification data
CREATE TABLE IF NOT EXISTS otp_verification (
    id INT AUTO_INCREMENT PRIMARY KEY,
    email VARCHAR(255) NOT NULL,
    otp VARCHAR(8) NOT NULL,  -- 6-digit OTP plus potential padding
    document_id VARCHAR(64) NULL,  -- Document being verified for
    created_at TIMESTAMP DEFAULT CURRENT_TIMESTAMP,
    expires_at TIMESTAMP NOT NULL,
    verified BOOLEAN DEFAULT FALSE,
    verification_ip VARCHAR(45) NULL,
    verification_timestamp BIGINT NULL,
    INDEX idx_email_created (email, created_at),
    INDEX idx_otp (otp),
    INDEX idx_expires_at (expires_at),
    INDEX idx_verified (verified),
    FOREIGN KEY (document_id) REFERENCES documents(document_id) ON DELETE CASCADE
);

-- Table for storing signatures
CREATE TABLE IF NOT EXISTS signatures (
    signature_id VARCHAR(128) PRIMARY KEY,  -- HMAC-SHA256 signature serves as ID
    document_id VARCHAR(64) NOT NULL,
    signer_email VARCHAR(255) NOT NULL,
    signature_timestamp BIGINT NOT NULL,
    signature_timestamp_iso VARCHAR(50) NOT NULL,
    signature_source VARCHAR(100) NOT NULL,  -- Source of the timestamp (e.g., 'worldtimeapi.org')
    signature_ip VARCHAR(45) NOT NULL,
    signature_data TEXT NOT NULL,  -- JSON with field positions and metadata
    created_at TIMESTAMP DEFAULT CURRENT_TIMESTAMP,
    INDEX idx_document_id (document_id),
    INDEX idx_signer_email (signer_email),
    INDEX idx_signature_timestamp (signature_timestamp),
    FOREIGN KEY (document_id) REFERENCES documents(document_id) ON DELETE CASCADE
);

-- Table for storing audit logs
CREATE TABLE IF NOT EXISTS audit_logs (
    log_id INT AUTO_INCREMENT PRIMARY KEY,
    document_id VARCHAR(64) NOT NULL,
    user_email VARCHAR(255) NOT NULL,
    action ENUM('document_uploaded', 'document_viewed', 'otp_requested', 'otp_verified', 'document_signed', 'document_downloaded', 'document_deleted') NOT NULL,
    timestamp BIGINT NOT NULL,
    timestamp_iso VARCHAR(50) NOT NULL,
    ip_address VARCHAR(45) NOT NULL,
    user_agent TEXT NULL,
    additional_data TEXT NULL,  -- JSON with any additional data
    created_at TIMESTAMP DEFAULT CURRENT_TIMESTAMP,
    INDEX idx_document_id (document_id),
    INDEX idx_user_email (user_email),
    INDEX idx_action (action),
    INDEX idx_timestamp (timestamp),
    INDEX idx_created_at (created_at),
    FOREIGN KEY (document_id) REFERENCES documents(document_id) ON DELETE CASCADE
);

-- Table for storing user consent for GDPR compliance
CREATE TABLE IF NOT EXISTS user_consents (
    id INT AUTO_INCREMENT PRIMARY KEY,
    user_email VARCHAR(255) NOT NULL,
    consent_type ENUM('data_processing', 'terms_of_service', 'signature_agreement') NOT NULL,
    consent_timestamp BIGINT NOT NULL,
    consent_ip VARCHAR(45) NOT NULL,
    consent_text TEXT NOT NULL,  -- The text the user agreed to
    revoked BOOLEAN DEFAULT FALSE,
    revoked_timestamp BIGINT NULL,
    created_at TIMESTAMP DEFAULT CURRENT_TIMESTAMP,
    INDEX idx_user_email (user_email),
    INDEX idx_consent_type (consent_type),
    INDEX idx_consent_timestamp (consent_timestamp),
    INDEX idx_revoked (revoked)
);

-- Table for storing encryption keys (securely)
CREATE TABLE IF NOT EXISTS encryption_keys (
    id INT AUTO_INCREMENT PRIMARY KEY,
    key_name VARCHAR(100) NOT NULL,
    key_value TEXT NOT NULL,  -- Encrypted key value
    active BOOLEAN DEFAULT TRUE,
    created_at TIMESTAMP DEFAULT CURRENT_TIMESTAMP,
    UNIQUE (key_name),
    INDEX idx_active (active)
);

-- Insert initial encryption key (would be done securely in production)
INSERT INTO encryption_keys (key_name, key_value) 
VALUES ('signature_hmac_key', SHA2(CONCAT(UUID(), RAND(), NOW()), 512))
ON DUPLICATE KEY UPDATE key_value = key_value;

-- Add triggers for automatic timestamp updates
DELIMITER //

CREATE TRIGGER before_document_update
BEFORE UPDATE ON documents
FOR EACH ROW
BEGIN
    SET NEW.updated_at = CURRENT_TIMESTAMP;
END//

CREATE TRIGGER before_signature_update
BEFORE UPDATE ON signatures
FOR EACH ROW
BEGIN
    SET NEW.updated_at = CURRENT_TIMESTAMP;
END//

DELIMITER ;