<?php
/**
 * Database Configuration
 * Handles database connection and error handling
 */

// Include configuration file
require_once 'config.php';

try {
    // Create PDO connection
    $db_conn = new PDO(
        "mysql:host=" . DB_HOST . ";dbname=" . DB_NAME . ";charset=utf8mb4",
        DB_USER,
        DB_PASS,
        [
            PDO::ATTR_ERRMODE => PDO::ERRMODE_EXCEPTION,
            PDO::ATTR_DEFAULT_FETCH_MODE => PDO::FETCH_ASSOC,
            PDO::ATTR_EMULATE_PREPARES => false,
            PDO::MYSQL_ATTR_INIT_COMMAND => "SET NAMES utf8mb4 COLLATE utf8mb4_unicode_ci"
        ]
    );
    
    // Set session parameters
    session_set_cookie_params(
        3600, // lifetime
        '/', // path
        '', // domain
        true, // secure
        true  // httponly
    );
    
    // Start session if not already started
    if (session_status() === PHP_SESSION_NONE) {
        session_start();
    }
    
} catch (PDOException $e) {
    // Log the error
    error_log("Database connection error: " . $e->getMessage());
    
    // In production, show generic error
    if (APP_ENV === 'production') {
        die("A database error occurred. Please try again later.");
    } else {
        // In development, show detailed error
        die("Database connection failed: " . $e->getMessage());
    }
}

/**
 * Function to safely execute database queries
 * @param string $query SQL query
 * @param array $params Query parameters
 * @return PDOStatement|false
 */
function executeQuery($query, $params = []) {
    global $db_conn;
    
    try {
        $stmt = $db_conn->prepare($query);
        $stmt->execute($params);
        return $stmt;
    } catch (PDOException $e) {
        error_log("Query execution error: " . $e->getMessage());
        return false;
    }
}

/**
 * Function to begin a database transaction
 * @return bool
 */
function beginTransaction() {
    global $db_conn;
    
    try {
        return $db_conn->beginTransaction();
    } catch (PDOException $e) {
        error_log("Transaction start error: " . $e->getMessage());
        return false;
    }
}

/**
 * Function to commit a database transaction
 * @return bool
 */
function commitTransaction() {
    global $db_conn;
    
    try {
        return $db_conn->commit();
    } catch (PDOException $e) {
        error_log("Transaction commit error: " . $e->getMessage());
        return false;
    }
}

/**
 * Function to rollback a database transaction
 * @return bool
 */
function rollbackTransaction() {
    global $db_conn;
    
    try {
        return $db_conn->rollBack();
    } catch (PDOException $e) {
        error_log("Transaction rollback error: " . $e->getMessage());
        return false;
    }
}

/**
 * Function to get last inserted ID
 * @return string
 */
function getLastInsertId() {
    global $db_conn;
    return $db_conn->lastInsertId();
}

/**
 * Function to escape string for safe database use
 * @param string $string String to escape
 * @return string
 */
function dbEscape($string) {
    global $db_conn;
    return substr($db_conn->quote($string), 1, -1);
}