<?php
/**
 * Document Storage and eIDAS Compliance Handler
 * Ensures proper storage and tracking of signed documents
 */

/**
 * Store signed document with eIDAS compliance
 * 
 * @param string $document_id Document hash identifier
 * @param string $user_email Signer's email
 * @param array $signature_data Signature data including timestamps
 * @param string $signed_file_content Base64 encoded signed document content
 * @return array Result with success status and message
 */
function storeSignedDocument($document_id, $user_email, $signature_data, $signed_file_content) {
    global $db_conn;
    
    try {
        // Begin transaction to ensure data consistency
        $db_conn->beginTransaction();
        
        // 1. Update document status in the documents table
        $update_doc = $db_conn->prepare("
            UPDATE documents 
            SET status = 'signed' 
            WHERE document_id = :doc_id
        ");
        $update_doc->bindParam(':doc_id', $document_id);
        $update_doc->execute();
        
        // 2. Store the signature information
        $stmt = $db_conn->prepare("
            INSERT INTO signatures (
                signature_id, document_id, signer_email, 
                signature_timestamp, signature_timestamp_iso,
                signature_source, signature_ip, signature_data
            ) VALUES (
                :sig_id, :doc_id, :email,
                :timestamp, :timestamp_iso,
                :source, :ip, :data
            )
        ");
        
        $signature_id = hash('sha256', $document_id . $user_email . time());
        $timestamp = $signature_data['timestamp'] ?? time();
        $timestamp_iso = $signature_data['timestamp_iso'] ?? date('Y-m-d H:i:s', $timestamp);
        $source = $signature_data['timestamp_source'] ?? 'system';
        $ip = $signature_data['ip_address'] ?? $_SERVER['REMOTE_ADDR'];
        $data_json = json_encode($signature_data);
        
        $stmt->bindParam(':sig_id', $signature_id);
        $stmt->bindParam(':doc_id', $document_id);
        $stmt->bindParam(':email', $user_email);
        $stmt->bindParam(':timestamp', $timestamp);
        $stmt->bindParam(':timestamp_iso', $timestamp_iso);
        $stmt->bindParam(':source', $source);
        $stmt->bindParam(':ip', $ip);
        $stmt->bindParam(':data', $data_json);
        $stmt->execute();
        
        // 3. Store the signed document in the filesystem
        $upload_dir = UPLOAD_DIR . '/signed';
        if (!is_dir($upload_dir)) {
            mkdir($upload_dir, 0755, true);
        }
        
        // Store with document ID and signature ID to ensure uniqueness
        $signed_filename = $document_id . '_' . substr($signature_id, 0, 8) . '.pdf';
        $signed_path = $upload_dir . '/' . $signed_filename;
        
        // Decode and save the file
        $decoded_content = base64_decode(str_replace('data:application/pdf;base64,', '', $signed_file_content));
        file_put_contents($signed_path, $decoded_content);
        
        // 4. Create audit log entry for signing
        $audit_log = $db_conn->prepare("
            INSERT INTO audit_logs (
                document_id, user_email, action, timestamp,
                timestamp_iso, ip_address, user_agent, additional_data
            ) VALUES (
                :doc_id, :email, 'document_signed', :timestamp,
                :timestamp_iso, :ip, :user_agent, :additional_data
            )
        ");
        
        $user_agent = $_SERVER['HTTP_USER_AGENT'] ?? null;
        $additional_data = json_encode([
            'signature_id' => $signature_id,
            'signed_file' => $signed_filename,
            'field_positions' => $signature_data['field_positions'] ?? null
        ]);
        
        $audit_log->bindParam(':doc_id', $document_id);
        $audit_log->bindParam(':email', $user_email);
        $audit_log->bindParam(':timestamp', $timestamp);
        $audit_log->bindParam(':timestamp_iso', $timestamp_iso);
        $audit_log->bindParam(':ip', $ip);
        $audit_log->bindParam(':user_agent', $user_agent);
        $audit_log->bindParam(':additional_data', $additional_data);
        $audit_log->execute();
        
        // Commit the transaction
        $db_conn->commit();
        
        return [
            'success' => true,
            'message' => 'Document signed and stored successfully',
            'document_id' => $document_id,
            'signature_id' => $signature_id,
            'timestamp' => $timestamp,
            'timestamp_iso' => $timestamp_iso,
            'signed_file' => $signed_filename
        ];
    } catch (Exception $e) {
        // Rollback the transaction on failure
        if ($db_conn->inTransaction()) {
            $db_conn->rollBack();
        }
        
        error_log("Error storing signed document: " . $e->getMessage());
        
        return [
            'success' => false,
            'message' => 'Error storing signed document: ' . $e->getMessage()
        ];
    }
}

/**
 * Retrieve signed document with verification data
 * 
 * @param string $document_id Document hash identifier
 * @param string $signature_id Signature identifier (optional)
 * @return array Document data with verification info
 */
function getSignedDocument($document_id, $signature_id = null) {
    global $db_conn;
    
    try {
        // Build the query based on whether signature_id is provided
        $query = "
            SELECT d.document_id, d.file_name, d.upload_timestamp, d.upload_timestamp_iso,
                   d.status, d.uploaded_by, s.signature_id, s.signer_email,
                   s.signature_timestamp, s.signature_timestamp_iso, 
                   s.signature_source, s.signature_data
            FROM documents d
            LEFT JOIN signatures s ON d.document_id = s.document_id
            WHERE d.document_id = :doc_id
        ";
        
        if ($signature_id) {
            $query .= " AND s.signature_id = :sig_id";
        }
        
        $query .= " LIMIT 1";
        
        $stmt = $db_conn->prepare($query);
        $stmt->bindParam(':doc_id', $document_id);
        
        if ($signature_id) {
            $stmt->bindParam(':sig_id', $signature_id);
        }
        
        $stmt->execute();
        $result = $stmt->fetch(PDO::FETCH_ASSOC);
        
        if (!$result) {
            return [
                'success' => false,
                'message' => 'Document not found or not signed'
            ];
        }
        
        // Get the file path
        $signed_dir = UPLOAD_DIR . '/signed';
        $original_dir = UPLOAD_DIR;
        
        // Check if signed version exists
        $signed_pattern = $signed_dir . '/' . $document_id . '_*.pdf';
        $signed_files = glob($signed_pattern);
        
        $file_path = '';
        $is_signed = false;
        
        if (!empty($signed_files) && $result['status'] === 'signed') {
            $file_path = $signed_files[0]; // Use the first signed version
            $is_signed = true;
        } else {
            // Fall back to original file
            $file_path = $original_dir . '/' . $document_id . '.pdf';
            $is_signed = false;
        }
        
        // Create audit log for document retrieval
        $audit_log = $db_conn->prepare("
            INSERT INTO audit_logs (
                document_id, user_email, action, timestamp,
                timestamp_iso, ip_address, user_agent
            ) VALUES (
                :doc_id, :email, 'document_viewed', :timestamp,
                :timestamp_iso, :ip, :user_agent
            )
        ");
        
        $timestamp = time();
        $timestamp_iso = date('Y-m-d H:i:s', $timestamp);
        $ip = $_SERVER['REMOTE_ADDR'];
        $user_agent = $_SERVER['HTTP_USER_AGENT'] ?? null;
        $email = $result['signer_email'] ?? $result['uploaded_by'];
        
        $audit_log->bindParam(':doc_id', $document_id);
        $audit_log->bindParam(':email', $email);
        $audit_log->bindParam(':timestamp', $timestamp);
        $audit_log->bindParam(':timestamp_iso', $timestamp_iso);
        $audit_log->bindParam(':ip', $ip);
        $audit_log->bindParam(':user_agent', $user_agent);
        $audit_log->execute();
        
        return [
            'success' => true,
            'document_id' => $document_id,
            'file_name' => $result['file_name'],
            'is_signed' => $is_signed,
            'status' => $result['status'],
            'uploaded_by' => $result['uploaded_by'],
            'upload_time' => $result['upload_timestamp_iso'],
            'signature_id' => $result['signature_id'] ?? null,
            'signer_email' => $result['signer_email'] ?? null,
            'signature_time' => $result['signature_timestamp_iso'] ?? null,
            'signature_source' => $result['signature_source'] ?? null,
            'file_path' => $file_path,
            'view_timestamp' => $timestamp_iso
        ];
    } catch (Exception $e) {
        error_log("Error retrieving signed document: " . $e->getMessage());
        
        return [
            'success' => false,
            'message' => 'Error retrieving document: ' . $e->getMessage()
        ];
    }
}

/**
 * Verify document integrity against stored hash
 * 
 * @param string $file_path Path to the file to verify
 * @param string $stored_hash Previously stored hash
 * @return boolean True if document is valid
 */
function verifyDocumentIntegrity($file_path, $stored_hash) {
    // Calculate hash of current file
    $current_hash = hash_file('sha256', $file_path);
    
    // Compare with stored hash
    return $current_hash === $stored_hash;
}

/**
 * Generate eIDAS-compliant verification report
 * 
 * @param string $document_id Document hash identifier
 * @return array Report data
 */
function generateVerificationReport($document_id) {
    global $db_conn;
    
    try {
        // Get document and signature data
        $document = getSignedDocument($document_id);
        
        if (!$document['success']) {
            return $document; // Return error
        }
        
        // Get audit trail
        $audit_stmt = $db_conn->prepare("
            SELECT *
            FROM audit_logs
            WHERE document_id = :doc_id
            ORDER BY timestamp ASC
        ");
        
        $audit_stmt->bindParam(':doc_id', $document_id);
        $audit_stmt->execute();
        $audit_trail = $audit_stmt->fetchAll(PDO::FETCH_ASSOC);
        
        // Check document integrity
        $integrity_verified = false;
        if (file_exists($document['file_path'])) {
            $integrity_verified = verifyDocumentIntegrity($document['file_path'], $document_id);
        }
        
        // Build verification report
        $report = [
            'success' => true,
            'document_id' => $document_id,
            'file_name' => $document['file_name'],
            'verification_time' => date('Y-m-d H:i:s'),
            'is_signed' => $document['is_signed'],
            'status' => $document['status'],
            'integrity_verified' => $integrity_verified,
            'uploaded_by' => $document['uploaded_by'],
            'upload_time' => $document['upload_time'],
            'signer_email' => $document['signer_email'],
            'signature_time' => $document['signature_time'],
            'signature_source' => $document['signature_source'],
            'audit_trail' => $audit_trail,
            'eidas_compliance_level' => 'SES', // Simple Electronic Signature
            'verification_url' => 'https://inkless-flow.com/verify?id=' . $document_id
        ];
        
        // Log verification in audit trail
        $audit_log = $db_conn->prepare("
            INSERT INTO audit_logs (
                document_id, user_email, action, timestamp,
                timestamp_iso, ip_address, user_agent, additional_data
            ) VALUES (
                :doc_id, :email, 'document_verified', :timestamp,
                :timestamp_iso, :ip, :user_agent, :additional_data
            )
        ");
        
        $timestamp = time();
        $timestamp_iso = date('Y-m-d H:i:s', $timestamp);
        $ip = $_SERVER['REMOTE_ADDR'];
        $user_agent = $_SERVER['HTTP_USER_AGENT'] ?? null;
        $email = $document['signer_email'] ?? $document['uploaded_by'];
        $additional_data = json_encode([
            'integrity_verified' => $integrity_verified,
            'verification_report_generated' => true
        ]);
        
        $audit_log->bindParam(':doc_id', $document_id);
        $audit_log->bindParam(':email', $email);
        $audit_log->bindParam(':timestamp', $timestamp);
        $audit_log->bindParam(':timestamp_iso', $timestamp_iso);
        $audit_log->bindParam(':ip', $ip);
        $audit_log->bindParam(':user_agent', $user_agent);
        $audit_log->bindParam(':additional_data', $additional_data);
        $audit_log->execute();
        
        return $report;
    } catch (Exception $e) {
        error_log("Error generating verification report: " . $e->getMessage());
        
        return [
            'success' => false,
            'message' => 'Error generating verification report: ' . $e->getMessage()
        ];
    }
}