<?php
/**
 * Document Download API
 * Securely serves signed documents with proper audit logging
 */

// Enable error reporting in development (disable in production)
error_reporting(E_ALL);
ini_set('display_errors', 1);

// Required files
require_once 'db-config.php';
require_once 'eidas-compliance.php';

// Validate request
if (!isset($_GET['document_id']) || !isset($_GET['email']) || !isset($_GET['signature'])) {
    http_response_code(400);
    header('Content-Type: text/html; charset=UTF-8');
    echo '<h1>Eroare</h1><p>Cerere invalidă. Lipsesc parametri necesari.</p>';
    exit;
}

$document_id = filter_var($_GET['document_id'], FILTER_SANITIZE_STRING);
$email = filter_var($_GET['email'], FILTER_SANITIZE_EMAIL);
$signature = filter_var($_GET['signature'], FILTER_SANITIZE_STRING);

// Initialize eIDAS compliance module
$eidas = new EidasCompliance();

try {
    // Verify document exists in database and is signed
    global $db_conn;
    
    $stmt = $db_conn->prepare("
        SELECT d.file_name, d.status, s.signature_id 
        FROM documents d
        LEFT JOIN signatures s ON d.document_id = s.document_id
        WHERE d.document_id = :doc_id
        LIMIT 1
    ");
    
    $stmt->bindParam(':doc_id', $document_id);
    $stmt->execute();
    
    $result = $stmt->fetch(PDO::FETCH_ASSOC);
    
    if (!$result) {
        http_response_code(404);
        header('Content-Type: text/html; charset=UTF-8');
        echo '<h1>Eroare</h1><p>Documentul solicitat nu a fost găsit.</p>';
        exit;
    }
    
    if ($result['status'] !== 'signed') {
        http_response_code(403);
        header('Content-Type: text/html; charset=UTF-8');
        echo '<h1>Eroare</h1><p>Documentul nu a fost semnat încă.</p>';
        exit;
    }
    
    if ($result['signature_id'] !== $signature) {
        http_response_code(403);
        header('Content-Type: text/html; charset=UTF-8');
        echo '<h1>Eroare</h1><p>Semnătura nu este validă pentru acest document.</p>';
        exit;
    }
    
    // Define file paths
    $signed_dir = __DIR__ . '/signed';
    $file_path = $signed_dir . '/' . $document_id . '_signed.pdf';
    
    if (!file_exists($file_path)) {
        http_response_code(404);
        header('Content-Type: text/html; charset=UTF-8');
        echo '<h1>Eroare</h1><p>Fișierul PDF semnat nu a fost găsit.</p>';
        exit;
    }
    
    // Create download filename
    $download_filename = str_replace(' ', '_', pathinfo($result['file_name'], PATHINFO_FILENAME)) . '_semnat.pdf';
    
    // Log the download
    $eidas->createAuditLog($document_id, $email, 'document_downloaded', null);
    
    // Send the file
    header('Content-Type: application/pdf');
    header('Content-Disposition: attachment; filename="' . $download_filename . '"');
    header('Content-Length: ' . filesize($file_path));
    header('Cache-Control: private, max-age=0, must-revalidate');
    header('Pragma: public');
    
    readfile($file_path);
    exit;
} catch (Exception $e) {
    http_response_code(500);
    header('Content-Type: text/html; charset=UTF-8');
    echo '<h1>Eroare</h1><p>A apărut o eroare la descărcarea documentului.</p>';
    error_log("Download error: " . $e->getMessage());
}