<?php
/**
 * eIDAS Compliance Class
 * PHP implementation for server-side operations
 */

class EidasCompliance {
    private $db_conn;
    private $signature_key;
    
    public function __construct() {
        global $db_conn;
        $this->db_conn = $db_conn;
        $this->signature_key = $this->getSignatureKey();
    }
    
    /**
     * Get signature key from database
     * @return string Signature key
     */
    private function getSignatureKey() {
        try {
            $stmt = $this->db_conn->prepare("
                SELECT key_value 
                FROM encryption_keys 
                WHERE key_name = 'signature_hmac_key' 
                AND active = 1
            ");
            $stmt->execute();
            $result = $stmt->fetch();
            
            if (!$result) {
                // Generate new key if none exists
                $key = bin2hex(random_bytes(32));
                $stmt = $this->db_conn->prepare("
                    INSERT INTO encryption_keys (key_name, key_value)
                    VALUES ('signature_hmac_key', :key)
                ");
                $stmt->bindParam(':key', $key);
                $stmt->execute();
                return $key;
            }
            
            return $result['key_value'];
        } catch (PDOException $e) {
            error_log("Error getting signature key: " . $e->getMessage());
            throw new Exception("Failed to get signature key");
        }
    }
    
    /**
     * Get trusted timestamp from external source
     * @return int Unix timestamp
     */
    public function getTrustedTimestamp() {
        try {
            // Use World Time API as a trusted source
            $response = file_get_contents('http://worldtimeapi.org/api/timezone/UTC');
            if ($response === false) {
                throw new Exception("Failed to get trusted timestamp");
            }
            
            $data = json_decode($response, true);
            if (!$data || !isset($data['unixtime'])) {
                throw new Exception("Invalid timestamp response");
            }
            
            return (int)$data['unixtime'];
        } catch (Exception $e) {
            error_log("Error getting trusted timestamp: " . $e->getMessage());
            // Fallback to server time if external source fails
            return time();
        }
    }
    
    /**
     * Generate document hash
     * @param string $file_path Path to file
     * @return string SHA-256 hash
     */
    public function generateDocumentHash($file_path) {
        return hash_file('sha256', $file_path);
    }
    
    /**
     * Store document in database
     * @param string $document_id Document hash ID
     * @param string $file_name Original filename
     * @param int $file_size File size in bytes
     * @param string $mime_type File MIME type
     * @param string $user_email User's email
     * @return bool Success status
     */
    public function storeDocument($document_id, $file_name, $file_size, $mime_type, $user_email) {
        try {
            $stmt = $this->db_conn->prepare("
                INSERT INTO documents (
                    document_id, file_name, original_file_size, 
                    mime_type, upload_timestamp, uploaded_by, status
                ) VALUES (
                    :doc_id, :file_name, :file_size, 
                    :mime_type, :timestamp, :email, 'uploaded'
                )
            ");
            
            $timestamp = time();
            
            $stmt->bindParam(':doc_id', $document_id);
            $stmt->bindParam(':file_name', $file_name);
            $stmt->bindParam(':file_size', $file_size);
            $stmt->bindParam(':mime_type', $mime_type);
            $stmt->bindParam(':timestamp', $timestamp);
            $stmt->bindParam(':email', $user_email);
            
            $stmt->execute();
            
            // Create audit log
            $this->createAuditLog($document_id, $user_email, 'document_uploaded');
            
            return true;
        } catch (PDOException $e) {
            error_log("Error storing document: " . $e->getMessage());
            return false;
        }
    }
    
    /**
     * Clean up expired OTPs
     * @return bool Success status
     */
    public function cleanupExpiredOTPs() {
        try {
            $stmt = $this->db_conn->prepare("
                DELETE FROM otp_verification 
                WHERE expires_at < NOW() 
                AND verified = 0
            ");
            $stmt->execute();
            return true;
        } catch (PDOException $e) {
            error_log("Error cleaning up expired OTPs: " . $e->getMessage());
            return false;
        }
    }
    
    /**
     * Add rate limiting for OTP requests
     * @param string $email User email
     * @return bool Whether request is allowed
     */
    public function checkOTPRateLimit($email) {
        try {
            $stmt = $this->db_conn->prepare("
                SELECT COUNT(*) as count 
                FROM otp_verification 
                WHERE email = :email 
                AND created_at > DATE_SUB(NOW(), INTERVAL 1 HOUR)
            ");
            $stmt->bindParam(':email', $email);
            $stmt->execute();
            $result = $stmt->fetch();
            
            return $result['count'] < 5; // Allow max 5 requests per hour
        } catch (PDOException $e) {
            error_log("Error checking OTP rate limit: " . $e->getMessage());
            return true; // Allow request if check fails
        }
    }
    
    /**
     * Generate OTP for email verification
     * @param string $email User email
     * @param string $document_id Document ID
     * @return string|bool OTP string or false on failure
     */
    public function generateOTP($email, $document_id = null) {
        try {
            // Check rate limit
            if (!$this->checkOTPRateLimit($email)) {
                throw new Exception("Rate limit exceeded");
            }
            
            // Clean up expired OTPs
            $this->cleanupExpiredOTPs();
            
            // Generate 6-digit OTP
            $otp = str_pad(mt_rand(100000, 999999), 6, '0', STR_PAD_LEFT);
            
            // Set expiration time (5 minutes)
            $created_at = date('Y-m-d H:i:s');
            $expires_at = date('Y-m-d H:i:s', strtotime('+5 minutes'));
            
            $stmt = $this->db_conn->prepare("
                INSERT INTO otp_verification (
                    email, otp, document_id, created_at, expires_at
                ) VALUES (
                    :email, :otp, :doc_id, :created_at, :expires_at
                )
            ");
            
            $stmt->bindParam(':email', $email);
            $stmt->bindParam(':otp', $otp);
            $stmt->bindParam(':doc_id', $document_id);
            $stmt->bindParam(':created_at', $created_at);
            $stmt->bindParam(':expires_at', $expires_at);
            
            $stmt->execute();
            
            // Create audit log if document_id is provided
            if ($document_id) {
                $this->createAuditLog($document_id, $email, 'otp_requested');
            }
            
            return $otp;
        } catch (PDOException $e) {
            error_log("Error generating OTP: " . $e->getMessage());
            return false;
        }
    }
    
    /**
     * Verify OTP
     * @param string $email User email
     * @param string $otp OTP code to verify
     * @param string $document_id Document ID
     * @return bool Verification status
     */
    public function verifyOTP($email, $otp, $document_id = null) {
        try {
            $stmt = $this->db_conn->prepare("
                SELECT * FROM otp_verification 
                WHERE email = :email 
                AND otp = :otp 
                AND expires_at > NOW()
                ORDER BY created_at DESC 
                LIMIT 1
            ");
            
            $stmt->bindParam(':email', $email);
            $stmt->bindParam(':otp', $otp);
            
            $stmt->execute();
            $result = $stmt->fetch();
            
            if ($result) {
                // Mark as verified
                $updateStmt = $this->db_conn->prepare("
                    UPDATE otp_verification 
                    SET verified = 1, 
                        verification_timestamp = :timestamp,
                        verification_ip = :ip
                    WHERE id = :id
                ");
                
                $timestamp = time();
                $ip = $_SERVER['REMOTE_ADDR'];
                
                $updateStmt->bindParam(':timestamp', $timestamp);
                $updateStmt->bindParam(':ip', $ip);
                $updateStmt->bindParam(':id', $result['id']);
                
                $updateStmt->execute();
                
                // Create audit log
                if ($document_id) {
                    $this->createAuditLog($document_id, $email, 'otp_verified');
                }
                
                return true;
            }
            
            return false;
        } catch (PDOException $e) {
            error_log("Error verifying OTP: " . $e->getMessage());
            return false;
        }
    }
    
    /**
     * Create audit log entry
     * @param string $document_id Document ID
     * @param string $user_email User email
     * @param string $action Action name
     * @param array $additional_data Additional data to log
     * @return bool Success status
     */
    public function createAuditLog($document_id, $user_email, $action, $additional_data = null) {
        try {
            $stmt = $this->db_conn->prepare("
                INSERT INTO audit_logs (
                    document_id, user_email, action, timestamp,
                    ip_address, user_agent, additional_data
                ) VALUES (
                    :doc_id, :email, :action, :timestamp,
                    :ip, :user_agent, :data
                )
            ");
            
            $timestamp = time();
            $ip = $_SERVER['REMOTE_ADDR'];
            $user_agent = isset($_SERVER['HTTP_USER_AGENT']) ? $_SERVER['HTTP_USER_AGENT'] : null;
            $data_json = $additional_data ? json_encode($additional_data) : null;
            
            $stmt->bindParam(':doc_id', $document_id);
            $stmt->bindParam(':email', $user_email);
            $stmt->bindParam(':action', $action);
            $stmt->bindParam(':timestamp', $timestamp);
            $stmt->bindParam(':ip', $ip);
            $stmt->bindParam(':user_agent', $user_agent);
            $stmt->bindParam(':data', $data_json);
            
            $stmt->execute();
            
            return true;
        } catch (PDOException $e) {
            error_log("Error creating audit log: " . $e->getMessage());
            return false;
        }
    }
    
    // Add other methods as needed
}