// Export functions for use in other modules
window.appIntegration = {
    goToStep,
    downloadFinalDocument,
    prepareDocumentForSigning,
    renderDocumentWithFields: renderFinalDocument,
    drawFieldsOnCanvas: renderFieldsOnCanvas
};

// Also expose key functions globally
window.goToStep = goToStep;
window.downloadFinalDocument = downloadFinalDocument;
window.prepareDocumentForSigning = prepareDocumentForSigning;
window.renderDocumentWithFields = renderFinalDocument;
window.drawFieldsOnCanvas = renderFieldsOnCanvas;
/**
 * Application Integration Module
 * Ties together all components and ensures proper initialization
 */

document.addEventListener('DOMContentLoaded', function() {
    console.log('Initializing Inkless-Flow application with improved components...');
    
    // Make elements globally accessible
    window.elements = {
        fileInput: document.getElementById('file-input'),
        fileDropArea: document.getElementById('file-drop-area'),
        fileNameDisplay: document.getElementById('file-name-display'),
        filePreviewContainer: document.getElementById('file-preview-container'),
        finalPreviewContainer: document.getElementById('final-preview-container'),
        downloadPreviewContainer: document.getElementById('download-preview-container'),
        downloadTemplateLink: document.getElementById('download-template'),
        userEmailDisplay: document.getElementById('user-email-display'),
        
        signatureCanvas: document.getElementById('signature-canvas'),
        clearSignatureBtn: document.getElementById('clear-signature'),
        saveSignatureBtn: document.getElementById('save-signature'),
        savedSignature: document.getElementById('saved-signature'),
        signatureImagePreview: document.getElementById('signature-image-preview'),
        signaturePlaceholder: document.getElementById('signature-placeholder'),
        
        documentDate: document.getElementById('document-date'),
        savedDate: document.getElementById('saved-date'),
        dateValue: document.getElementById('date-value'),
        
        customFieldTitle: document.getElementById('custom-field-title'),
        customFieldContent: document.getElementById('custom-field-content'),
        addCustomFieldBtn: document.getElementById('add-custom-field'),
        customFieldList: document.getElementById('custom-field-list'),
        
        documentImage: document.getElementById('document-image'),
        savedImage: document.getElementById('saved-image'),
        imagePreview: document.getElementById('image-preview'),
        imagePlaceholder: document.getElementById('image-placeholder'),
        
        recipientName: document.getElementById('recipient-name'),
        recipientEmail: document.getElementById('recipient-email'),
        addRecipientBtn: document.getElementById('add-recipient'),
        recipientsList: document.getElementById('recipients-list'),
        
        signatureFieldTemplate: document.getElementById('signature-field-template'),
        dateFieldTemplate: document.getElementById('date-field-template'),
        imageFieldTemplate: document.getElementById('image-field-template'),
        recipientFieldTemplate: document.getElementById('recipient-field-template'),
        
        stepIndicators: document.querySelectorAll('.step-item'),
        steps: document.querySelectorAll('.step'),
        step1NextBtn: document.getElementById('step1-next'),
        step2BackBtn: document.getElementById('step2-back'),
        step2NextBtn: document.getElementById('step2-next'),
        step3BackBtn: document.getElementById('step3-back'),
        step3NextBtn: document.getElementById('step3-next'),
        step4BackBtn: document.getElementById('step4-back'),
        step4NextBtn: document.getElementById('step4-next'),
        step5BackBtn: document.getElementById('step5-back'),
        step5NextBtn: document.getElementById('step5-next'),
        step6BackBtn: document.getElementById('step6-back'),
        step6NextBtn: document.getElementById('step6-next'),
        step7BackBtn: document.getElementById('step7-back'),
        signAndSendBtn: document.getElementById('sign-and-send'),
        step8BackBtn: document.getElementById('step8-back'),
        downloadDocumentBtn: document.getElementById('download-document'),
        
        pageNavPrev: document.getElementById('prev-page'),
        pageNavNext: document.getElementById('next-page'),
        pageNavIndicator: document.getElementById('page-indicator')
    };
    
    // Initialize global application state
    if (!window.appState) {
        window.appState = {
            placedFields: [],
            fieldCounter: 0,
            customTextFields: [],
            uploadedFile: null,
            documentHash: null,
            signatureImage: null,
            selectedDate: null,
            uploadedImage: null,
            recipients: [],
            pageImages: [],
            totalPages: 0,
            pageDimensions: [],
            otpVerified: false,
            currentStep: 1,
            signatureData: null
        };
    }
    
    // Initialize all modules
    initializeModules();
    
    // Initialize step navigation
    initStepNavigation();
    
    // Hide loading screen when everything is loaded
    const loadingScreen = document.getElementById('loading-screen');
    if (loadingScreen) {
        loadingScreen.style.opacity = '0';
        setTimeout(() => {
            loadingScreen.style.display = 'none';
        }, 500);
    }
    
    /**
     * Initialize all modules
     */
    function initializeModules() {
        // Initialize modules with error handling
        try {
            // Initialize file handling
            if (typeof window.initFileHandling === 'function') {
                window.initFileHandling(window.elements);
                console.log('File handling module initialized');
            } else {
                console.error('initFileHandling function not available');
            }
            
            // Initialize signature pad
            if (typeof window.initSignaturePad === 'function') {
                window.initSignaturePad(window.elements);
                console.log('Signature pad module initialized');
            } else {
                console.error('initSignaturePad function not available');
            }
            
            // Initialize custom text fields
            if (typeof window.initCustomTextFields === 'function') {
                window.initCustomTextFields(window.elements);
                console.log('Custom text fields initialized');
            } else {
                console.error('initCustomTextFields function not available');
            }
            
            // Initialize recipient handling
            if (typeof window.initRecipientHandling === 'function') {
                window.initRecipientHandling(window.elements);
                console.log('Recipient handling initialized');
            } else {
                console.error('initRecipientHandling function not available');
            }
            
            // Initialize field handling
            if (typeof window.initFieldHandling === 'function') {
                window.initFieldHandling(window.elements);
                console.log('Field handling module initialized');
            } else {
                console.error('initFieldHandling function not available');
            }
            
            // Initialize OTP handlers
            if (typeof window.initOtpHandlers === 'function') {
                window.initOtpHandlers();
                console.log('OTP handlers initialized');
            } else {
                console.error('initOtpHandlers function not available');
            }
        } catch (error) {
            console.error('Error initializing modules:', error);
        }
    }
    
    /**
     * Initialize step navigation
     */
    function initStepNavigation() {
        // Set up navigation between steps
        const setupStepButton = (buttonId, action, targetStep, validationFunc) => {
            const button = document.getElementById(buttonId);
            if (button) {
                button.addEventListener('click', function() {
                    // Run validation if provided
                    if (validationFunc && !validationFunc()) {
                        return; // Validation failed
                    }
                    
                    goToStep(targetStep);
                });
            }
        };
        
        /**
         * Navigate to a specific step
         */
        function goToStep(stepNumber) {
            // Update app state
            window.appState.currentStep = stepNumber;
            
            // Update step visibility
            window.elements.steps.forEach(step => step.classList.remove('active'));
            const targetStep = document.getElementById('step' + stepNumber);
            if (targetStep) {
                targetStep.classList.add('active');
            } else {
                console.error(`Step ${stepNumber} element not found`);
                return;
            }
            
            // Update step indicators
            window.elements.stepIndicators.forEach((indicator, index) => {
                indicator.classList.remove('active', 'completed');
                
                // Mark previous steps as completed
                if (index + 1 < stepNumber) {
                    indicator.classList.add('completed');
                }
                
                // Mark current step as active
                if (index + 1 === stepNumber) {
                    indicator.classList.add('active');
                }
            });
            
            // Handle special step transitions
            if (stepNumber === 7) {
                prepareDocumentForSigning();
            } else if (stepNumber === 8) {
                prepareDownloadStep();
            }
        }
        
        /**
         * Prepare document for signing in Step 7
         */
        function prepareDocumentForSigning() {
            console.log("Preparing document for signing...");
            
            // Clear any existing fields
            const finalPreviewContainer = window.elements.finalPreviewContainer;
            if (finalPreviewContainer) {
                finalPreviewContainer.innerHTML = '';
            }
            
            // Check if we have document pages, if not, try to re-load them
            if (!window.appState.pageImages || window.appState.pageImages.length === 0) {
                console.log("No page images found, attempting to reload from file...");
                
                // Try to re-render the original file if it exists
                if (window.appState.uploadedFile && typeof window.handleFile === 'function') {
                    console.log("Re-loading file:", window.appState.uploadedFile.name);
                    // This will trigger the document rendering
                    window.handleFile(window.appState.uploadedFile);
                    
                    // Return early and let the file handler re-trigger this function
                    return;
                }
            }
            
            // Display all document pages
            if (window.appState.pageImages && window.appState.pageImages.length > 0) {
                // Create page elements
                for (let i = 0; i < window.appState.pageImages.length; i++) {
                    const pageNum = i + 1;
                    const dims = window.appState.pageDimensions[i];
                    
                    // Create page div
                    const pageDiv = document.createElement('div');
                    pageDiv.className = 'document-page';
                    pageDiv.dataset.pageNum = pageNum;
                    pageDiv.style.marginBottom = '20px';
                    
                    // Create canvas
                    const canvas = document.createElement('canvas');
                    canvas.className = 'preview-canvas';
                    canvas.width = dims.width;
                    canvas.height = dims.height;
                    canvas.style.maxWidth = '100%';
                    
                    // Draw page image
                    const ctx = canvas.getContext('2d');
                    const img = new Image();
                    img.onload = function() {
                        ctx.drawImage(img, 0, 0, canvas.width, canvas.height);
                    };
                    img.src = window.appState.pageImages[i];
                    
                    // Add page number
                    const pageNumber = document.createElement('div');
                    pageNumber.className = 'page-number';
                    pageNumber.textContent = `Page ${pageNum}`;
                    
                    // Add field overlay
                    const fieldOverlay = document.createElement('div');
                    fieldOverlay.className = 'field-overlay';
                    fieldOverlay.style.position = 'absolute';
                    fieldOverlay.style.top = '0';
                    fieldOverlay.style.left = '0';
                    fieldOverlay.style.width = '100%';
                    fieldOverlay.style.height = '100%';
                    
                    // Set page to relative positioning
                    pageDiv.style.position = 'relative';
                    
                    // Add elements to page
                    pageDiv.appendChild(canvas);
                    pageDiv.appendChild(pageNumber);
                    pageDiv.appendChild(fieldOverlay);
                    
                    // Add page to container
                    finalPreviewContainer.appendChild(pageDiv);
                }
                
                // Set up page navigation
                updatePageNavigation();
                
                // Restore placed fields if any
                if (window.appState.placedFields.length > 0) {
                    setTimeout(() => {
                        restorePlacedFields();
                    }, 100);
                }
            }
        }
        
        /**
         * Prepare final step for download
         */
        function prepareDownloadStep() {
            console.log("Preparing download step...");
            
            // Make sure download preview is up to date
            renderDocumentWithFields();
            
            // Set user email display
            if (window.elements.userEmailDisplay) {
                const email = document.getElementById('yourEmail')?.value || 'your email address';
                window.elements.userEmailDisplay.textContent = email;
            }
            
            // Add eIDAS verification information if available
            if (window.appState.signatureData) {
                addSignatureInfoToFinalStep();
            }
        }
        
        /**
         * Add signature information to the final step
         */
        function addSignatureInfoToFinalStep() {
            const data = window.appState.signatureData;
            if (!data) return;
            
            // Remove any existing signature info
            const existingInfo = document.querySelector('.signature-info-section');
            if (existingInfo) existingInfo.remove();
            
            // Create signature information div
            const signatureInfoDiv = document.createElement('div');
            signatureInfoDiv.className = 'signature-info-section';
            signatureInfoDiv.style.marginTop = '20px';
            
            signatureInfoDiv.innerHTML = `
                <h3>eIDAS Compliant Signature Information</h3>
                <table style="width: 100%; border-collapse: collapse;">
                    <tr>
                        <td style="padding: 8px; border-bottom: 1px solid #ddd;"><strong>Signer:</strong></td>
                        <td style="padding: 8px; border-bottom: 1px solid #ddd;">${data.userId}</td>
                    </tr>
                    <tr>
                        <td style="padding: 8px; border-bottom: 1px solid #ddd;"><strong>Timestamp:</strong></td>
                        <td style="padding: 8px; border-bottom: 1px solid #ddd;">${new Date(data.timestamp * 1000).toLocaleString()}</td>
                    </tr>
                    <tr>
                        <td style="padding: 8px; border-bottom: 1px solid #ddd;"><strong>Document Hash:</strong></td>
                        <td style="padding: 8px; border-bottom: 1px solid #ddd;"><code>${data.documentHash}</code></td>
                    </tr>
                    <tr>
                        <td style="padding: 8px; border-bottom: 1px solid #ddd;"><strong>Timestamp Source:</strong></td>
                        <td style="padding: 8px; border-bottom: 1px solid #ddd;">${data.timestampSource}</td>
                    </tr>
                    <tr>
                        <td style="padding: 8px;"><strong>Verification Status:</strong></td>
                        <td style="padding: 8px; color: green;"><strong>✓ Verified</strong></td>
                    </tr>
                </table>
                <p style="margin-top: 10px; color: #666; font-size: 12px;">
                    This signature is compliant with eIDAS regulations for Simple Electronic Signatures (SES).
                    A cryptographic record of this signature has been stored in the audit log.
                </p>
            `;
            
            // Add to the page
            const downloadContainer = document.getElementById('step8');
            if (downloadContainer) {
                const card = downloadContainer.querySelector('.card');
                if (card) {
                    card.appendChild(signatureInfoDiv);
                }
            }
        }
        
        /**
         * Restore placed fields on the document
         */
        function restorePlacedFields() {
            for (const field of window.appState.placedFields) {
                const pageDiv = document.querySelector(`.document-page[data-page-num="${field.pageNum}"]`);
                if (!pageDiv) continue;
                
                const overlay = pageDiv.querySelector('.field-overlay');
                if (!overlay) continue;
                
                // Create field element
                const fieldElement = document.createElement('div');
                fieldElement.id = field.id;
                fieldElement.className = `field-container ${field.verified ? 'verified' : 'draft'}`;
                fieldElement.dataset.type = field.type;
                fieldElement.dataset.pageNum = field.pageNum;
                
                // Set position and size
                fieldElement.style.position = 'absolute';
                fieldElement.style.left = field.left + 'px';
                fieldElement.style.top = field.top + 'px';
                fieldElement.style.width = field.width + 'px';
                fieldElement.style.height = field.height + 'px';
                fieldElement.style.zIndex = '10';
                
                // Add content based on field type
                let fieldContent = '';
                let customData = field.customData;
                
                switch (field.type) {
                    case 'signature-field':
                        fieldContent = `
                            <div class="field-label">Your Signature</div>
                            <div class="field-content">
                                <img src="${window.appState.signatureImage}" style="max-width: 100%; max-height: 100%; background: transparent;">
                            </div>
                        `;
                        break;
                    case 'date-field':
                        fieldContent = `
                            <div class="field-label">Date</div>
                            <div class="field-content">
                                ${new Date(window.appState.selectedDate).toLocaleDateString()}
                            </div>
                        `;
                        break;
                    case 'image-field':
                        fieldContent = `
                            <div class="field-label">Image</div>
                            <div class="field-content">
                                <div id="image-placeholder-${field.id}">Image Field</div>
                            </div>
                        `;
                        break;
                    case 'recipient-field':
                        const recipient = window.appState.recipients.length > 0 ? 
                            window.appState.recipients[0].name : 'Recipient';
                        fieldContent = `
                            <div class="field-label">${recipient} Signature</div>
                            <div class="field-content">
                                Signature Field
                            </div>
                        `;
                        break;
                    case 'custom-text-field':
                        if (customData) {
                            fieldContent = `
                                <div class="field-label">${customData.title}</div>
                                <div class="field-content">
                                    ${customData.content}
                                </div>
                            `;
                        }
                        break;
                }
                
                // Add controls based on verified status
                if (field.verified) {
                    fieldContent += `
                        <div class="field-controls">
                            <button class="field-control-btn delete-btn" title="Delete field">×</button>
                        </div>
                    `;
                } else {
                    fieldContent += `
                        <div class="field-controls">
                            <button class="field-control-btn place-btn" title="Confirm placement">✓</button>
                            <button class="field-control-btn remove-btn" title="Remove field">×</button>
                        </div>
                        <div class="resize-handle top-left"></div>
                        <div class="resize-handle top-right"></div>
                        <div class="resize-handle bottom-left"></div>
                        <div class="resize-handle bottom-right"></div>
                    `;
                }
                
                fieldElement.innerHTML = fieldContent;
                
                // Add to overlay
                overlay.appendChild(fieldElement);
                
                // Setup field behaviors
                if (field.verified) {
                    // For verified fields, just add delete button handler
                    const deleteBtn = fieldElement.querySelector('.delete-btn');
                    if (deleteBtn) {
                        deleteBtn.addEventListener('click', function(e) {
                            e.stopPropagation();
                            window.removeField(fieldElement);
                        });
                    }
                } else {
                    // Make field draggable and resizable
                    if (typeof window.makeDraggable === 'function') {
                        window.makeDraggable(fieldElement);
                    }
                    
                    if (typeof window.makeResizable === 'function') {
                        window.makeResizable(fieldElement);
                    }
                    
                    // Add button handlers
                    const placeBtn = fieldElement.querySelector('.place-btn');
                    if (placeBtn) {
                        placeBtn.addEventListener('click', function(e) {
                            e.stopPropagation();
                            window.placeField(fieldElement);
                        });
                    }
                    
                    const removeBtn = fieldElement.querySelector('.remove-btn');
                    if (removeBtn) {
                        removeBtn.addEventListener('click', function(e) {
                            e.stopPropagation();
                            window.removeField(fieldElement);
                        });
                    }
                }
            }
        }
        
        /**
         * Update page navigation controls
         */
        function updatePageNavigation() {
            const prevBtn = window.elements.pageNavPrev;
            const nextBtn = window.elements.pageNavNext;
            const indicator = window.elements.pageNavIndicator;
            
            if (!prevBtn || !nextBtn || !indicator) return;
            
            const totalPages = window.appState.totalPages || 1;
            
            // Update indicator
            indicator.textContent = `Page 1 of ${totalPages}`;
            
            // Update button states
            prevBtn.disabled = true;
            nextBtn.disabled = totalPages <= 1;
            
            // Add click handlers
            prevBtn.onclick = function() {
                navigatePages('prev');
            };
            
            nextBtn.onclick = function() {
                navigatePages('next');
            };
        }
        
        /**
         * Navigate between pages
         */
        function navigatePages(direction) {
            const indicator = window.elements.pageNavIndicator;
            if (!indicator) return;
            
            const match = indicator.textContent.match(/Page (\d+) of (\d+)/);
            if (!match) return;
            
            let currentPage = parseInt(match[1]);
            const totalPages = parseInt(match[2]);
            
            // Calculate new page
            if (direction === 'next' && currentPage < totalPages) {
                currentPage++;
            } else if (direction === 'prev' && currentPage > 1) {
                currentPage--;
            } else {
                return; // No change
            }
            
            // Update indicator
            indicator.textContent = `Page ${currentPage} of ${totalPages}`;
            
            // Update button states
            if (window.elements.pageNavPrev) {
                window.elements.pageNavPrev.disabled = currentPage <= 1;
            }
            
            if (window.elements.pageNavNext) {
                window.elements.pageNavNext.disabled = currentPage >= totalPages;
            }
            
            // Show current page
            const pages = document.querySelectorAll('.document-page');
            pages.forEach((page, index) => {
                if (index + 1 === currentPage) {
                    page.style.display = 'block';
                } else {
                    page.style.display = 'none';
                }
            });
        }
        
        // Set up all navigation buttons
        setupStepButton('step1-next', 'next', 2, validateStep1);
        setupStepButton('step2-back', 'back', 1);
        setupStepButton('step2-next', 'next', 3, validateStep2);
        setupStepButton('step3-back', 'back', 2);
        setupStepButton('step3-next', 'next', 4);
        setupStepButton('step4-back', 'back', 3);
        setupStepButton('step4-next', 'next', 5);
        setupStepButton('step5-back', 'back', 4);
        setupStepButton('step5-next', 'next', 6);
        setupStepButton('step6-back', 'back', 5);
        setupStepButton('step6-next', 'next', 7);
        setupStepButton('step7-back', 'back', 6);
        
        // Special handling for final step
        if (window.elements.signAndSendBtn) {
            window.elements.signAndSendBtn.addEventListener('click', async function() {
                if (!validateStep7()) {
                    return;
                }
                
                try {
                    // Generate final document with signatures
                    await generateFinalDocument();
                    
                    // Go to final step
                    goToStep(8);
                } catch (error) {
                    console.error('Error generating final document:', error);
                    alert('An error occurred while preparing the final document. Please try again.');
                }
            });
        }
        
        // Back button from final step
        setupStepButton('step8-back', 'back', 7);
        
        // Download button in final step
        if (window.elements.downloadDocumentBtn) {
            window.elements.downloadDocumentBtn.addEventListener('click', function() {
                downloadFinalDocument();
            });
        }
        
        // Template download link
        if (window.elements.downloadTemplateLink) {
            window.elements.downloadTemplateLink.addEventListener('click', function(e) {
                e.preventDefault();
                downloadTemplateDocument();
            });
        }
    }
    
    /**
     * Validate Step 1: File upload and user information
     */
    function validateStep1() {
        // Check if file is uploaded
        if (!window.appState.uploadedFile) {
            alert('Please upload a document first');
            return false;
        }
        
        // Check name
        const yourName = document.getElementById('yourName');
        if (yourName && !yourName.value.trim()) {
            alert('Please enter your name');
            yourName.focus();
            return false;
        }
        
        // Check email
        const yourEmail = document.getElementById('yourEmail');
        if (yourEmail) {
            const email = yourEmail.value.trim();
            if (!email) {
                alert('Please enter your email address');
                yourEmail.focus();
                return false;
            }
            
            // Validate email format
            const emailRegex = /^[^\s@]+@[^\s@]+\.[^\s@]+$/;
            if (!emailRegex.test(email)) {
                alert('Please enter a valid email address');
                yourEmail.focus();
                return false;
            }
            
            // Update email display in step 8
            if (window.elements.userEmailDisplay) {
                window.elements.userEmailDisplay.textContent = email;
            }
        }
        
        // Check privacy consent
        const privacyCheckbox = document.getElementById('data-privacy-checkbox');
        if (privacyCheckbox && !privacyCheckbox.checked) {
            alert('Please consent to data processing before continuing');
            return false;
        }
        
        return true;
    }
    
    /**
     * Validate Step 2: Signature
     */
    function validateStep2() {
        if (!window.appState.signatureImage) {
            alert('Please draw and save your signature');
            return false;
        }
        return true;
    }
    
    /**
     * Generate final document with signatures and fields
     */
    async function generateFinalDocument() {
        // Show loading indicator
        if (typeof window.showLoadingIndicator === 'function') {
            window.showLoadingIndicator();
        }
        
        try {
            // Render the document with all fields
            await renderDocumentWithFields();
            
            // Generate trusted timestamp
            const timestamp = await getTrustedTimestamp();
            
            // Store signature data in app state
            window.appState.signatureData = {
                userId: document.getElementById('yourEmail').value,
                documentHash: window.appState.documentHash,
                timestamp: timestamp.timestamp,
                timestampIso: timestamp.iso,
                timestampSource: timestamp.source,
                ipAddress: '127.0.0.1' // Placeholder
            };
            
            // Hide loading indicator
            if (typeof window.hideLoadingIndicator === 'function') {
                window.hideLoadingIndicator();
            }
            
            return true;
        } catch (error) {
            console.error('Error generating final document:', error);
            
            // Hide loading indicator
            if (typeof window.hideLoadingIndicator === 'function') {
                window.hideLoadingIndicator();
            }
            
            throw error;
        }
    }
    
    /**
     * Get trusted timestamp from a reliable source
     */
    async function getTrustedTimestamp() {
        try {
            // Try to get timestamp from worldtimeapi.org
            const response = await fetch('https://worldtimeapi.org/api/ip');
            const data = await response.json();
            
            return {
                timestamp: data.unixtime,
                iso: data.datetime,
                source: 'worldtimeapi.org'
            };
        } catch (error) {
            console.error('Error getting trusted timestamp:', error);
            
            // Fallback to local time
            const now = new Date();
            return {
                timestamp: Math.floor(now.getTime() / 1000),
                iso: now.toISOString(),
                source: 'local'
            };
        }
    }
    
    /**
     * Render the document with all fields
     */
    async function renderDocumentWithFields() {
        // Get the preview container for the download
        const container = window.elements.downloadPreviewContainer;
        if (!container) return;
        
        // Clear the container
        container.innerHTML = '';
        
        // Create canvases for each page
        for (let pageNum = 1; pageNum <= window.appState.totalPages; pageNum++) {
            const dims = window.appState.pageDimensions[pageNum - 1];
            if (!dims) continue;
            
            // Create canvas for this page
            const canvas = document.createElement('canvas');
            canvas.className = 'final-canvas';
            canvas.width = dims.width;
            canvas.height = dims.height;
            canvas.style.maxWidth = '100%';
            canvas.style.marginBottom = '20px';
            
            // Draw the background image
            const img = new Image();
            img.onload = function() {
                const ctx = canvas.getContext('2d');
                ctx.drawImage(img, 0, 0, canvas.width, canvas.height);
                
                // Draw the fields for this page
                drawFieldsOnCanvas(ctx, pageNum, canvas.width, canvas.height);
            };
            img.src = window.appState.pageImages[pageNum - 1];
            
            // Add to container
            container.appendChild(canvas);
        }
        
        // Wait for all images to load
        return new Promise((resolve) => {
            setTimeout(resolve, 500); // Simple timeout to ensure rendering completes
        });
    }
    
    /**
     * Draw fields on a canvas
     */
    function drawFieldsOnCanvas(ctx, pageNum, width, height) {
        // Get all fields for this page
        const fields = window.appState.placedFields.filter(field => 
            field.pageNum === pageNum && field.verified);
        
        // Draw each field
        fields.forEach(field => {
            // Draw field content based on type
            if (field.type === 'signature-field' && window.appState.signatureImage) {
                const img = new Image();
                img.onload = function() {
                    // Calculate aspect ratio
                    const aspectRatio = img.width / img.height;
                    let drawWidth = field.width - 10;
                    let drawHeight = drawWidth / aspectRatio;
                    
                    // Adjust if too tall
                    if (drawHeight > field.height - 10) {
                        drawHeight = field.height - 10;
                        drawWidth = drawHeight * aspectRatio;
                    }
                    
                    // Center the image in the field
                    const centerX = field.left + (field.width - drawWidth) / 2;
                    const centerY = field.top + (field.height - drawHeight) / 2;
                    
                    ctx.drawImage(img, centerX, centerY, drawWidth, drawHeight);
                };
                img.src = window.appState.signatureImage;
            } else if (field.type === 'date-field' && window.appState.selectedDate) {
                ctx.font = '14px Arial';
                ctx.fillStyle = '#000000';
                ctx.fillText(new Date(window.appState.selectedDate).toLocaleDateString(), field.left + 10, field.top + 25);
            } else if (field.type === 'custom-text-field' && field.customData) {
                ctx.font = '14px Arial';
                ctx.fillStyle = '#000000';
                ctx.fillText(field.customData.content, field.left + 10, field.top + 25);
            } else if (field.type === 'image-field' && window.appState.uploadedImage) {
                const reader = new FileReader();
                reader.onload = function(e) {
                    const img = new Image();
                    img.onload = function() {
                        // Calculate aspect ratio
                        const aspectRatio = img.width / img.height;
                        let drawWidth = field.width - 10;
                        let drawHeight = drawWidth / aspectRatio;
                        
                        // Adjust if too tall
                        if (drawHeight > field.height - 10) {
                            drawHeight = field.height - 10;
                            drawWidth = drawHeight * aspectRatio;
                        }
                        
                        // Center the image in the field
                        const centerX = field.left + (field.width - drawWidth) / 2;
                        const centerY = field.top + (field.height - drawHeight) / 2;
                        
                        ctx.drawImage(img, centerX, centerY, drawWidth, drawHeight);
                    };
                    img.src = e.target.result;
                };
                reader.readAsDataURL(window.appState.uploadedImage);
            } else if (field.type === 'recipient-field') {
                ctx.font = '14px Arial';
                ctx.fillStyle = '#000000';
                const recipient = window.appState.recipients.length > 0 ? 
                    window.appState.recipients[0].name : 'Recipient';
                ctx.fillText(`${recipient} (Pending)`, field.left + 10, field.top + 25);
            }
        });
        
        // Add timestamp at the bottom
        ctx.font = '10px Arial';
        ctx.fillStyle = '#999999';
        ctx.textAlign = 'right';
        ctx.fillText(`Signed with Inkless-Flow on ${new Date().toLocaleDateString()} ${new Date().toLocaleTimeString()}`, width - 10, height - 10);
        ctx.textAlign = 'left';
    }
    
    /**
     * Download the final document as PDF
     */
    function downloadFinalDocument() {
        const canvases = window.elements.downloadPreviewContainer.querySelectorAll('canvas');
        if (canvases.length === 0) {
            alert('No document to download. Please try again.');
            return;
        }
        
        try {
            // Check if jsPDF is available
            if (typeof window.jspdf === 'undefined' || typeof window.jspdf.jsPDF === 'undefined') {
                alert('PDF generation library not loaded. Please check your internet connection and try again.');
                return;
            }
            
            const { jsPDF } = window.jspdf;
            const doc = new jsPDF({
                orientation: 'portrait',
                unit: 'px',
                format: [canvases[0].width, canvases[0].height]
            });
            
            // Add each page to the PDF
            canvases.forEach((canvas, index) => {
                if (index > 0) {
                    doc.addPage([canvas.width, canvas.height]);
                }
                
                const imgData = canvas.toDataURL('image/png');
                doc.addImage(imgData, 'PNG', 0, 0, canvas.width, canvas.height);
            });
            
            // Add signature metadata if available
            if (window.appState.signatureData) {
                const data = window.appState.signatureData;
                const info = {
                    Subject: 'Digitally Signed Document',
                    Author: data.userId,
                    Keywords: 'eIDAS, electronic signature, Inkless-Flow',
                    Creator: 'Inkless-Flow eSignature',
                    SignatureTime: data.timestampIso,
                    SignatureSource: data.timestampSource
                };
                
                doc.setDocumentProperties(info);
            }
            
            // Get document name
            const fileName = window.appState.uploadedFile ? 
                `signed_${window.appState.uploadedFile.name.replace(/\.[^/.]+$/, '')}.pdf` : 
                'signed_document.pdf';
                
            // Download the PDF
            doc.save(fileName);
            
            console.log('Document downloaded successfully!');
        } catch (error) {
            console.error('Error generating PDF:', error);
            alert('Error creating PDF. Please try again.');
        }
    }
    
    /**
     * Download a template document
     */
    function downloadTemplateDocument() {
        try {
            // Check if jsPDF is available
            if (typeof window.jspdf === 'undefined' || typeof window.jspdf.jsPDF === 'undefined') {
                alert('PDF generation library not loaded. Please check your internet connection and try again.');
                return;
            }
            
            const { jsPDF } = window.jspdf;
            const doc = new jsPDF();
            
            // Create a simple contract template
            doc.setFontSize(22);
            doc.text('Inkless-Flow Contract Template', 105, 20, { align: 'center' });
            
            doc.setFontSize(12);
            doc.text('This is a sample contract document template.', 20, 40);
            doc.text('Please fill in the following information:', 20, 50);
            
            doc.text('Party A: ________________________________', 20, 70);
            doc.text('Party B: ________________________________', 20, 80);
            
            doc.text('Date: ____________________', 20, 100);
            
            doc.text('Terms and Conditions:', 20, 120);
            doc.text('1. Lorem ipsum dolor sit amet, consectetur adipiscing elit.', 30, 130);
            doc.text('2. Sed do eiusmod tempor incididunt ut labore et dolore magna aliqua.', 30, 140);
            doc.text('3. Ut enim ad minim veniam, quis nostrud exercitation ullamco laboris.', 30, 150);
            
            doc.text('Signatures:', 20, 180);
            doc.text('Party A: ________________________________', 20, 200);
            doc.text('Party B: ________________________________', 20, 220);
            
            doc.text('Created with Inkless-Flow eSignature Platform', 105, 280, { align: 'center' });
            
            // Download the PDF
            doc.save('inkless_flow_contract_template.pdf');
            
            console.log('Template downloaded successfully!');
        } catch (error) {
            console.error('Error generating template PDF:', error);
            alert('Error creating template. Please try again.');
        }
    }
    
    /**
     * Validate Step 7: Field placement and OTP verification
     */
    function validateStep7() {
        // Check if signature field is placed and verified
        const signatureVerified = window.appState.placedFields.some(
            field => field.type === 'signature-field' && field.verified
        );
        
        if (!signatureVerified) {
            alert('Please place and confirm your signature field on the document');
            return false;
        }
        
        // Check if OTP verification is complete
        if (!window.appState.otpVerified) {
            // For development only - allow skipping OTP verification
            if (location.hostname === 'localhost' || location.hostname === '127.0.0.1') {
                if (confirm('For development purposes: Skip OTP verification?')) {
                    window.appState.otpVerified = true;
                } else {
                    alert('Please complete identity verification before signing');
                    return false;
                }
            } else {
                alert('Please complete identity verification before signing');
                return false;
            }
        }
        
        // Check legal disclaimer checkbox
        const legalDisclaimerCheckbox = document.getElementById('legal-disclaimer-checkbox');
        if (legalDisclaimerCheckbox && !legalDisclaimerCheckbox.checked) {
            alert('Please accept the legal disclaimer before continuing');
            return false;
        }
        
        return true;
    }
});