/**
 * Main Application JavaScript
 * Initializes the application and coordinates between modules
 */

/**
 * OTP Integration Module
 * Connects the frontend OTP system with backend PHP endpoints
 */

// Initialize OTP handlers when document is ready
document.addEventListener('DOMContentLoaded', function() {
    initOtpHandlers();
});

/**
 * Initialize OTP verification handlers
 */
function initOtpHandlers() {
    const requestOtpBtn = document.getElementById('request-otp-btn');
    const verifyOtpBtn = document.getElementById('verify-otp-btn');
    const otpInput = document.getElementById('otp-input');
    const otpInputContainer = document.getElementById('otp-input-container');
    const verificationStatus = document.getElementById('verification-status');
    const legalDisclaimerCheckbox = document.getElementById('legal-disclaimer-checkbox');
    const signAndSendBtn = document.getElementById('sign-and-send');
    
    // Initially disable sign button until verification
    if (signAndSendBtn) {
        signAndSendBtn.disabled = true;
    }
    
    // Request OTP button click handler
    if (requestOtpBtn) {
        requestOtpBtn.addEventListener('click', async function() {
            try {
                // Get email from the form
                const email = document.getElementById('yourEmail').value;
                if (!email) {
                    alert('Please enter your email address in Step 1.');
                    return;
                }
                
                // Show loading state
                requestOtpBtn.disabled = true;
                requestOtpBtn.innerHTML = '<span class="spinner-border spinner-border-sm mr-2" role="status" aria-hidden="true"></span> Sending...';
                
                // Prepare request data
                const requestData = {
                    email: email,
                    document_id: window.appState.documentHash || 'temp_document'
                };
                
                // Send request to backend
                const response = await fetch('send-otp.php', {
                    method: 'POST',
                    headers: {
                        'Content-Type': 'application/json'
                    },
                    body: JSON.stringify(requestData)
                });
                
                const result = await response.json();
                
                // Show OTP input field
                if (otpInputContainer) {
                    otpInputContainer.style.display = 'block';
                }
                if (otpInput) {
                    otpInput.focus();
                }
                
                // Reset button state
                requestOtpBtn.disabled = false;
                requestOtpBtn.innerHTML = '<i class="fas fa-envelope mr-1"></i> Resend verification code';
                
                // Show success message
                if (result.success) {
                    // For development only
                    if (result.dev_otp) {
                        console.log('Development OTP:', result.dev_otp);
                        if (otpInput) {
                            otpInput.value = result.dev_otp; // Auto-fill for development
                        }
                    }
                    
                    alert('A verification code has been sent to your email address. Please check your spam folder if necessary.');
                } else {
                    alert('Error sending verification code: ' + result.message);
                }
            } catch (error) {
                console.error('Error requesting OTP:', error);
                alert('An error occurred while requesting the verification code. Please try again.');
                
                // Reset button state
                requestOtpBtn.disabled = false;
                requestOtpBtn.innerHTML = '<i class="fas fa-envelope mr-1"></i> Request verification code';
            }
        });
    }
    
    // Verify OTP button click handler
    if (verifyOtpBtn && otpInput) {
        verifyOtpBtn.addEventListener('click', function() {
            verifyOTP();
        });
        
        // Allow pressing Enter to verify
        otpInput.addEventListener('keyup', function(e) {
            if (e.key === 'Enter') {
                verifyOTP();
            }
        });
    }
    
    // OTP verification function
    async function verifyOTP() {
        const otp = otpInput.value.trim();
        if (!otp || otp.length !== 6 || !/^\d+$/.test(otp)) {
            alert('Please enter the 6-digit code received by email.');
            return;
        }
        
        try {
            // Get email from the form
            const email = document.getElementById('yourEmail').value;
            
            // Show loading state
            verifyOtpBtn.disabled = true;
            verifyOtpBtn.innerHTML = '<span class="spinner-border spinner-border-sm" role="status" aria-hidden="true"></span>';
            
            // Prepare verification data
            const verificationData = {
                email: email,
                otp: otp,
                document_id: window.appState.documentHash || 'temp_document'
            };
            
            // Send verification request
            const response = await fetch('verify-otp.php', {
                method: 'POST',
                headers: {
                    'Content-Type': 'application/json'
                },
                body: JSON.stringify(verificationData)
            });
            
            const result = await response.json();
            
            // Reset button state
            verifyOtpBtn.disabled = false;
            verifyOtpBtn.innerHTML = 'Verify';
            
            // Handle verification result
            if (result.success) {
                // Update verification status
                if (verificationStatus) {
                    verificationStatus.textContent = 'Verified';
                    verificationStatus.className = 'badge badge-success';
                }
                
                // Update global status
                window.appState.otpVerified = true;
                
                // Enable legal disclaimer checkbox
                if (legalDisclaimerCheckbox) {
                    legalDisclaimerCheckbox.disabled = false;
                    
                    // Setup legal disclaimer checkbox handler
                    legalDisclaimerCheckbox.addEventListener('change', function() {
                        if (signAndSendBtn) {
                            signAndSendBtn.disabled = !this.checked;
                        }
                    });
                }
                
                // Show success message
                alert('Identity verified successfully!');
            } else {
                // Show error message
                alert('Verification failed: ' + result.message);
            }
        } catch (error) {
            console.error('Error verifying OTP:', error);
            alert('An error occurred while verifying the code. Please try again.');
            
            // Reset button state
            verifyOtpBtn.disabled = false;
            verifyOtpBtn.innerHTML = 'Verify';
        }
    }
}

// Export function for global usage
window.initOtpHandlers = initOtpHandlers;

    // DOM Elements
    const elements = {
        fileInput: document.getElementById('file-input'),
        fileDropArea: document.getElementById('file-drop-area'),
        fileNameDisplay: document.getElementById('file-name-display'),
        filePreviewContainer: document.getElementById('file-preview-container'),
        finalPreviewContainer: document.getElementById('final-preview-container'),
        downloadPreviewContainer: document.getElementById('download-preview-container'),
        downloadTemplateLink: document.getElementById('download-template'),
        userEmailDisplay: document.getElementById('user-email-display'),
        
        signatureCanvas: document.getElementById('signature-canvas'),
        clearSignatureBtn: document.getElementById('clear-signature'),
        saveSignatureBtn: document.getElementById('save-signature'),
        savedSignature: document.getElementById('saved-signature'),
        signatureImagePreview: document.getElementById('signature-image-preview'),
        signaturePlaceholder: document.getElementById('signature-placeholder'),
        
        documentDate: document.getElementById('document-date'),
        savedDate: document.getElementById('saved-date'),
        dateValue: document.getElementById('date-value'),
        
        customFieldTitle: document.getElementById('custom-field-title'),
        customFieldContent: document.getElementById('custom-field-content'),
        addCustomFieldBtn: document.getElementById('add-custom-field'),
        customFieldList: document.getElementById('custom-field-list'),
        
        documentImage: document.getElementById('document-image'),
        savedImage: document.getElementById('saved-image'),
        imagePreview: document.getElementById('image-preview'),
        imagePlaceholder: document.getElementById('image-placeholder'),
        imageFileInput: document.getElementById('image-file-input'),
        imageDropArea: document.getElementById('image-drop-area'),
        
        recipientName: document.getElementById('recipient-name'),
        recipientEmail: document.getElementById('recipient-email'),
        addRecipientBtn: document.getElementById('add-recipient'),
        recipientsList: document.getElementById('recipients-list'),
        
        signatureFieldTemplate: document.getElementById('signature-field-template'),
        dateFieldTemplate: document.getElementById('date-field-template'),
        imageFieldTemplate: document.getElementById('image-field-template'),
        recipientFieldTemplate: document.getElementById('recipient-field-template'),
        
        stepIndicators: document.querySelectorAll('.step-item'),
        steps: document.querySelectorAll('.step'),
        step1NextBtn: document.getElementById('step1-next'),
        step2BackBtn: document.getElementById('step2-back'),
        step2NextBtn: document.getElementById('step2-next'),
        step3BackBtn: document.getElementById('step3-back'),
        step3NextBtn: document.getElementById('step3-next'),
        step4BackBtn: document.getElementById('step4-back'),
        step4NextBtn: document.getElementById('step4-next'),
        step5BackBtn: document.getElementById('step5-back'),
        step5NextBtn: document.getElementById('step5-next'),
        step6BackBtn: document.getElementById('step6-back'),
        step6NextBtn: document.getElementById('step6-next'),
        step7BackBtn: document.getElementById('step7-back'),
        signAndSendBtn: document.getElementById('sign-and-send'),
        step8BackBtn: document.getElementById('step8-back'),
        downloadDocumentBtn: document.getElementById('download-document'),
        
        pageNavPrev: document.getElementById('prev-page'),
        pageNavNext: document.getElementById('next-page'),
        pageNavIndicator: document.getElementById('page-indicator')
    };

    // Make elements globally accessible
    window.elements = elements;
    
    // Initialize modules with error handling and proper ordering
    initializeModules();
    
    // Initialize step navigation
    initStepNavigation();
    
    // Initialize page navigation controls
    initPageNavigation();
    
    // Initialize image upload functionality
    initImageUpload();
    
    /**
     * Initialize all modules in the correct order with proper error handling
     */
    function initializeModules() {
        // Initialize eIDAS compliance module first (dependency for others)
        try {
            if (typeof window.eidasCompliance === 'object' && typeof window.eidasCompliance.init === 'function') {
                window.eidasCompliance.init();
                console.log('eIDAS compliance module initialized');
            } else {
                console.warn('eIDAS compliance module not available, some features may be limited');
            }
        } catch (error) {
            console.error('Error initializing eIDAS compliance module:', error);
        }
        
        // Initialize GDPR compliance module
        try {
            if (typeof window.gdprCompliance === 'object' && typeof window.gdprCompliance.init === 'function') {
                window.gdprCompliance.init();
                console.log('GDPR compliance module initialized');
            } else {
                console.warn('GDPR compliance module not available, some features may be limited');
            }
        } catch (error) {
            console.error('Error initializing GDPR compliance module:', error);
        }
        
        // Initialize file handling
        try {
            if (typeof window.initFileHandling === 'function') {
                window.initFileHandling(elements);
                console.log('File handling module initialized');
            } else {
                console.error('initFileHandling function not available');
            }
        } catch (error) {
            console.error('Error initializing file handling module:', error);
        }
        
        // Initialize signature pad
        try {
            if (typeof window.initSignaturePad === 'function') {
                window.initSignaturePad(elements);
                console.log('Signature pad module initialized');
            } else {
                console.error('initSignaturePad function not available');
            }
        } catch (error) {
            console.error('Error initializing signature pad module:', error);
        }
        
        // Initialize field handling last (depends on other modules)
        try {
            if (typeof window.initFieldHandling === 'function') {
                window.initFieldHandling(elements);
                console.log('Field handling module initialized');
            } else {
                console.error('initFieldHandling function not available');
            }
        } catch (error) {
            console.error('Error initializing field handling module:', error);
        }
        
        // Initialize field templates
        try {
            document.querySelectorAll('.template').forEach(template => {
                template.draggable = true;
                template.addEventListener('dragstart', function(e) {
                    e.dataTransfer.setData('text/plain', template.id);
                });
            });
            
            if (typeof window.updateFieldAvailability === 'function') {
                window.updateFieldAvailability();
                console.log('Field templates initialized');
            } else {
                console.error('updateFieldAvailability function not available');
            }
        } catch (error) {
            console.error('Error initializing field templates:', error);
        }
        
        // Modify file input handler for document hashing
        try {
            if (elements.fileInput) {
                const originalChangeHandler = elements.fileInput.onchange;
                elements.fileInput.onchange = async function(e) {
                    if (this.files && this.files.length > 0) {
                        const file = this.files[0];
                        
                        // Generate and store document hash
                        try {
                            if (window.eidasCompliance && window.eidasCompliance.generateDocumentHash) {
                                const hash = await window.eidasCompliance.generateDocumentHash(file);
                                window.appState.documentHash = hash;
                                console.log('Document hash generated:', hash);
                            }
                        } catch (error) {
                            console.error('Error generating document hash:', error);
                        }
                        
                        // Call original handler if it exists
                        if (typeof originalChangeHandler === 'function') {
                            originalChangeHandler.call(this, e);
                        } else if (typeof window.handleFile === 'function') {
                            window.handleFile(file);
                        }
                    }
                };
                console.log('File input handler modified for document hashing');
            }
        } catch (error) {
            console.error('Error modifying file input handler:', error);
        }
    }
    
    /**
     * Initialize page navigation controls for multi-page documents
     */
    function initPageNavigation() {
        if (!elements.pageNavPrev || !elements.pageNavNext || !elements.pageNavIndicator) {
            console.warn('Page navigation elements not found');
            return;
        }
        
        elements.pageNavPrev.addEventListener('click', function() {
            navigatePages('prev');
        });
        
        elements.pageNavNext.addEventListener('click', function() {
            navigatePages('next');
        });
        
        // Initial state
        updatePageNavigation();
    }
    
    /**
     * Navigate between pages in the document preview
     */
    function navigatePages(direction) {
        const container = elements.finalPreviewContainer;
        if (!container) return;
        
        const pages = container.querySelectorAll('.document-page');
        if (pages.length <= 1) return;
        
        const currentPageNum = getVisiblePageNum();
        let targetPageNum;
        
        if (direction === 'prev') {
            targetPageNum = Math.max(1, currentPageNum - 1);
        } else {
            targetPageNum = Math.min(pages.length, currentPageNum + 1);
        }
        
        if (targetPageNum !== currentPageNum) {
            const targetPage = container.querySelector(`.document-page[data-page-num="${targetPageNum}"]`);
            if (targetPage) {
                container.scrollTo({
                    top: targetPage.offsetTop - container.offsetTop,
                    behavior: 'smooth'
                });
                
                // Update page navigation state
                updatePageNavigation(targetPageNum);
            }
        }
    }
    
    /**
     * Get the currently visible page number
     * @returns {number} Current page number (1-based)
     */
    function getVisiblePageNum() {
        const container = elements.finalPreviewContainer;
        if (!container) return 1;
        
        const pages = container.querySelectorAll('.document-page');
        for (let i = 0; i < pages.length; i++) {
            if (pages[i].style.display !== 'none') {
                return parseInt(pages[i].dataset.pageNum) || (i + 1);
            }
        }
        return 1; // Default to first page if none found
    }
    
    /**
     * Update page navigation controls based on current state
     */
    function updatePageNavigation(currentPage) {
        const container = elements.finalPreviewContainer;
        if (!container) return;
        
        const pages = container.querySelectorAll('.document-page');
        if (pages.length <= 0) return;
        
        const currentPageNum = currentPage || getVisiblePageNum();
        const totalPages = pages.length;
        
        // Update page indicator
        if (elements.pageNavIndicator) {
            elements.pageNavIndicator.textContent = `Page ${currentPageNum} of ${totalPages}`;
        }
        
        // Update button states
        if (elements.pageNavPrev) {
            elements.pageNavPrev.disabled = currentPageNum <= 1;
        }
        
        if (elements.pageNavNext) {
            elements.pageNavNext.disabled = currentPageNum >= totalPages;
        }
    }
    
    /**
     * Initialize image upload with better error handling
     */
    function initImageUpload() {
        // Check if elements are provided
        if (!elements) {
            console.warn('Elements object not provided to initImageUpload');
            return;
        }
        
        // Check if document image element exists
        if (!elements.documentImage) {
            console.warn('Image upload elements not found - documentImage element missing');
            // Try to find the element by ID directly
            const documentImage = document.getElementById('document-image');
            if (documentImage) {
                elements.documentImage = documentImage;
            } else {
                // Create a new file input if none exists
                const newFileInput = document.createElement('input');
                newFileInput.type = 'file';
                newFileInput.id = 'document-image';
                newFileInput.accept = 'image/*';
                newFileInput.style.display = 'none';
                document.body.appendChild(newFileInput);
                elements.documentImage = newFileInput;
                console.log('Created new document image input element');
            }
        }
        
        console.log('Setting up image upload...');
        
        // Set up file input styles
        elements.documentImage.style.position = "absolute";
        elements.documentImage.style.width = "100%";
        elements.documentImage.style.height = "100%";
        elements.documentImage.style.top = "0";
        elements.documentImage.style.left = "0";
        elements.documentImage.style.opacity = "0";
        elements.documentImage.style.cursor = "pointer";
        elements.documentImage.style.zIndex = "10";
        
        // Handle image selection
        elements.documentImage.addEventListener('change', function() {
            if (this.files && this.files.length > 0) {
                handleImageSelection(this.files[0]);
            }
        });
        
        // If image drop area doesn't exist, look for it or create it
        if (!elements.imageDropArea) {
            elements.imageDropArea = document.getElementById('image-drop-area');
            if (!elements.imageDropArea && elements.documentImage.parentNode) {
                // Create a drop area for the image
                const imageDropArea = document.createElement('div');
                imageDropArea.id = 'image-drop-area';
                imageDropArea.className = 'file-input-container';
                imageDropArea.innerHTML = `
                    <div class="file-input-icon">🖼️</div>
                    <div class="file-input-text">Drag & drop your image here</div>
                    <div class="file-input-text">or click to browse files</div>
                `;
                
                // Replace parent with new drop area
                const parent = elements.documentImage.parentNode;
                const nextSibling = elements.documentImage.nextSibling;
                
                imageDropArea.appendChild(elements.documentImage);
                
                if (nextSibling) {
                    parent.insertBefore(imageDropArea, nextSibling);
                } else {
                    parent.appendChild(imageDropArea);
                }
                
                elements.imageDropArea = imageDropArea;
                console.log('Created new image drop area');
            }
        }
        
        // Set up drag and drop for image area if it exists
        if (elements.imageDropArea) {
            elements.imageDropArea.style.position = "relative";
            elements.imageDropArea.style.cursor = "pointer";
            
            // Fix for click handler
            elements.imageDropArea.onclick = function(e) {
                if (e.target !== elements.documentImage) {
                    e.preventDefault();
                    elements.documentImage.click();
                }
            };
            
            // Drag and drop handlers
            ['dragenter', 'dragover', 'dragleave', 'drop'].forEach(eventName => {
                elements.imageDropArea.addEventListener(eventName, function(e) {
                    e.preventDefault();
                    e.stopPropagation();
                    
                    if (eventName === 'dragenter' || eventName === 'dragover') {
                        this.classList.add('drag-active');
                    } else {
                        this.classList.remove('drag-active');
                    }
                });
            });
            
            elements.imageDropArea.addEventListener('drop', function(e) {
                this.classList.remove('drag-active');
                if (e.dataTransfer.files && e.dataTransfer.files.length > 0) {
                    const file = e.dataTransfer.files[0];
                    if (file.type.startsWith('image/')) {
                        handleImageSelection(file);
                    } else {
                        alert('Please select an image file (JPEG, PNG, etc.)');
                    }
                }
            });
        }
        
        console.log('Image upload initialization complete');
    }
    
    /**
     * Handle the uploaded image file
     */
    function handleImageFile(file) {
        // Validate file type
        if (!file.type.match('image.*')) {
            alert('Please upload an image file (JPEG, PNG, GIF, etc.)');
            return;
        }
        
        // Store the image in app state
        window.appState.uploadedImage = file;
        
        // Update UI
        if (elements.savedImage) {
            elements.savedImage.textContent = file.name;
        }
        
        // Preview the image
        if (elements.imagePreview) {
            const reader = new FileReader();
            reader.onload = function(e) {
                elements.imagePreview.src = e.target.result;
                elements.imagePreview.style.display = 'block';
                
                if (elements.imagePlaceholder) {
                    elements.imagePlaceholder.style.display = 'none';
                }
            };
            reader.readAsDataURL(file);
        }
        
        // Update field availability
        if (typeof window.updateFieldAvailability === 'function') {
            window.updateFieldAvailability();
        }
        
        console.log('Image file processed:', file.name);
    }

    /**
     * Initialize step navigation
     */
    function initStepNavigation() {
        console.log('Initializing step navigation');
        
        // Event listeners for step navigation
        if (elements.step1NextBtn) {
            elements.step1NextBtn.addEventListener('click', function() {
                if (!window.appState.uploadedFile) {
                    alert('Please upload a document first');
                    return;
                }
                
                const yourName = document.getElementById('yourName');
                if (yourName && !yourName.value) {
                    alert('Please enter your name');
                    return;
                }
                
                const yourEmail = document.getElementById('yourEmail');
                const userEmail = yourEmail ? yourEmail.value : '';
                if (!userEmail) {
                    alert('Please enter your email address');
                    return;
                }
                
                // Validate email format
                const emailRegex = /^[^\s@]+@[^\s@]+\.[^\s@]+$/;
                if (!emailRegex.test(userEmail)) {
                    alert('Please enter a valid email address');
                    return;
                }
                
                if (elements.userEmailDisplay) {
                    elements.userEmailDisplay.textContent = userEmail;
                }
                
                // Check GDPR consent
                const gdprCheckbox = document.getElementById('data-privacy-checkbox');
                if (gdprCheckbox && !gdprCheckbox.checked) {
                    alert('Please consent to data processing before continuing');
                    return;
                }
                
                goToStep(2);
            });
        }
        
        if (elements.step2BackBtn) elements.step2BackBtn.addEventListener('click', function() { goToStep(1); });
        if (elements.step2NextBtn) elements.step2NextBtn.addEventListener('click', function() {
            if (!window.appState.signatureImage) {
                alert('Please draw and save your signature');
                return;
            }
            goToStep(3);
        });
        
        if (elements.step3BackBtn) elements.step3BackBtn.addEventListener('click', function() { goToStep(2); });
        if (elements.step3NextBtn) elements.step3NextBtn.addEventListener('click', function() { 
            // Validate date range (between 2024 and 2100)
            if (elements.documentDate) {
                const dateValue = elements.documentDate.value;
                if (dateValue) {
                    const selectedDate = new Date(dateValue);
                    const currentYear = selectedDate.getFullYear();
                    
                    if (currentYear < 2024 || currentYear > 2100) {
                        alert('Please select a date between 2024 and 2100');
                        return;
                    }
                }
            }
            
            goToStep(4); 
        });
        
        if (elements.step4BackBtn) elements.step4BackBtn.addEventListener('click', function() { goToStep(3); });
        if (elements.step4NextBtn) elements.step4NextBtn.addEventListener('click', function() { goToStep(5); });
        
        if (elements.step5BackBtn) elements.step5BackBtn.addEventListener('click', function() { goToStep(4); });
        if (elements.step5NextBtn) elements.step5NextBtn.addEventListener('click', function() { goToStep(6); });
        
        if (elements.step6BackBtn) elements.step6BackBtn.addEventListener('click', function() { goToStep(5); });
        if (elements.step6NextBtn) elements.step6NextBtn.addEventListener('click', function() { goToStep(7); });
        
        if (elements.step7BackBtn) elements.step7BackBtn.addEventListener('click', function() { goToStep(6); });
        if (elements.signAndSendBtn) elements.signAndSendBtn.addEventListener('click', async function() {
            const signatureFields = window.appState.placedFields.filter(f => f.type === 'signature-field' && f.verified);
            if (signatureFields.length === 0) {
                alert('Please place and verify your signature field');
                return;
            }
            
            // Check if legal disclaimer is accepted
            const disclaimerCheckbox = document.getElementById('legal-disclaimer-checkbox');
            if (disclaimerCheckbox && !disclaimerCheckbox.checked) {
                alert('Please accept the legal disclaimer before signing');
                return;
            }
            
            // Check if OTP verification is complete
            if (!window.appState.otpVerified) {
                // For testing/demo purposes - skip OTP verification
                if (confirm('For demo purposes: Skip OTP verification?')) {
                    window.appState.otpVerified = true;
                } else {
                    alert('Please complete identity verification before signing');
                    return;
                }
            }
            
            try {
                // Generate electronic signature
                if (window.eidasCompliance) {
                    // Get user information
                    const userId = document.getElementById('yourEmail').value || 'unknown';
                    const documentHash = window.appState.documentHash;
                    
                    // Get trusted timestamp
                    const timestampData = await window.eidasCompliance.getTrustedTimestamp();
                    const timestamp = timestampData.timestamp;
                    
                    // Generate signature
                    const signature = await window.eidasCompliance.generateSignature(
                        userId, 
                        documentHash,
                        timestamp
                    );
                    
                    // Get IP address
                    const ipAddress = await window.eidasCompliance.getUserIP();
                    
                    // Create signature data object
                    const signatureData = {
                        userId,
                        documentHash,
                        signature,
                        timestamp,
                        timestampIso: timestampData.iso,
                        timestampSource: timestampData.source,
                        ipAddress
                    };
                    
                    // Store signature data
                    window.appState.signatureData = signatureData;
                    
                    // Create audit log
                    window.eidasCompliance.createAuditLog({
                        userId,
                        documentHash,
                        signature,
                        timestamp,
                        ipAddress,
                        action: 'document_signed'
                    });
                    
                    console.log('Document successfully signed with eIDAS compliance');
                    
                    // Prepare final document
                    renderFinalDocument();
                    
                    // Proceed to final step
                    goToStep(8);
                } else {
                    // Fallback if eIDAS module is not available
                    renderFinalDocument();
                    goToStep(8);
                }
            } catch (error) {
                console.error('Error during signing process:', error);
                alert('An error occurred during the signing process. Please try again.');
            }
        });
        
        if (elements.step8BackBtn) elements.step8BackBtn.addEventListener('click', function() { goToStep(7); });
        
        if (elements.downloadDocumentBtn) elements.downloadDocumentBtn.addEventListener('click', async function() {
            downloadFinalDocument();
        });
    }

    /**
     * Navigate between steps
     */
    function goToStep(stepNumber) {
        console.log(`Moving to step ${stepNumber}`);
        
        // Update app state
        window.appState.currentStep = stepNumber;
        
        // Update step visibility
        elements.steps.forEach(step => step.classList.remove('active'));
        const targetStep = document.getElementById('step' + stepNumber);
        if (targetStep) {
            targetStep.classList.add('active');
        } else {
            console.error(`Step ${stepNumber} element not found`);
            return;
        }
        
        // Update step indicators
        elements.stepIndicators.forEach(indicator => {
            indicator.classList.remove('active', 'completed');
        });
        
        for (let i = 1; i < stepNumber; i++) {
            const indicator = document.getElementById('step' + i + '-indicator');
            if (indicator) {
                indicator.classList.add('completed');
            }
        }
        
        const currentIndicator = document.getElementById('step' + stepNumber + '-indicator');
        if (currentIndicator) {
            currentIndicator.classList.add('active');
        }
        
        // Handle specific step transitions
        if (stepNumber === 7 && window.appState.uploadedFile) {
            console.log("Going to step 7, preparing document...");
            if (window.appState.pageImages && window.appState.pageImages.length > 0) {
                console.log("Using existing page images:", window.appState.pageImages.length);
                displayPages(window.appState.pageImages.map((src, index) => {
                    return {
                        pageNumber: index + 1,
                        dataUrl: src,
                        width: window.appState.pageDimensions[index]?.width || 800,
                        height: window.appState.pageDimensions[index]?.height || 1000
                    };
                }));
                
                // Initialize/update page navigation
                setTimeout(() => {
                    updatePageNavigation();
                }, 100);
            } else {
                console.warn('No page images available for Step 7, attempting to re-render');
                if (typeof window.handleFile === 'function' && window.appState.uploadedFile) {
                    window.handleFile(window.appState.uploadedFile);
                    
                    // Initialize/update page navigation after rendering
                    setTimeout(() => {
                        updatePageNavigation();
                    }, 500);
                } else {
                    console.error("handleFile function not available or no uploaded file");
                }
            }
        } else if (stepNumber === 8) {
            console.log("Going to step 8, preparing final document...");
            renderFinalDocument();
        }
    }
    
    /**
     * Render the final document with all fields
     */
    function renderFinalDocument() {
        if (!window.appState.pageImages || window.appState.pageImages.length === 0) {
            console.error('No page images available for final document');
            return;
        }
        
        console.log('Rendering final document with fields...');
        
        // Display pages in the download container
        displayPages(window.appState.pageImages.map((src, index) => {
            return {
                pageNumber: index + 1,
                dataUrl: src,
                width: window.appState.pageDimensions[index]?.width || 800,
                height: window.appState.pageDimensions[index]?.height || 1000
            };
        }), elements.downloadPreviewContainer);
        
        // Render fields on the document
        if (typeof window.renderFieldsOnDocument === 'function') {
            window.renderFieldsOnDocument(function() {
                console.log('Fields rendered on final document');
                
                // Add signature data to the displayed info
                if (window.appState.signatureData) {
                    addSignatureInfoToFinalStep();
                }
            });
        } else {
            console.error('renderFieldsOnDocument function not available');
            
            // Still add signature data even if fields couldn't be rendered
            if (window.appState.signatureData) {
                addSignatureInfoToFinalStep();
            }
        }
    }
    
    /**
     * Add signature information to the final step
     */
    function addSignatureInfoToFinalStep() {
        const data = window.appState.signatureData;
        if (!data) return;
        
        // Remove any existing signature info
        const existingInfo = document.querySelector('.signature-info-section');
        if (existingInfo) existingInfo.remove();
        
        // Create signature information div to show on the final step
        const signatureInfoDiv = document.createElement('div');
        signatureInfoDiv.className = 'signature-info-section';
        signatureInfoDiv.style.marginTop = '20px';
        
        signatureInfoDiv.innerHTML = `
            <h3>eIDAS Compliant Signature Information</h3>
            <table style="width: 100%; border-collapse: collapse;">
                <tr>
                    <td style="padding: 8px; border-bottom: 1px solid #ddd;"><strong>Signer:</strong></td>
                    <td style="padding: 8px; border-bottom: 1px solid #ddd;">${data.userId}</td>
                </tr>
                <tr>
                    <td style="padding: 8px; border-bottom: 1px solid #ddd;"><strong>Timestamp:</strong></td>
                    <td style="padding: 8px; border-bottom: 1px solid #ddd;">${new Date(data.timestamp * 1000).toLocaleString()}</td>
                </tr>
                <tr>
                    <td style="padding: 8px; border-bottom: 1px solid #ddd;"><strong>Document Hash:</strong></td>
                    <td style="padding: 8px; border-bottom: 1px solid #ddd;"><code>${data.documentHash}</code></td>
                </tr>
                <tr>
                    <td style="padding: 8px; border-bottom: 1px solid #ddd;"><strong>Signature:</strong></td>
                    <td style="padding: 8px; border-bottom: 1px solid #ddd;"><code>${data.signature.substring(0, 20)}...</code></td>
                </tr>
                <tr>
                    <td style="padding: 8px;"><strong>Verification Status:</strong></td>
                    <td style="padding: 8px; color: green;"><strong>✓ Verified</strong></td>
                </tr>
            </table>
            <p style="margin-top: 10px; color: #666; font-size: 12px;">
                This signature is compliant with eIDAS regulations for Simple Electronic Signatures (SES).
                A cryptographic record of this signature has been stored in the audit log.
            </p>
        `;
        
        // Add to the page
        const downloadContainer = document.getElementById('step8');
        if (downloadContainer) {
            const card = downloadContainer.querySelector('.card');
            if (card) {
                card.appendChild(signatureInfoDiv);
            }
        }
    }
    
    /**
     * Download the final document as PDF
     */
    function downloadFinalDocument() {
        const canvases = elements.downloadPreviewContainer.querySelectorAll('.preview-canvas');
        if (canvases.length > 0) {
            try {
                const { jsPDF } = window.jspdf;
                const doc = new jsPDF({
                    orientation: 'portrait',
                    unit: 'px',
                    format: [canvases[0].width, canvases[0].height]
                });
                
                // First create the PDF normally
                canvases.forEach((canvas, index) => {
                    const imgData = canvas.toDataURL('image/png');
                    if (index > 0) doc.addPage();
                    doc.addImage(imgData, 'PNG', 0, 0, canvas.width, canvas.height);
                });
                
                // Add signature information footer to the last page
                if (window.appState.signatureData) {
                    const data = window.appState.signatureData;
                    
                    // Add to the last page
                    doc.setFontSize(8);
                    doc.setTextColor(100, 100, 100);
                    
                    // Add signature footer
                    const footerY = doc.internal.pageSize.getHeight() - 20;
                    doc.text(`Document signed electronically by ${data.userId}`, 10, footerY);
                    doc.text(`Signature: ${data.signature.substring(0, 20)}...`, 10, footerY + 5);
                    doc.text(`Timestamp: ${new Date(data.timestamp * 1000).toISOString()}`, 10, footerY + 10);
                    doc.text(`Document Hash: ${data.documentHash.substring(0, 20)}...`, 10, footerY + 15);
                }
                
                const fileName = window.appState.uploadedFile ? 
                    `signed_${window.appState.uploadedFile.name.replace(/\.[^/.]+$/, '')}.pdf` : 
                    'signed_document.pdf';
                
                doc.save(fileName);
                
                // Create audit log for download
                if (window.eidasCompliance && window.appState.signatureData) {
                    window.eidasCompliance.createAuditLog({
                        ...window.appState.signatureData,
                        action: 'document_downloaded'
                    });
                }
                
                console.log('Document downloaded successfully');
            } catch (error) {
                console.error('Error generating PDF:', error);
                alert('Error generating PDF. Please make sure jsPDF is loaded correctly.');
            }
        } else {
            alert('No document available for download. Please ensure the document is rendered correctly.');
            console.error('No canvases found in downloadPreviewContainer');
        }
    }
    
    // Handle template download if applicable
    if (elements.downloadTemplateLink) {
        elements.downloadTemplateLink.addEventListener('click', function(e) {
            e.preventDefault();
            
            // Generate a simple contract template PDF
            try {
                const { jsPDF } = window.jspdf;
                const doc = new jsPDF();
                
                doc.setFontSize(22);
                doc.text('Inkless-Flow Contract Template', 105, 20, { align: 'center' });
                
                doc.setFontSize(12);
                doc.text('This is a sample contract document template.', 20, 40);
                doc.text('Please fill in the following information:', 20, 50);
                
                doc.text('Party A: ________________________________', 20, 70);
                doc.text('Party B: ________________________________', 20, 80);
                
                doc.text('Date: ____________________', 20, 100);
                
                doc.text('Terms and Conditions:', 20, 120);
                doc.text('1. Lorem ipsum dolor sit amet, consectetur adipiscing elit.', 30, 130);
                doc.text('2. Sed do eiusmod tempor incididunt ut labore et dolore magna aliqua.', 30, 140);
                doc.text('3. Ut enim ad minim veniam, quis nostrud exercitation ullamco laboris.', 30, 150);
                
                doc.text('Signatures:', 20, 180);
                doc.text('Party A: ________________________________', 20, 200);
                doc.text('Party B: ________________________________', 20, 220);
                
                doc.text('Created with Inkless-Flow eSignature Platform', 105, 280, { align: 'center' });
                
                doc.save('inkless_flow_contract_template.pdf');
                console.log('Template downloaded successfully');
            } catch (error) {
                console.error('Error generating template PDF:', error);
                alert('Error generating template PDF. Please make sure jsPDF is loaded correctly.');
            }
        });
    }
    
    /**
     * Display pages in the preview containers
     * @param {Array} pages Array of page objects with dataUrl, pageNumber, width, height
     * @param {HTMLElement} specificContainer Optional specific container to update
     */
    function displayPages(pages, specificContainer) {
        console.log('Displaying pages:', pages.length);
        
        const previewContainers = specificContainer ? 
            [specificContainer] : 
            [
                window.elements.filePreviewContainer,
                window.elements.finalPreviewContainer,
                window.elements.downloadPreviewContainer
            ].filter(container => container);
        
        previewContainers.forEach(container => {
            if (!container) return;
            
            console.log('Updating container:', container.id || 'unnamed');
            container.innerHTML = '';
            
            // Sort pages by page number
            pages.sort((a, b) => a.pageNumber - b.pageNumber);
            
            // Create a wrapper div for all pages
            const wrapper = document.createElement('div');
            wrapper.className = 'document-pages';
            wrapper.style.display = 'flex';
            wrapper.style.flexDirection = 'column';
            wrapper.style.gap = '20px';
            container.appendChild(wrapper);
            
            // Process each page
            pages.forEach(page => {
                const pageDiv = document.createElement('div');
                pageDiv.className = 'document-page';
                pageDiv.dataset.pageNum = page.pageNumber;
                pageDiv.style.position = 'relative';
                pageDiv.style.marginBottom = '20px';
                pageDiv.style.border = '1px solid #ddd';
                pageDiv.style.backgroundColor = '#fff';
                pageDiv.style.boxShadow = '0 2px 4px rgba(0,0,0,0.1)';
                
                // Create canvas for the page
                const canvas = document.createElement('canvas');
                canvas.className = 'preview-canvas';
                canvas.width = page.width;
                canvas.height = page.height;
                canvas.style.maxWidth = '100%';
                canvas.style.height = 'auto';
                canvas.style.display = 'block';
                
                // Get the 2D context
                const ctx = canvas.getContext('2d');
                
                // Draw the page image
                ctx.drawImage(page.dataUrl, 0, 0, canvas.width, canvas.height);
                
                // Add the canvas to the page div
                pageDiv.appendChild(canvas);
                
                // Add page number indicator
                const pageNumIndicator = document.createElement('div');
                pageNumIndicator.className = 'page-number';
                pageNumIndicator.textContent = `Page ${page.pageNumber}`;
                pageNumIndicator.style.position = 'absolute';
                pageNumIndicator.style.top = '10px';
                pageNumIndicator.style.right = '10px';
                pageNumIndicator.style.backgroundColor = 'rgba(0,0,0,0.5)';
                pageNumIndicator.style.color = '#fff';
                pageNumIndicator.style.padding = '4px 8px';
                pageNumIndicator.style.borderRadius = '4px';
                pageNumIndicator.style.fontSize = '12px';
                pageDiv.appendChild(pageNumIndicator);
                
                // Add the page div to the wrapper
                wrapper.appendChild(pageDiv);
                
                console.log(`Page ${page.pageNumber} drawn on canvas in ${container.id || 'unnamed'}`);
            });
            
            // If this is the final preview container, restore any placed fields
            if (container === window.elements.finalPreviewContainer) {
                restorePlacedFields();
            }
        });
        
        // Update page navigation if needed
        if (typeof updatePageNavigation === 'function') {
            updatePageNavigation();
        }
    }
    
    /**
     * Restore previously placed fields on the document
     */
    function restorePlacedFields() {
        if (!window.appState.placedFields || window.appState.placedFields.length === 0) {
            return;
        }
        
        const container = window.elements.finalPreviewContainer;
        if (!container) return;
        
        window.appState.placedFields.forEach(field => {
            const pageDiv = container.querySelector(`.document-page[data-page-num="${field.pageNum}"]`);
            if (!pageDiv) return;
            
            const overlay = pageDiv.querySelector('.field-overlay');
            if (!overlay) return;
            
            // Check if field already exists (avoid duplicates)
            if (overlay.querySelector(`#${field.id}`)) return;
            
            // Create field element
            const fieldElement = document.createElement('div');
            fieldElement.id = field.id;
            fieldElement.className = `field-container ${field.verified ? 'verified' : 'draft'}`;
            fieldElement.style.position = 'absolute';
            fieldElement.style.left = field.left + 'px';
            fieldElement.style.top = field.top + 'px';
            fieldElement.style.width = field.width + 'px';
            fieldElement.style.height = field.height + 'px';
            
            // Get custom data if needed
            let customData = null;
            if (field.type === 'custom-text-field' && field.customData) {
                customData = field.customData;
            }
            
            // Add content
            fieldElement.innerHTML = `
                <div class="field-label">${window.getFieldLabel ? window.getFieldLabel(field.type, customData) : field.type}</div>
                <div class="field-content">${window.getFieldContent ? window.getFieldContent(field.type, customData) : ''}</div>
                <div class="field-controls">
                    ${field.verified ? 
                        '<button class="field-control-btn delete-btn" title="Delete field">×</button>' : 
                        '<button class="field-control-btn place-btn" title="Confirm placement">✓</button>' +
                        '<button class="field-control-btn remove-btn" title="Remove field">×</button>'
                    }
                </div>
            `;
            
            // Add event listeners
            if (field.verified) {
                fieldElement.querySelector('.delete-btn').addEventListener('click', function() {
                    if (typeof window.removeField === 'function') {
                        window.removeField(fieldElement);
                    } else {
                        fieldElement.remove();
                    }
                });
            } else {
                // Add place and remove buttons functionality
                fieldElement.querySelector('.place-btn').addEventListener('click', function() {
                    if (typeof window.placeField === 'function') {
                        window.placeField(fieldElement);
                    }
                });
                
                fieldElement.querySelector('.remove-btn').addEventListener('click', function() {
                    if (typeof window.removeField === 'function') {
                        window.removeField(fieldElement);
                    } else {
                        fieldElement.remove();
                    }
                });
                
                // Make the field draggable and resizable
                if (typeof window.makeDraggable === 'function') {
                    window.makeDraggable(fieldElement);
                }
                
                if (typeof window.makeResizable === 'function') {
                    window.makeResizable(fieldElement);
                }
            }
            
            // Append the field to the overlay
            overlay.appendChild(fieldElement);
        });
    }
    
    // Expose globally needed functions
    window.displayPages = displayPages;
    window.goToStep = goToStep;
    window.renderFinalDocument = renderFinalDocument;
    window.handleImageFile = handleImageFile;
;