/**
 * Document Rendering Module
 * Handles final document rendering, fields placement, and exporting
 */

/**
 * Initialize document rendering module
 */
function initDocumentRendering() {
    console.log('Initializing document rendering module...');
    
    // Initialize state
    window.appState = window.appState || {};
    window.appState.placedFields = window.appState.placedFields || [];
    window.appState.pageDimensions = window.appState.pageDimensions || [];
    window.appState.pageImages = window.appState.pageImages || [];
    
    // Set up event listeners for field interactions
    document.addEventListener('fieldAdded', function(e) {
        const field = e.detail;
        window.appState.placedFields.push(field);
        console.log('Field added:', field);
    });
    
    document.addEventListener('fieldRemoved', function(e) {
        const fieldId = e.detail;
        window.appState.placedFields = window.appState.placedFields.filter(f => f.id !== fieldId);
        console.log('Field removed:', fieldId);
    });
    
    document.addEventListener('fieldUpdated', function(e) {
        const updatedField = e.detail;
        const index = window.appState.placedFields.findIndex(f => f.id === updatedField.id);
        if (index !== -1) {
            window.appState.placedFields[index] = updatedField;
            console.log('Field updated:', updatedField);
        }
    });
    
    console.log('Document rendering module initialized');
}

function renderFinalDocument() {
    console.log('Rendering final document with fields...');
    const elements = window.elements;
    
    // Clear the download preview container
    if (elements.downloadPreviewContainer) {
        elements.downloadPreviewContainer.innerHTML = '';
    } else {
        console.error('Download preview container not found');
        return;
    }
    
    // Get all page divs from the final preview container
    const pageDivs = elements.finalPreviewContainer.querySelectorAll('.document-page');
    
    if (pageDivs.length === 0) {
        console.error('No pages to render in final document');
        return;
    }
    
    // Process each page
    pageDivs.forEach(pageDiv => {
        const pageNum = parseInt(pageDiv.dataset.pageNum);
        const dims = window.appState.pageDimensions[pageNum - 1];
        
        if (!dims) {
            console.error(`No dimensions found for page ${pageNum}`);
            return;
        }
        
        // Create a new canvas for the final document
        const newCanvas = document.createElement('canvas');
        newCanvas.className = 'preview-canvas';
        newCanvas.width = dims.width;
        newCanvas.height = dims.height;
        newCanvas.style.maxWidth = '100%';
        
        // Get the 2D context for drawing
        const ctx = newCanvas.getContext('2d');
        
        // Find the original canvas and draw it onto the new canvas
        const originalCanvas = pageDiv.querySelector('.preview-canvas');
        
        if (originalCanvas) {
            // Draw the original page content
            ctx.drawImage(originalCanvas, 0, 0, newCanvas.width, newCanvas.height);
            
            // Draw the fields on the canvas
            const fieldsToRender = window.appState.placedFields.filter(field => 
                field.pageNum === pageNum);
            
            fieldsToRender.forEach(field => {
                drawFieldContentTransparent(ctx, field, field.left, field.top, field.width, field.height);
            });
            
            // Add the canvas to the download preview container
            elements.downloadPreviewContainer.appendChild(newCanvas);
        } else {
            console.error(`No original canvas found for page ${pageNum}`);
        }
    });
    
    // Add a message indicating processing is complete
    const completeMsg = document.createElement('div');
    completeMsg.className = 'alert alert-success';
    completeMsg.style.marginTop = '20px';
    completeMsg.innerHTML = '<strong>Document processing complete!</strong> You can now download your signed document.';
    
    elements.downloadPreviewContainer.appendChild(completeMsg);
}

function renderFieldsOnCanvas(ctx, pageNum, width, height) {
    // Get all verified fields for this page
    const fieldsToRender = window.appState.placedFields.filter(field => 
        field.pageNum === pageNum && field.verified);
    
    // Draw each field
    fieldsToRender.forEach(fieldData => {
        const x = fieldData.left;
        const y = fieldData.top;
        const w = fieldData.width;
        const h = fieldData.height;
        
        // Draw field content without background or borders
        drawFieldContentTransparent(ctx, fieldData, x, y, w, h);
    });
    
    // Draw timestamp at the bottom of the page
    drawTimestamp(ctx, width, height);
}

function drawFieldContentTransparent(ctx, fieldData, x, y, width, height) {
    const padding = 0;
    const innerX = x + padding;
    const innerY = y + padding;
    const innerWidth = width - (padding * 2);
    const innerHeight = height - (padding * 2);
    
    // Draw content based on field type
    switch (fieldData.type) {
        case 'signature-field':
            if (window.appState.signatureImage) {
                const img = new Image();
                img.onload = function() {
                    // Calculate aspect ratio
                    const aspectRatio = img.width / img.height;
                    let drawWidth = innerWidth;
                    let drawHeight = drawWidth / aspectRatio;
                    
                    // Adjust if too tall
                    if (drawHeight > innerHeight) {
                        drawHeight = innerHeight;
                        drawWidth = drawHeight * aspectRatio;
                    }
                    
                    // Center the signature in the field
                    const centerX = innerX + (innerWidth - drawWidth) / 2;
                    const centerY = innerY + (innerHeight - drawHeight) / 2;
                    
                    ctx.drawImage(img, centerX, centerY, drawWidth, drawHeight);
                };
                img.src = window.appState.signatureImage;
            }
            break;
            
        case 'date-field':
            if (window.appState.selectedDate) {
                // Format the date
                const dateObj = new Date(window.appState.selectedDate);
                const dateStr = dateObj.toLocaleDateString();
                
                // Draw date
                ctx.fillStyle = '#000000';
                ctx.font = '14px Arial';
                ctx.textAlign = 'center';
                ctx.fillText(dateStr, innerX + innerWidth/2, innerY + innerHeight/2);
                ctx.textAlign = 'left';
            }
            break;
            
        case 'custom-text-field':
            if (fieldData.customData) {
                // Draw text
                ctx.fillStyle = '#000000';
                ctx.font = '14px Arial';
                
                // Handle text wrapping
                const text = fieldData.customData.content;
                const words = text.split(' ');
                let line = '';
                let lineY = innerY + 16;
                
                for (let i = 0; i < words.length; i++) {
                    const testLine = line + words[i] + ' ';
                    const metrics = ctx.measureText(testLine);
                    
                    if (metrics.width > innerWidth && i > 0) {
                        ctx.fillText(line, innerX, lineY);
                        line = words[i] + ' ';
                        lineY += 16;
                        
                        // Check if we're running out of space
                        if (lineY > innerY + innerHeight) {
                            ctx.fillText('...', innerX, lineY);
                            break;
                        }
                    } else {
                        line = testLine;
                    }
                }
                
                // Draw the last line
                if (lineY <= innerY + innerHeight) {
                    ctx.fillText(line, innerX, lineY);
                }
            }
            break;
            
        case 'image-field':
            if (window.appState.uploadedImage) {
                const reader = new FileReader();
                reader.onload = function(e) {
                    const img = new Image();
                    img.onload = function() {
                        // Calculate aspect ratio
                        const aspectRatio = img.width / img.height;
                        let drawWidth = innerWidth;
                        let drawHeight = drawWidth / aspectRatio;
                        
                        // Adjust if too tall
                        if (drawHeight > innerHeight) {
                            drawHeight = innerHeight;
                            drawWidth = drawHeight * aspectRatio;
                        }
                        
                        // Center the image in the field
                        const centerX = innerX + (innerWidth - drawWidth) / 2;
                        const centerY = innerY + (innerHeight - drawHeight) / 2;
                        
                        ctx.drawImage(img, centerX, centerY, drawWidth, drawHeight);
                    };
                    img.src = e.target.result;
                };
                reader.readAsDataURL(window.appState.uploadedImage);
            }
            break;
            
        case 'recipient-field':
            const recipient = window.appState.recipients.length > 0 ? 
                window.appState.recipients[0].name : 'Recipient';
            
            ctx.fillStyle = '#000000';
            ctx.font = '14px Arial';
            ctx.textAlign = 'center';
            ctx.fillText(recipient, innerX + innerWidth/2, innerY + innerHeight/2);
            ctx.textAlign = 'left';
            break;
    }
}

function drawTimestamp(ctx, width, height) {
    // Get current timestamp
    const now = new Date();
    const dateStr = now.toLocaleDateString();
    const timeStr = now.toLocaleTimeString();
    
    // Draw timestamp at the bottom of the page
    ctx.fillStyle = '#999999';
    ctx.font = '10px Arial';
    ctx.textAlign = 'right';
    
    // Create timestamp text
    const timestampText = `Generated by Inkless-Flow: ${dateStr} ${timeStr}`;
    
    // Draw text at the bottom right
    ctx.fillText(timestampText, width - 10, height - 10);
    
    // Reset text alignment
    ctx.textAlign = 'left';
}

// Add function to create a download URL for a PDF
function createPdfDownloadUrl() {
    console.log('Creating PDF download URL...');
    
    // Return a promise that resolves with the URL
    return new Promise((resolve, reject) => {
        try {
            // Code for PDF generation would go here
            resolve('#');
        } catch (error) {
            reject(error);
        }
    });
}

// Export functions for use in other modules
window.documentRendering = {
    init: initDocumentRendering,
    renderFinalDocument,
    renderFieldsOnCanvas,
    drawFieldContentTransparent,
    drawTimestamp,
    createPdfDownloadUrl
};

// Also expose key functions globally
window.initDocumentRendering = initDocumentRendering;
window.renderFinalDocument = renderFinalDocument;
window.renderFieldsOnCanvas = renderFieldsOnCanvas;
window.drawFieldsOnCanvas = renderFieldsOnCanvas;