/**
 * eIDAS Compatibility Module for Inkless-Flow
 * Connects the frontend to the eIDAS-compliant backend
 */

// Initialize the eIDAS compliance module
window.eidasCompliance = {
    baseUrl: '/inklessflow-3',
    currentUser: null,
    documentHash: null,
    otpVerified: false,
    
    /**
     * Initialize the module
     */
    init: function() {
        console.log('Initializing eIDAS compliance module...');
        this.currentUser = {
            email: document.getElementById('yourEmail')?.value || null
        };
        
        // Add event listeners for legal disclaimer checkbox
        const disclaimerCheckbox = document.getElementById('legal-disclaimer-checkbox');
        if (disclaimerCheckbox) {
            disclaimerCheckbox.addEventListener('change', function() {
                const signBtn = document.getElementById('sign-and-send');
                if (signBtn && signBtn.classList.contains('fields-ready')) {
                    signBtn.disabled = !this.checked || !window.eidasCompliance.otpVerified;
                }
            });
        }
        
        // Add event listeners for OTP request and verification
        this.setupOtpHandlers();
    },
    
    /**
     * Set up OTP request and verification handlers
     */
    setupOtpHandlers: function() {
        const requestOtpBtn = document.getElementById('request-otp-btn');
        const verifyOtpBtn = document.getElementById('verify-otp-btn');
        const otpInput = document.getElementById('otp-input');
        
        if (requestOtpBtn) {
            requestOtpBtn.addEventListener('click', async function() {
                await window.eidasCompliance.requestOTP();
            });
        }
        
        if (verifyOtpBtn && otpInput) {
            verifyOtpBtn.addEventListener('click', async function() {
                const otp = otpInput.value.trim();
                if (otp.length === 6) {
                    await window.eidasCompliance.verifyOTP(otp);
                } else {
                    alert('Vă rugăm să introduceți codul OTP din 6 cifre primit prin email.');
                }
            });
            
            // Enable pressing Enter to verify OTP
            otpInput.addEventListener('keyup', function(e) {
                if (e.key === 'Enter') {
                    verifyOtpBtn.click();
                }
            });
        }
    },
    
    /**
     * Generate a document hash from a file
     * 
     * @param {File} file Document file object
     * @returns {Promise<string>} Document hash
     */
    generateDocumentHash: async function(file) {
        try {
            // For security, we rely on the server-side hash generation
            // This function is a placeholder for the frontend to track the process
            
            console.log('Document hash will be generated on the server.');
            
            // We'll return a temporary hash or null
            // The real hash will be returned by the upload API
            return null;
        } catch (error) {
            console.error('Error generating document hash:', error);
            throw error;
        }
    },
    
    /**
     * Upload a document to the server
     * 
     * @param {File} file Document file object
     * @param {string} email User email
     * @returns {Promise<object>} Upload result
     */
    uploadDocument: async function(file, email) {
        try {
            const formData = new FormData();
            formData.append('document', file);
            formData.append('email', email);
            
            const response = await fetch(this.baseUrl + '/upload-document.php', {
                method: 'POST',
                body: formData
            });
            
            const result = await response.json();
            
            if (result.success) {
                this.documentHash = result.document_id;
                console.log('Document uploaded successfully with hash:', this.documentHash);
            } else {
                console.error('Document upload failed:', result.message);
            }
            
            return result;
        } catch (error) {
            console.error('Error uploading document:', error);
            throw error;
        }
    },
    
    /**
     * Request an OTP for verification
     * 
     * @returns {Promise<object>} OTP request result
     */
    requestOTP: async function() {
        try {
            const email = this.currentUser?.email || document.getElementById('yourEmail')?.value;
            
            if (!email) {
                alert('Vă rugăm să introduceți adresa de email.');
                return { success: false, message: 'Email not provided' };
            }
            
            // Show loading state
            const requestOtpBtn = document.getElementById('request-otp-btn');
            if (requestOtpBtn) {
                requestOtpBtn.disabled = true;
                requestOtpBtn.textContent = 'Se trimite...';
            }
            
            const data = {
                email: email,
                document_id: this.documentHash
            };
            
            const response = await fetch(this.baseUrl + '/send-otp.php', {
                method: 'POST',
                headers: {
                    'Content-Type': 'application/json'
                },
                body: JSON.stringify(data)
            });
            
            const result = await response.json();
            
            // Restore button state
            if (requestOtpBtn) {
                requestOtpBtn.disabled = false;
                requestOtpBtn.textContent = 'Solicită cod OTP';
            }
            
            if (result.success) {
                // Show OTP input field
                const otpSection = document.getElementById('otp-verification-section');
                if (otpSection) {
                    otpSection.style.display = 'block';
                }
                
                alert('Codul OTP a fost trimis la adresa de email ' + email);
                
                // Focus the OTP input field
                const otpInput = document.getElementById('otp-input');
                if (otpInput) {
                    otpInput.focus();
                }
            } else {
                alert('Eroare la trimiterea codului OTP: ' + result.message);
            }
            
            return result;
        } catch (error) {
            console.error('Error requesting OTP:', error);
            
            // Restore button state
            const requestOtpBtn = document.getElementById('request-otp-btn');
            if (requestOtpBtn) {
                requestOtpBtn.disabled = false;
                requestOtpBtn.textContent = 'Solicită cod OTP';
            }
            
            alert('Eroare la trimiterea codului OTP. Vă rugăm să încercați din nou.');
            throw error;
        }
    },
    
    /**
     * Verify an OTP
     * 
     * @param {string} otp OTP code
     * @returns {Promise<object>} Verification result
     */
    verifyOTP: async function(otp) {
        try {
            const email = this.currentUser?.email || document.getElementById('yourEmail')?.value;
            
            if (!email) {
                alert('Vă rugăm să introduceți adresa de email.');
                return { success: false, message: 'Email not provided' };
            }
            
            // Show loading state
            const verifyOtpBtn = document.getElementById('verify-otp-btn');
            if (verifyOtpBtn) {
                verifyOtpBtn.disabled = true;
                verifyOtpBtn.textContent = 'Se verifică...';
            }
            
            const data = {
                email: email,
                otp: otp,
                document_id: this.documentHash
            };
            
            const response = await fetch(this.baseUrl + '/verify-otp.php', {
                method: 'POST',
                headers: {
                    'Content-Type': 'application/json'
                },
                body: JSON.stringify(data)
            });
            
            const result = await response.json();
            
            // Restore button state
            if (verifyOtpBtn) {
                verifyOtpBtn.disabled = false;
                verifyOtpBtn.textContent = 'Verifică cod OTP';
            }
            
            if (result.success) {
                // Update verification status
                this.otpVerified = true;
                window.appState.otpVerified = true;
                
                // Update UI
                const verificationStatus = document.getElementById('verification-status');
                if (verificationStatus) {
                    verificationStatus.textContent = 'Verificat';
                    verificationStatus.className = 'text-success';
                }
                
                // Hide OTP section
                const otpSection = document.getElementById('otp-verification-section');
                if (otpSection) {
                    otpSection.innerHTML = '<div class="alert alert-success">Identitate verificată cu succes!</div>';
                }
                
                // Enable sign button if legal disclaimer is checked
                const signBtn = document.getElementById('sign-and-send');
                const disclaimerCheckbox = document.getElementById('legal-disclaimer-checkbox');
                if (signBtn && disclaimerCheckbox && disclaimerCheckbox.checked) {
                    signBtn.disabled = false;
                }
                
                alert('Codul OTP a fost verificat cu succes!');
            } else {
                alert('Eroare la verificarea codului OTP: ' + result.message);
            }
            
            return result;
        } catch (error) {
            console.error('Error verifying OTP:', error);
            
            // Restore button state
            const verifyOtpBtn = document.getElementById('verify-otp-btn');
            if (verifyOtpBtn) {
                verifyOtpBtn.disabled = false;
                verifyOtpBtn.textContent = 'Verifică cod OTP';
            }
            
            alert('Eroare la verificarea codului OTP. Vă rugăm să încercați din nou.');
            throw error;
        }
    },
    
    /**
     * Get a trusted timestamp from a public source
     * 
     * @returns {Promise<object>} Timestamp data
     */
    getTrustedTimestamp: async function() {
        try {
            // Call worldtimeapi.org for a trusted timestamp
            const response = await fetch('https://worldtimeapi.org/api/ip');
            const data = await response.json();
            
            return {
                timestamp: data.unixtime,
                iso: data.datetime,
                source: 'worldtimeapi.org'
            };
        } catch (error) {
            console.error('Error getting trusted timestamp:', error);
            
            // Fallback to local time
            const now = new Date();
            return {
                timestamp: Math.floor(now.getTime() / 1000),
                iso: now.toISOString(),
                source: 'local_time'
            };
        }
    },
    
    /**
     * Generate a signature for a document
     * 
     * @param {string} userId User ID (email)
     * @param {string} documentHash Document hash
     * @param {number} timestamp Trusted timestamp
     * @returns {Promise<string>} Generated signature
     */
    generateSignature: async function(userId, documentHash, timestamp) {
        try {
            // For security, we rely on the server-side signature generation
            // Return a placeholder until the server generates the real signature
            return 'pending_' + Math.random().toString(36).substring(2, 15);
        } catch (error) {
            console.error('Error generating signature:', error);
            throw error;
        }
    },
    
    /**
     * Sign a document
     * 
     * @param {string} documentId Document ID (hash)
     * @param {string} email User email
     * @param {Array} fields Field positions and data
     * @returns {Promise<object>} Signing result
     */
    signDocument: async function(documentId, email, fields) {
        try {
            // Check OTP verification
            if (!this.otpVerified) {
                alert('Vă rugăm să verificați identitatea prin cod OTP înainte de a semna.');
                return { success: false, message: 'OTP verification required' };
            }
            
            // Check legal disclaimer
            const disclaimerCheckbox = document.getElementById('legal-disclaimer-checkbox');
            if (!disclaimerCheckbox || !disclaimerCheckbox.checked) {
                alert('Vă rugăm să acceptați condițiile legale înainte de a semna.');
                return { success: false, message: 'Legal disclaimer not accepted' };
            }
            
            // Prepare data
            const data = {
                email: email,
                document_id: documentId,
                signature_fields: fields,
                legal_disclaimer_accepted: true
            };
            
            // Send request
            const response = await fetch(this.baseUrl + '/sign-document.php', {
                method: 'POST',
                headers: {
                    'Content-Type': 'application/json'
                },
                body: JSON.stringify(data)
            });
            
            const result = await response.json();
            
            if (result.success) {
                console.log('Document signed successfully:', result);
                
                // Store signature data in app state
                window.appState.signatureData = {
                    userId: email,
                    documentHash: documentId,
                    signature: result.signature,
                    timestamp: result.timestamp,
                    timestampIso: result.timestamp_iso
                };
            } else {
                console.error('Document signing failed:', result.message);
                alert('Eroare la semnarea documentului: ' + result.message);
            }
            
            return result;
        } catch (error) {
            console.error('Error signing document:', error);
            alert('Eroare la semnarea documentului. Vă rugăm să încercați din nou.');
            throw error;
        }
    },
    
    /**
     * Get the user's IP address
     * 
     * @returns {Promise<string>} IP address
     */
    getUserIP: async function() {
        try {
            // Use ipify API to get the user's IP
            const response = await fetch('https://api.ipify.org?format=json');
            const data = await response.json();
            return data.ip;
        } catch (error) {
            console.error('Error getting IP address:', error);
            return '127.0.0.1'; // Fallback to localhost
        }
    },
    
    /**
     * Create an audit log entry
     * 
     * @param {object} logData Log data
     * @returns {Promise<object>} Result
     */
    createAuditLog: async function(logData) {
        // In the browser, we don't create audit logs directly
        // We rely on the server to create audit logs for security
        console.log('Audit log will be created on the server:', logData);
        return { success: true };
    }
};

// Initialize when the DOM is loaded
document.addEventListener('DOMContentLoaded', function() {
    if (window.eidasCompliance) {
        window.eidasCompliance.init();
    }
});