/**
 * eIDAS Compliance Module
 * Handles document integrity, user authentication, signature generation,
 * audit logging, timestamping, and other eIDAS compliance requirements
 */

// Configure secret key for HMAC signature generation
const SECRET_KEY = 'inkless-flow-client-side-key-2024';

// Initialize the eIDAS compliance module
function initEidasCompliance() {
    console.log('Initializing eIDAS compliance module...');
    
    // Add legal disclaimer checkbox to the UI
    addLegalDisclaimerToUI();
    
    // Set up event listeners for the OTP verification
    setupOTPVerification();
}

// ====== DOCUMENT INTEGRITY ======

/**
 * Generate a SHA-256 hash for a file
 * @param {File} file - The file to hash
 * @returns {Promise<string>} - Promise resolving to the hex digest of the hash
 */
async function generateDocumentHash(file) {
    return new Promise((resolve, reject) => {
        const reader = new FileReader();
        
        reader.onload = async function(e) {
            try {
                const arrayBuffer = e.target.result;
                const hashBuffer = await crypto.subtle.digest('SHA-256', arrayBuffer);
                const hashArray = Array.from(new Uint8Array(hashBuffer));
                const hashHex = hashArray.map(b => b.toString(16).padStart(2, '0')).join('');
                resolve(hashHex);
            } catch (error) {
                console.error('Error generating document hash:', error);
                reject(error);
            }
        };
        
        reader.onerror = function(error) {
            console.error('Error reading file for hashing:', error);
            reject(error);
        };
        
        reader.readAsArrayBuffer(file);
    });
}

/**
 * Verify document integrity by comparing current hash with stored hash
 * @param {File} file - The current file
 * @param {string} storedHash - Previously stored hash
 * @returns {Promise<boolean>} - Promise resolving to true if hashes match
 */
async function verifyDocumentIntegrity(file, storedHash) {
    try {
        const currentHash = await generateDocumentHash(file);
        return currentHash === storedHash;
    } catch (error) {
        console.error('Error verifying document integrity:', error);
        return false;
    }
}

// ====== USER AUTHENTICATION ======

/**
 * Generate a random OTP for email verification
 * @returns {string} - 6-digit OTP
 */
function generateOTP() {
    return Math.floor(100000 + Math.random() * 900000).toString();
}

/**
 * Send OTP to user's email using server API
 * @param {string} email - User's email address
 * @param {string} otp - The generated OTP
 * @returns {Promise<boolean>} - Promise resolving to true if sent successfully
 */
async function sendOTPEmail(email, otp) {
    try {
        // Show loading indicator
        const button = document.getElementById('request-otp-btn');
        if (button) {
            button.disabled = true;
            button.innerHTML = '<span class="loading-spinner-small"></span> Sending...';
        }
        
        // In development mode, log the OTP to console for testing
        console.log(`Development mode: OTP for ${email} is ${otp}`);
        
        // Make API call to send OTP
        const response = await fetch('send-otp.php', {
            method: 'POST',
            headers: {
                'Content-Type': 'application/json',
            },
            body: JSON.stringify({ 
                email, 
                otp,
                domain: 'inkless-flow.com', 
                subject: 'Your Inkless-Flow Verification Code',
                message: `Your verification code is ${otp}. This code will expire in 10 minutes.`
            }),
        });
        
        if (button) {
            if (response.ok) {
                button.innerHTML = 'Code Sent';
            } else {
                button.innerHTML = 'Send Verification Code';
                button.disabled = false;
            }
        }
        
        // If we get a successful response
        if (response.ok) {
            const result = await response.json();
            if (result.success) {
                return true;
            } else {
                console.error('Server error sending OTP:', result.message);
                alert('Error: ' + (result.message || 'Failed to send verification code'));
                return false;
            }
        } else {
            console.error('HTTP error sending OTP:', response.status);
            alert('Error: Failed to send verification code. Please try again.');
            return false;
        }
    } catch (error) {
        console.error('Error sending OTP:', error);
        
        if (button) {
            button.innerHTML = 'Send Verification Code';
            button.disabled = false;
        }
        
        alert('Error: Failed to send verification code. Please try again.');
        return false;
    }
}

/**
 * Set up the OTP verification flow in the UI
 */
function setupOTPVerification() {
    // Add OTP verification UI to step before signing
    const stepContainer = document.getElementById('step7');
    if (!stepContainer) return;
    
    // Check if the OTP verification UI already exists
    if (document.getElementById('otp-verification-container')) {
        // Set up the existing OTP verification UI
        setupExistingOTPVerification();
        return;
    }
    
    // Create OTP verification UI
    const otpContainer = document.createElement('div');
    otpContainer.id = 'otp-verification-container';
    otpContainer.className = 'card';
    otpContainer.style.marginBottom = '20px';
    otpContainer.innerHTML = `
        <h3>Verify Your Identity</h3>
        <p>To comply with eIDAS regulations, we need to verify your identity before signing.</p>
        <div class="form-group">
            <button id="request-otp-btn" class="btn">Send Verification Code</button>
        </div>
        <div id="otp-input-container" style="display: none;">
            <div class="form-group">
                <label for="otp-input">Enter Verification Code</label>
                <input type="text" id="otp-input" placeholder="Enter 6-digit code" maxlength="6" pattern="[0-9]{6}">
            </div>
            <div class="form-group">
                <button id="verify-otp-btn" class="btn">Verify</button>
            </div>
        </div>
    `;
    
    // Insert before the sign-and-send button
    const signBtn = document.getElementById('sign-and-send');
    if (signBtn && signBtn.parentNode) {
        signBtn.parentNode.insertBefore(otpContainer, signBtn);
    } else {
        stepContainer.querySelector('.card').appendChild(otpContainer);
    }
    
    // Set up the OTP verification flow
    setupExistingOTPVerification();
}

/**
 * Set up event handlers for existing OTP verification UI
 */
function setupExistingOTPVerification() {
    // Add event listeners for OTP flow
    const requestOTPBtn = document.getElementById('request-otp-btn');
    const verifyOTPBtn = document.getElementById('verify-otp-btn');
    const otpInput = document.getElementById('otp-input');
    const otpInputContainer = document.getElementById('otp-input-container');
    const signBtn = document.getElementById('sign-and-send');
    
    let isSendingOTP = false;
    
    if (requestOTPBtn) {
        requestOTPBtn.addEventListener('click', async function() {
            if (isSendingOTP) return;
            isSendingOTP = true;
            
            const email = document.getElementById('yourEmail')?.value;
            if (!email) {
                alert('Please enter your email address in Step 1');
                isSendingOTP = false;
                return;
            }
            
            try {
                // Generate and store OTP
                const otp = generateOTP();
                window.appState.currentOTP = otp;
                
                // Send OTP email
                const sent = await sendOTPEmail(email, otp);
                if (sent) {
                    if (otpInputContainer) {
                        otpInputContainer.style.display = 'block';
                    }
                    if (otpInput) {
                        otpInput.focus();
                    }
                }
            } catch (error) {
                console.error('Error in OTP flow:', error);
                alert('An error occurred while sending the verification code. Please try again.');
            } finally {
                isSendingOTP = false;
            }
        });
    }
    
    if (verifyOTPBtn && otpInput) {
        verifyOTPBtn.addEventListener('click', function() {
            const inputOTP = otpInput.value.trim();
            if (!inputOTP) {
                alert('Please enter the verification code');
                return;
            }
            
            // For development, accept any 6-digit code
            if (inputOTP === window.appState.currentOTP || 
                (inputOTP.length === 6 && /^\d+$/.test(inputOTP) && window.location.hostname === 'localhost')) {
                window.appState.otpVerified = true;
                
                // Show success message
                const otpContainer = document.getElementById('otp-verification-container');
                if (otpContainer) {
                    otpContainer.innerHTML = '<div class="alert alert-success">✓ Identity Verified</div>';
                }
                
                // Enable the sign button
                if (signBtn) {
                    signBtn.disabled = false;
                }
            } else {
                alert('Invalid verification code. Please try again.');
            }
        });
    }
    
    // Disable the sign button until OTP verification
    if (signBtn) {
        signBtn.disabled = true;
    }
    
    // Add styles for the loading spinner if not already present
    if (!document.getElementById('loading-spinner-small-style')) {
        const style = document.createElement('style');
        style.id = 'loading-spinner-small-style';
        style.textContent = `
            .loading-spinner-small {
                display: inline-block;
                width: 14px;
                height: 14px;
                border: 2px solid rgba(255, 255, 255, 0.3);
                border-top: 2px solid #ffffff;
                border-radius: 50%;
                animation: spin 1s linear infinite;
                vertical-align: middle;
                margin-right: 5px;
            }
        `;
        document.head.appendChild(style);
    }
}

// ====== SIGNATURE GENERATION ======

/**
 * Generate HMAC-SHA256 signature
 * @param {string} userId - User identifier (email or name)
 * @param {string} documentHash - Document SHA-256 hash
 * @param {number} timestamp - Unix timestamp
 * @returns {Promise<string>} - Promise resolving to the hex signature
 */
async function generateSignature(userId, documentHash, timestamp) {
    try {
        // Create the message to sign
        const message = `${userId}|${documentHash}|${timestamp}`;
        
        // Convert message and key to ArrayBuffer
        const encoder = new TextEncoder();
        const messageBuffer = encoder.encode(message);
        const keyBuffer = encoder.encode(SECRET_KEY);
        
        // Import the key
        const key = await crypto.subtle.importKey(
            'raw',
            keyBuffer,
            { name: 'HMAC', hash: 'SHA-256' },
            false,
            ['sign']
        );
        
        // Sign the message
        const signatureBuffer = await crypto.subtle.sign(
            'HMAC',
            key,
            messageBuffer
        );
        
        // Convert to hex
        const signatureArray = Array.from(new Uint8Array(signatureBuffer));
        const signatureHex = signatureArray.map(b => b.toString(16).padStart(2, '0')).join('');
        
        return signatureHex;
    } catch (error) {
        console.error('Error generating signature:', error);
        throw error;
    }
}

// ====== TIMESTAMPING ======

/**
 * Get a trusted timestamp with better error handling
 * @returns {Promise<Object>} Timestamp data
 */
async function getTrustedTimestamp() {
    try {
        // Try to get timestamp from worldtimeapi.org with a timeout
        const controller = new AbortController();
        const timeoutId = setTimeout(() => controller.abort(), 3000); // 3 second timeout
        
        const response = await fetch('https://worldtimeapi.org/api/ip', {
            signal: controller.signal
        });
        
        clearTimeout(timeoutId);
        
        if (!response.ok) {
            throw new Error('API response not OK');
        }
        
        const data = await response.json();
        
        return {
            timestamp: data.unixtime,
            iso: data.datetime,
            source: 'worldtimeapi.org'
        };
    } catch (error) {
        console.error('Error fetching trusted timestamp:', error);
        
        // Fallback to local time
        const now = new Date();
        return {
            timestamp: Math.floor(now.getTime() / 1000),
            iso: now.toISOString(),
            source: 'local'
        };
    }
}

// ====== AUDIT LOGGING ======

/**
 * Create an audit log with better error handling
 * @param {Object} data Audit log data
 * @returns {Object} Created log entry
 */
function createAuditLog(data) {
    // Create log entry
    const logEntry = {
        userId: data.userId || 'unknown',
        documentHash: data.documentHash || 'unknown',
        signature: data.signature || '',
        timestamp: data.timestamp || Math.floor(Date.now() / 1000),
        ipAddress: data.ipAddress || '127.0.0.1',
        userAgent: navigator.userAgent,
        action: data.action || 'document_signed'
    };
    
    // Store in localStorage as a fallback
    const auditLogs = JSON.parse(localStorage.getItem('auditLogs') || '[]');
    auditLogs.push(logEntry);
    localStorage.setItem('auditLogs', JSON.stringify(auditLogs));
    
    console.log('Audit log created:', logEntry);
    
    // Try to send to server if available
    try {
        // Check if log-audit.php exists with a timeout
        fetch('log-audit.php', { 
            method: 'HEAD',
            signal: AbortSignal.timeout(1000) // 1 second timeout
        }).then(response => {
            if (response.ok) {
                // File exists, send the log
                return fetch('log-audit.php', {
                    method: 'POST',
                    headers: {
                        'Content-Type': 'application/json',
                    },
                    body: JSON.stringify(logEntry),
                });
            }
        }).catch(() => {
            // Silently fail - we already saved to localStorage
        });
    } catch (error) {
        // Silently fail - we already saved to localStorage
    }
    
    return logEntry;
}

/**
 * Get user's IP address with better error handling
 * @returns {Promise<string>} IP address
 */
async function getUserIP() {
    try {
        // Try to get IP from server with a timeout
        const controller = new AbortController();
        const timeoutId = setTimeout(() => controller.abort(), 2000); // 2 second timeout
        
        // Check if get-ip.php exists first
        const checkResponse = await fetch('get-ip.php', { 
            method: 'HEAD',
            signal: controller.signal
        });
        
        clearTimeout(timeoutId);
        
        if (checkResponse.ok) {
            const response = await fetch('get-ip.php');
            if (response.ok) {
                const data = await response.json();
                if (data.ip) {
                    return data.ip;
                }
            }
        }
        
        // If we get here, the endpoint doesn't exist or didn't return valid data
        return '127.0.0.1';
    } catch (error) {
        console.error('Error getting IP:', error);
        return '127.0.0.1';
    }
}

// ====== LEGAL DISCLAIMER ======

/**
 * Add legal disclaimer checkbox to the UI
 */
function addLegalDisclaimerToUI() {
    // Add legal disclaimer checkbox to step 7
    const stepContainer = document.getElementById('step7');
    if (!stepContainer) return;
    
    // Check if the legal disclaimer already exists
    if (document.getElementById('legal-disclaimer-container')) return;
    
    // Create legal disclaimer UI
    const disclaimerContainer = document.createElement('div');
    disclaimerContainer.id = 'legal-disclaimer-container';
    disclaimerContainer.className = 'card';
    disclaimerContainer.style.marginBottom = '20px';
    disclaimerContainer.innerHTML = `
        <h3>Legal Disclaimer</h3>
        <div class="form-group">
            <div style="padding: 10px; border: 1px solid #ddd; background-color: #f9f9f9; margin-bottom: 10px;">
                <p>By signing this document electronically, you agree to the following:</p>
                <ul>
                    <li>This electronic signature is being created using a Simple Electronic Signature (SES) system.</li>
                    <li>SES does not provide the same legal weight as Advanced Electronic Signatures (AES) or Qualified Electronic Signatures (QES).</li>
                    <li>This signature is legally binding in accordance with eIDAS Regulation (EU) No 910/2014, but may not meet the highest level of security requirements.</li>
                    <li>Your signature will be accompanied by a timestamp and cryptographic proof of signing.</li>
                    <li>You consent to the processing of your personal data for the purpose of electronic signing in accordance with GDPR regulations.</li>
                </ul>
            </div>
            <label>
                <input type="checkbox" id="legal-disclaimer-checkbox"> 
                I have read and agree to the legal disclaimer.
            </label>
        </div>
    `;
    
    // Insert before the sign-and-send button
    const signBtn = document.getElementById('sign-and-send');
    if (signBtn && signBtn.parentNode) {
        signBtn.parentNode.insertBefore(disclaimerContainer, signBtn);
    } else {
        stepContainer.querySelector('.card').appendChild(disclaimerContainer);
    }
    
    // Disable the sign button until disclaimer is accepted
    if (signBtn) {
        const originalClickHandler = signBtn.onclick;
        signBtn.onclick = function(e) {
            const checkbox = document.getElementById('legal-disclaimer-checkbox');
            if (!checkbox || !checkbox.checked) {
                e.preventDefault();
                alert('Please accept the legal disclaimer before signing.');
                return false;
            }
            
            if (!window.appState.otpVerified) {
                e.preventDefault();
                alert('Please complete identity verification before signing.');
                return false;
            }
            
            // Call the original click handler
            if (typeof originalClickHandler === 'function') {
                return originalClickHandler.call(this, e);
            }
        };
    }
}

// ====== PDF SIGNING ======

/**
 * Embed signature information in PDF
 * This is a mock function as we would need a PDF generation library
 * @param {string} signature - The signature to embed
 * @param {Object} signatureData - Additional signature data
 * @returns {Promise<Blob>} - Promise resolving to the signed PDF blob
 */
async function embedSignatureInPDF(pdfArrayBuffer, signature, signatureData) {
    // In a real implementation, you would use a PDF library like PDF.js or jsPDF
    // to modify the PDF and add the signature information
    
    console.log('Embedding signature in PDF:', signature);
    console.log('Signature data:', signatureData);
    
    // For demo purposes, just return the original PDF
    return new Blob([pdfArrayBuffer], { type: 'application/pdf' });
}

/**
 * Notify recipients about the signed document
 * @param {Array} recipients - Array of recipient objects with name and email
 * @param {string} senderName - Name of the sender
 * @param {string} senderEmail - Email of the sender
 * @param {string} documentName - Name of the document
 * @returns {Promise<boolean>} - Promise resolving to true if sent successfully
 */
async function notifyRecipients(recipients, senderName, senderEmail, documentName) {
    if (!recipients || recipients.length === 0) {
        console.log('No recipients to notify');
        return true;
    }
    
    try {
        const response = await fetch('notify-recipients.php', {
            method: 'POST',
            headers: {
                'Content-Type': 'application/json',
            },
            body: JSON.stringify({
                recipients,
                senderName,
                senderEmail,
                documentName,
                subject: `${senderName} has sent you a document to sign`,
                message: `${senderName} (${senderEmail}) has sent you a document to sign using Inkless-Flow.
                          Please follow the link below to view and sign the document.`
            }),
        });
        
        if (response.ok) {
            const result = await response.json();
            if (result.success) {
                return true;
            } else {
                console.error('Server error notifying recipients:', result.message);
                return false;
            }
        } else {
            console.error('HTTP error notifying recipients:', response.status);
            return false;
        }
    } catch (error) {
        console.error('Error notifying recipients:', error);
        return false;
    }
}

// Export functions for use in other modules
window.eidasCompliance = {
    init: initEidasCompliance,
    generateDocumentHash,
    verifyDocumentIntegrity,
    generateOTP,
    sendOTPEmail,
    generateSignature,
    getTrustedTimestamp,
    createAuditLog,
    getUserIP,
    embedSignatureInPDF,
    notifyRecipients
};