/**
 * Improved Field Placement Module
 * Handles dragging, placing, and managing signature and other fields
 */

// Initialize field handling
function initFieldHandling(elements) {
    console.log('Initializing improved field handling...');
    
    // Make template fields draggable
    initDraggableFields();
    
    // Set up context menu for right-click field placement
    setupContextMenu();
    
    // Set up field controls (verify, remove buttons)
    setupFieldControls();
    
    // Update field availability based on current state
    updateFieldAvailability();

// Export functions for use in other modules
window.initFieldHandling = initFieldHandling;
window.updateFieldAvailability = updateFieldAvailability;
window.getFieldLabel = getFieldLabel;
window.getFieldContent = getFieldContent;
window.makeDraggable = makeDraggable;
window.makeResizable = makeResizable;
window.placeField = verifyField;
window.removeField = removeField;
}

// Initialize draggable field templates
function initDraggableFields() {
    // Make all template fields draggable
    document.querySelectorAll('.field-container.template').forEach(field => {
        field.draggable = true;
        
        field.addEventListener('dragstart', function(e) {
            e.dataTransfer.setData('text/plain', this.id);
            e.dataTransfer.effectAllowed = 'copy';
        });
    });
    
    // Set up document area as drop target
    const finalPreviewContainer = document.getElementById('final-preview-container');
    if (finalPreviewContainer) {
        // Handle drag over
        finalPreviewContainer.addEventListener('dragover', function(e) {
            e.preventDefault();
            e.dataTransfer.dropEffect = 'copy';
            
            // Highlight drop area
            this.classList.add('dragover');
        });
        
        // Handle drag leave
        finalPreviewContainer.addEventListener('dragleave', function(e) {
            this.classList.remove('dragover');
        });
        
        // Handle drop
        finalPreviewContainer.addEventListener('drop', function(e) {
            e.preventDefault();
            this.classList.remove('dragover');
            
            const fieldId = e.dataTransfer.getData('text/plain');
            if (!fieldId) return;
            
            // Get the target page and position
            const targetPage = findTargetPage(e.clientX, e.clientY, this);
            if (!targetPage) return;
            
            // Calculate position within the page
            const rect = targetPage.getBoundingClientRect();
            const pageOffsetX = e.clientX - rect.left;
            const pageOffsetY = e.clientY - rect.top;
            
            // Create the field
            const pageNum = parseInt(targetPage.dataset.pageNum);
            createField(fieldId, targetPage, pageOffsetX, pageOffsetY, pageNum);
        });
    }
}

// Find the target page for a drop or right-click
function findTargetPage(clientX, clientY, container) {
    const pages = container.querySelectorAll('.document-page');
    for (const page of pages) {
        const rect = page.getBoundingClientRect();
        if (clientX >= rect.left && clientX <= rect.right && 
            clientY >= rect.top && clientY <= rect.bottom) {
            return page;
        }
    }
    return null;
}

// Setup context menu for right-click field placement
function setupContextMenu() {
    // Create context menu
    const contextMenu = document.createElement('div');
    contextMenu.className = 'context-menu';
    contextMenu.style.display = 'none';
    document.body.appendChild(contextMenu);
    
    // Hide context menu on click outside
    document.addEventListener('click', function() {
        contextMenu.style.display = 'none';
    });
    
    // Handle right-click on document preview
    const finalPreviewContainer = document.getElementById('final-preview-container');
    if (finalPreviewContainer) {
        finalPreviewContainer.addEventListener('contextmenu', function(e) {
            e.preventDefault();
            
            // Find target page
            const targetPage = findTargetPage(e.clientX, e.clientY, this);
            if (!targetPage) return;
            
            // Store position for later use
            const rect = targetPage.getBoundingClientRect();
            const pageOffsetX = e.clientX - rect.left;
            const pageOffsetY = e.clientY - rect.top;
            const pageNum = parseInt(targetPage.dataset.pageNum);
            
            // Build context menu items
            contextMenu.innerHTML = '';
            const availableFields = getAvailableFields();
            
            if (availableFields.length === 0) {
                contextMenu.innerHTML = '<div class="context-menu-item">No fields available</div>';
            } else {
                availableFields.forEach(field => {
                    const menuItem = document.createElement('div');
                    menuItem.className = 'context-menu-item';
                    menuItem.textContent = getFieldLabel(field);
                    
                    menuItem.addEventListener('click', function() {
                        createField(field, targetPage, pageOffsetX, pageOffsetY, pageNum);
                    });
                    
                    contextMenu.appendChild(menuItem);
                });
            }
            
            // Position and display context menu
            contextMenu.style.left = e.clientX + 'px';
            contextMenu.style.top = e.clientY + 'px';
            contextMenu.style.display = 'block';
        });
    }
}

// Get the list of available field types
function getAvailableFields() {
    const fields = [];
    
    // Include signature field if signature exists
    if (window.appState.signatureImage) {
        fields.push('signature-field-template');
    }
    
    // Include date field if date is selected
    if (window.appState.selectedDate) {
        fields.push('date-field-template');
    }
    
    // Include image field if image is uploaded
    if (window.appState.uploadedImage) {
        fields.push('image-field-template');
    }
    
    // Include recipient fields if recipients exist
    if (window.appState.recipients && window.appState.recipients.length > 0) {
        fields.push('recipient-field-template');
    }
    
    // Include custom text fields if they exist
    if (window.appState.customTextFields && window.appState.customTextFields.length > 0) {
        // Add each custom text field as its own entry
        window.appState.customTextFields.forEach(field => {
            fields.push('custom-text-field-template');
        });
    }
    
    return fields;
}

// Get human-readable field label
function getFieldLabel(fieldType) {
    switch (fieldType) {
        case 'signature-field-template':
            return 'Signature';
        case 'date-field-template':
            return 'Date';
        case 'image-field-template':
            return 'Image';
        case 'recipient-field-template':
            return 'Recipient Signature';
        case 'custom-text-field-template':
            return 'Custom Text';
        default:
            return fieldType.replace('-field-template', '').replace('-', ' ');
    }
}

// Create a new field instance
function createField(fieldId, targetPage, x, y, pageNum) {
    // Check if field is available
    if (isFieldAlreadyPlaced(fieldId, pageNum)) {
        alert(`A ${getFieldLabel(fieldId)} field is already placed on this page.`);
        return;
    }
    
    // Get field overlay
    const overlay = targetPage.querySelector('.field-overlay');
    if (!overlay) return;
    
    // Create new field ID
    const fieldType = fieldId.replace('-template', '');
    const newId = fieldType + '-' + Date.now();
    
    // Create field element
    const field = document.createElement('div');
    field.id = newId;
    field.className = 'field-container draft';
    field.dataset.type = fieldType;
    field.dataset.pageNum = pageNum;
    
    // Set initial position
    field.style.position = 'absolute';
    field.style.left = Math.max(0, x - 75) + 'px'; // Center the field
    field.style.top = Math.max(0, y - 30) + 'px';
    field.style.width = '150px';
    field.style.height = '60px';
    field.style.zIndex = '10';
    
    // Add field content
    let fieldContent = '';
    let customData = null;
    
    switch (fieldType) {
        case 'signature-field':
            fieldContent = `
                <div class="field-label">Your Signature</div>
                <div class="field-content">
                    <img src="${window.appState.signatureImage}" style="max-width: 100%; max-height: 100%; background: transparent;">
                </div>
            `;
            break;
            
        case 'date-field':
            fieldContent = `
                <div class="field-label">Date</div>
                <div class="field-content">
                    ${new Date(window.appState.selectedDate).toLocaleDateString()}
                </div>
            `;
            break;
            
        case 'image-field':
            // For images, use a placeholder until final rendering
            fieldContent = `
                <div class="field-label">Image</div>
                <div class="field-content">
                    <div id="image-placeholder-${newId}">Image Field</div>
                </div>
            `;
            break;
            
        case 'recipient-field':
            const recipient = window.appState.recipients.length > 0 ? 
                window.appState.recipients[0].name : 'Recipient';
            fieldContent = `
                <div class="field-label">${recipient} Signature</div>
                <div class="field-content">
                    Signature Field
                </div>
            `;
            break;
            
        case 'custom-text-field':
            // Get the first available custom text field
            const customField = window.appState.customTextFields[0];
            if (customField) {
                customData = {
                    title: customField.title,
                    content: customField.content
                };
                fieldContent = `
                    <div class="field-label">${customField.title}</div>
                    <div class="field-content">
                        ${customField.content}
                    </div>
                `;
                
                // Remove this field from available custom fields
                window.appState.customTextFields = window.appState.customTextFields.filter(
                    f => f.title !== customField.title || f.content !== customField.content
                );
            }
            break;
            
        default:
            fieldContent = `
                <div class="field-label">Field</div>
                <div class="field-content">Content</div>
            `;
    }
    
    // Add controls
    fieldContent += `
        <div class="field-controls">
            <button class="field-control-btn place-btn" title="Confirm placement">✓</button>
            <button class="field-control-btn remove-btn" title="Remove field">×</button>
        </div>
    `;
    
    // Add resize handles
    fieldContent += `
        <div class="resize-handle top-left"></div>
        <div class="resize-handle top-right"></div>
        <div class="resize-handle bottom-left"></div>
        <div class="resize-handle bottom-right"></div>
    `;
    
    field.innerHTML = fieldContent;
    
    // Add the field to the page
    overlay.appendChild(field);
    
    // Make field draggable
    makeDraggable(field);
    
    // Make field resizable
    makeResizable(field);
    
    // Add to placed fields
    window.appState.placedFields.push({
        id: newId,
        type: fieldType,
        pageNum: pageNum,
        left: parseInt(field.style.left),
        top: parseInt(field.style.top),
        width: parseInt(field.style.width),
        height: parseInt(field.style.height),
        verified: false,
        customData: customData
    });
    
    // Add control event listeners
    const placeBtn = field.querySelector('.place-btn');
    if (placeBtn) {
        placeBtn.addEventListener('click', function(e) {
            e.stopPropagation();
            verifyField(field);
        });
    }
    
    const removeBtn = field.querySelector('.remove-btn');
    if (removeBtn) {
        removeBtn.addEventListener('click', function(e) {
            e.stopPropagation();
            removeField(field);
        });
    }
    
    // Update field availability
    updateFieldAvailability();
    
    // Return the created field
    return field;
}

// Check if a field of the same type is already placed on the page
function isFieldAlreadyPlaced(fieldType, pageNum) {
    const type = fieldType.replace('-template', '');
    
    // Signature and recipient fields can only be placed once per page
    if (type === 'signature-field' || type === 'recipient-field') {
        return window.appState.placedFields.some(field => 
            field.type === type && field.pageNum === pageNum && field.verified);
    }
    
    return false;
}

// Make a field draggable within its page
function makeDraggable(field) {
    let isDragging = false;
    let startX, startY;
    let startLeft, startTop;
    
    field.addEventListener('mousedown', function(e) {
        // Only handle mousedown on the field itself or label, not on buttons or resize handles
        if (e.target.closest('.field-control-btn') || e.target.closest('.resize-handle')) {
            return;
        }
        
        e.preventDefault();
        isDragging = true;
        
        startX = e.clientX;
        startY = e.clientY;
        startLeft = parseInt(field.style.left) || 0;
        startTop = parseInt(field.style.top) || 0;
        
        field.style.zIndex = '20'; // Bring to front while dragging
        
        // Add event listeners for drag and drop
        document.addEventListener('mousemove', drag);
        document.addEventListener('mouseup', stopDrag);
    });
    
    // Touch support for mobile
    field.addEventListener('touchstart', function(e) {
        // Only handle touch on the field itself or label, not on buttons or resize handles
        if (e.target.closest('.field-control-btn') || e.target.closest('.resize-handle')) {
            return;
        }
        
        e.preventDefault();
        isDragging = true;
        
        const touch = e.touches[0];
        startX = touch.clientX;
        startY = touch.clientY;
        startLeft = parseInt(field.style.left) || 0;
        startTop = parseInt(field.style.top) || 0;
        
        field.style.zIndex = '20'; // Bring to front while dragging
        
        // Add event listeners for drag and drop
        document.addEventListener('touchmove', dragTouch);
        document.addEventListener('touchend', stopDragTouch);
    });
    
    function drag(e) {
        if (!isDragging) return;
        
        // Calculate new position
        const deltaX = e.clientX - startX;
        const deltaY = e.clientY - startY;
        
        // Get page boundaries
        const page = field.parentNode.parentNode; // .field-overlay -> .document-page
        const pageRect = page.getBoundingClientRect();
        
        // Limit to page boundaries
        let newLeft = startLeft + deltaX;
        let newTop = startTop + deltaY;
        
        // Keep field within the page bounds
        const fieldWidth = parseInt(field.style.width) || 150;
        const fieldHeight = parseInt(field.style.height) || 60;
        
        newLeft = Math.max(0, Math.min(newLeft, pageRect.width - fieldWidth));
        newTop = Math.max(0, Math.min(newTop, pageRect.height - fieldHeight));
        
        // Update position
        field.style.left = newLeft + 'px';
        field.style.top = newTop + 'px';
        
        // Update stored position
        updateFieldPosition(field.id, newLeft, newTop);
    }
    
    function dragTouch(e) {
        if (!isDragging) return;
        
        const touch = e.touches[0];
        
        // Calculate new position
        const deltaX = touch.clientX - startX;
        const deltaY = touch.clientY - startY;
        
        // Get page boundaries
        const page = field.parentNode.parentNode; // .field-overlay -> .document-page
        const pageRect = page.getBoundingClientRect();
        
        // Limit to page boundaries
        let newLeft = startLeft + deltaX;
        let newTop = startTop + deltaY;
        
        // Keep field within the page bounds
        const fieldWidth = parseInt(field.style.width) || 150;
        const fieldHeight = parseInt(field.style.height) || 60;
        
        newLeft = Math.max(0, Math.min(newLeft, pageRect.width - fieldWidth));
        newTop = Math.max(0, Math.min(newTop, pageRect.height - fieldHeight));
        
        // Update position
        field.style.left = newLeft + 'px';
        field.style.top = newTop + 'px';
        
        // Update stored position
        updateFieldPosition(field.id, newLeft, newTop);
    }
    
    function stopDrag() {
        if (!isDragging) return;
        
        isDragging = false;
        field.style.zIndex = '10';
        
        document.removeEventListener('mousemove', drag);
        document.removeEventListener('mouseup', stopDrag);
    }
    
    function stopDragTouch() {
        if (!isDragging) return;
        
        isDragging = false;
        field.style.zIndex = '10';
        
        document.removeEventListener('touchmove', dragTouch);
        document.removeEventListener('touchend', stopDragTouch);
    }
}

// Make a field resizable
function makeResizable(field) {
    const handles = field.querySelectorAll('.resize-handle');
    
    handles.forEach(handle => {
        let isResizing = false;
        let startX, startY, startWidth, startHeight, startLeft, startTop;
        
        handle.addEventListener('mousedown', function(e) {
            e.preventDefault();
            e.stopPropagation();
            
            isResizing = true;
            startX = e.clientX;
            startY = e.clientY;
            startWidth = parseInt(field.style.width) || 150;
            startHeight = parseInt(field.style.height) || 60;
            startLeft = parseInt(field.style.left) || 0;
            startTop = parseInt(field.style.top) || 0;
            
            document.addEventListener('mousemove', resize);
            document.addEventListener('mouseup', stopResize);
        });
        
        // Touch support for mobile
        handle.addEventListener('touchstart', function(e) {
            e.preventDefault();
            e.stopPropagation();
            
            isResizing = true;
            const touch = e.touches[0];
            startX = touch.clientX;
            startY = touch.clientY;
            startWidth = parseInt(field.style.width) || 150;
            startHeight = parseInt(field.style.height) || 60;
            startLeft = parseInt(field.style.left) || 0;
            startTop = parseInt(field.style.top) || 0;
            
            document.addEventListener('touchmove', resizeTouch);
            document.addEventListener('touchend', stopResizeTouch);
        });
        
        function resize(e) {
            if (!isResizing) return;
            
            const deltaX = e.clientX - startX;
            const deltaY = e.clientY - startY;
            
            // Handle different resize directions
            if (handle.classList.contains('top-left')) {
                field.style.width = Math.max(50, startWidth - deltaX) + 'px';
                field.style.height = Math.max(50, startHeight - deltaY) + 'px';
                field.style.left = startLeft + (startWidth - parseInt(field.style.width)) + 'px';
                field.style.top = startTop + (startHeight - parseInt(field.style.height)) + 'px';
            } else if (handle.classList.contains('top-right')) {
                field.style.width = Math.max(50, startWidth + deltaX) + 'px';
                field.style.height = Math.max(50, startHeight - deltaY) + 'px';
                field.style.top = startTop + (startHeight - parseInt(field.style.height)) + 'px';
            } else if (handle.classList.contains('bottom-left')) {
                field.style.width = Math.max(50, startWidth - deltaX) + 'px';
                field.style.height = Math.max(50, startHeight + deltaY) + 'px';
                field.style.left = startLeft + (startWidth - parseInt(field.style.width)) + 'px';
            } else if (handle.classList.contains('bottom-right')) {
                field.style.width = Math.max(50, startWidth + deltaX) + 'px';
                field.style.height = Math.max(50, startHeight + deltaY) + 'px';
            }
            
            // Update stored dimensions
            updateFieldDimensions(
                field.id, 
                parseInt(field.style.left), 
                parseInt(field.style.top), 
                parseInt(field.style.width), 
                parseInt(field.style.height)
            );
        }
        
        function resizeTouch(e) {
            if (!isResizing) return;
            
            const touch = e.touches[0];
            const deltaX = touch.clientX - startX;
            const deltaY = touch.clientY - startY;
            
            // Handle different resize directions
            if (handle.classList.contains('top-left')) {
                field.style.width = Math.max(50, startWidth - deltaX) + 'px';
                field.style.height = Math.max(50, startHeight - deltaY) + 'px';
                field.style.left = startLeft + (startWidth - parseInt(field.style.width)) + 'px';
                field.style.top = startTop + (startHeight - parseInt(field.style.height)) + 'px';
            } else if (handle.classList.contains('top-right')) {
                field.style.width = Math.max(50, startWidth + deltaX) + 'px';
                field.style.height = Math.max(50, startHeight - deltaY) + 'px';
                field.style.top = startTop + (startHeight - parseInt(field.style.height)) + 'px';
            } else if (handle.classList.contains('bottom-left')) {
                field.style.width = Math.max(50, startWidth - deltaX) + 'px';
                field.style.height = Math.max(50, startHeight + deltaY) + 'px';
                field.style.left = startLeft + (startWidth - parseInt(field.style.width)) + 'px';
            } else if (handle.classList.contains('bottom-right')) {
                field.style.width = Math.max(50, startWidth + deltaX) + 'px';
                field.style.height = Math.max(50, startHeight + deltaY) + 'px';
            }
            
            // Update stored dimensions
            updateFieldDimensions(
                field.id, 
                parseInt(field.style.left), 
                parseInt(field.style.top), 
                parseInt(field.style.width), 
                parseInt(field.style.height)
            );
        }
        
        function stopResize() {
            if (!isResizing) return;
            
            isResizing = false;
            document.removeEventListener('mousemove', resize);
            document.removeEventListener('mouseup', stopResize);
        }
        
        function stopResizeTouch() {
            if (!isResizing) return;
            
            isResizing = false;
            document.removeEventListener('touchmove', resizeTouch);
            document.removeEventListener('touchend', stopResizeTouch);
        }
    });
}

// Update field position in app state
function updateFieldPosition(fieldId, left, top) {
    const fieldIndex = window.appState.placedFields.findIndex(field => field.id === fieldId);
    if (fieldIndex !== -1) {
        window.appState.placedFields[fieldIndex].left = left;
        window.appState.placedFields[fieldIndex].top = top;
    }
}

// Update field dimensions in app state
function updateFieldDimensions(fieldId, left, top, width, height) {
    const fieldIndex = window.appState.placedFields.findIndex(field => field.id === fieldId);
    if (fieldIndex !== -1) {
        window.appState.placedFields[fieldIndex].left = left;
        window.appState.placedFields[fieldIndex].top = top;
        window.appState.placedFields[fieldIndex].width = width;
        window.appState.placedFields[fieldIndex].height = height;
    }
}

// Verify field placement
function verifyField(field) {
    // Change field appearance
    field.classList.remove('draft');
    field.classList.add('verified');
    
    // Remove resize handles
    field.querySelectorAll('.resize-handle').forEach(handle => handle.remove());
    
    // Replace controls with delete button
    const controls = field.querySelector('.field-controls');
    if (controls) {
        controls.innerHTML = '<button class="field-control-btn delete-btn" title="Delete field">×</button>';
        
        // Add event listener to delete button
        const deleteBtn = controls.querySelector('.delete-btn');
        if (deleteBtn) {
            deleteBtn.addEventListener('click', function(e) {
                e.stopPropagation();
                removeField(field);
            });
        }
    }
    
    // Mark as verified in app state
    const fieldIndex = window.appState.placedFields.findIndex(f => f.id === field.id);
    if (fieldIndex !== -1) {
        window.appState.placedFields[fieldIndex].verified = true;
    }
    
    // Make field not draggable
    field.onmousedown = null;
    field.ontouchstart = null;
    
    // Update field availability
    updateFieldAvailability();
    
    // Check if all required fields are placed and verified
    checkFieldsComplete();
}

// Remove a field
function removeField(field) {
    // Remove from DOM
    field.remove();
    
    // Get the field data
    const fieldData = window.appState.placedFields.find(f => f.id === field.id);
    
    // If it's a custom text field, add it back to available fields
    if (fieldData && fieldData.type === 'custom-text-field' && fieldData.customData) {
        window.appState.customTextFields.push(fieldData.customData);
    }
    
    // Remove from app state
    window.appState.placedFields = window.appState.placedFields.filter(f => f.id !== field.id);
    
    // Update field availability
    updateFieldAvailability();
    
    // Update sign button state
    checkFieldsComplete();
}

// Check if all required fields are placed
function checkFieldsComplete() {
    // Check if signature field is placed and verified
    const signatureVerified = window.appState.placedFields.some(
        field => field.type === 'signature-field' && field.verified
    );
    
    // Enable/disable sign button
    const signBtn = document.getElementById('sign-and-send');
    if (signBtn) {
        // Don't override OTP verification requirement
        if (signatureVerified) {
            signBtn.classList.add('fields-ready');
        } else {
            signBtn.classList.remove('fields-ready');
        }
    }
}

// Setup field controls for existing fields
function setupFieldControls() {
    // Look for any existing fields and set up their controls
    document.querySelectorAll('.field-container:not(.template)').forEach(field => {
        if (field.classList.contains('draft')) {
            // Draft fields: add place and remove buttons
            const placeBtn = field.querySelector('.place-btn');
            if (placeBtn) {
                placeBtn.addEventListener('click', function(e) {
                    e.stopPropagation();
                    verifyField(field);
                });
            }
            
            const removeBtn = field.querySelector('.remove-btn');
            if (removeBtn) {
                removeBtn.addEventListener('click', function(e) {
                    e.stopPropagation();
                    removeField(field);
                });
            }
            
            // Make draggable and resizable
            makeDraggable(field);
            makeResizable(field);
        } else if (field.classList.contains('verified')) {
            // Verified fields: add delete button
            const deleteBtn = field.querySelector('.delete-btn');
            if (deleteBtn) {
                deleteBtn.addEventListener('click', function(e) {
                    e.stopPropagation();
                    removeField(field);
                });
            }
        }
    });
}

// Update field template availability based on app state
function updateFieldAvailability() {
    const elements = window.elements;
    
    // Signature field
    if (elements.signatureFieldTemplate) {
        elements.signatureFieldTemplate.style.display = 
            window.appState.signatureImage ? 'block' : 'none';
    }
    
    // Date field
    if (elements.dateFieldTemplate) {
        elements.dateFieldTemplate.style.display = 
            window.appState.selectedDate ? 'block' : 'none';
    }
    
    // Image field
    if (elements.imageFieldTemplate) {
        elements.imageFieldTemplate.style.display = 
            window.appState.uploadedImage ? 'block' : 'none';
    }
    
    // Recipient field
    if (elements.recipientFieldTemplate) {
        elements.recipientFieldTemplate.style.display = 
            (window.appState.recipients && window.appState.recipients.length > 0) ? 'block' : 'none';
    }
}

// Get field content for a specific field type
function getFieldContent(type, customData) {
    switch (type) {
        case 'signature-field': 
            return `<img src="${window.appState.signatureImage || ''}" 
                        style="display: ${window.appState.signatureImage ? 'block' : 'none'}; 
                               max-width: 100%; 
                               max-height: 100%; 
                               object-fit: contain;
                               background: transparent;">
                   <div style="display: ${window.appState.signatureImage ? 'none' : 'block'}">Signature Field</div>`;
                   
        case 'date-field': 
            return `<div>${window.appState.selectedDate ? new Date(window.appState.selectedDate).toLocaleDateString() : 'Date Field'}</div>`;
            
        case 'custom-text-field': 
            return `<div>${customData ? customData.content : 'Custom Text Field'}</div>`;
            
        case 'image-field': 
            return `<img src="${window.appState.uploadedImage ? URL.createObjectURL(window.appState.uploadedImage) : ''}" 
                        style="display: ${window.appState.uploadedImage ? 'block' : 'none'}; 
                               max-width: 100%; 
                               max-height: 100%; 
                               object-fit: contain;">
                   <div style="display: ${window.appState.uploadedImage ? 'none' : 'block'}">Image Field</div>`;
                   
        case 'recipient-field': 
            const recipient = window.appState.recipients.length > 0 ? window.appState.recipients[0].name : 'Recipient';
            return `<div>${recipient} Signature Field</div>`;
            
        default: 
            return '';
    }
}