/**
 * Improved File Handling Module
 * Handles document uploads, rendering, and display with fixed issues
 */

// Initialize file handling with proper error handling
function initFileHandling(elements) {
    console.log('Initializing improved file handling module...');
    
    // File input handler
    if (elements.fileInput) {
        console.log('Setting up file input listener');
        
        // Make sure the input is properly styled
        elements.fileInput.style.position = "absolute";
        elements.fileInput.style.width = "100%";
        elements.fileInput.style.height = "100%";
        elements.fileInput.style.top = "0";
        elements.fileInput.style.left = "0";
        elements.fileInput.style.opacity = "0";
        elements.fileInput.style.cursor = "pointer";
        elements.fileInput.style.zIndex = "10";
        
        elements.fileInput.addEventListener('change', function() {
            if (this.files && this.files.length > 0) {
                console.log('File selected:', this.files[0].name);
                handleFile(this.files[0]);
            }
        });
    } else {
        console.error('File input element not found!');
    }
    
    // Make the drop area clickable to trigger file input
    if (elements.fileDropArea && elements.fileInput) {
        console.log('Setting up file drop area');
        elements.fileDropArea.style.position = "relative";
        elements.fileDropArea.style.cursor = "pointer";
        
        // Fix: Use direct click handler
        elements.fileDropArea.onclick = function(e) {
            // Directly trigger file input click if not clicking on the input itself
            if (e.target !== elements.fileInput) {
                e.preventDefault();
                elements.fileInput.click();
            }
        };
        
        // Prevent default click behavior on the file input itself
        elements.fileInput.addEventListener('click', function(e) {
            e.stopPropagation();
        });
        
        // Drag and drop handlers
        ['dragenter', 'dragover', 'dragleave', 'drop'].forEach(eventName => {
            elements.fileDropArea.addEventListener(eventName, function(e) {
                e.preventDefault();
                e.stopPropagation();
                
                if (eventName === 'dragenter' || eventName === 'dragover') {
                    this.classList.add('drag-active');
                } else {
                    this.classList.remove('drag-active');
                }
            });
        });
        
        elements.fileDropArea.addEventListener('drop', function(e) {
            this.classList.remove('drag-active');
            if (e.dataTransfer.files && e.dataTransfer.files.length > 0) {
                handleFile(e.dataTransfer.files[0]);
            }
        });
    }
    
    // Initialize document image upload in Step 5
    initImageUpload(elements);
}

// Handle the uploaded file
function handleFile(file) {
    console.log('Processing file:', file.name, 'type:', file.type);
    
    const elements = window.elements;
    window.appState.uploadedFile = file;
    
    if (elements.fileNameDisplay) {
        elements.fileNameDisplay.textContent = 'Selected file: ' + file.name;
        elements.fileNameDisplay.style.display = 'block';
    }
    
    if (file.type === 'application/pdf') {
        renderPDF(file);
    } else if (file.type.startsWith('image/')) {
        renderImage(file);
    } else {
        // Fallback for other file types
        renderPlaceholder(file);
    }
    
    if (elements.filePreviewContainer) {
        elements.filePreviewContainer.style.display = 'block';
    }
    
    // Enable next step button
    const step1NextBtn = document.getElementById('step1-next');
    if (step1NextBtn) {
        step1NextBtn.disabled = false;
    }
    
    // Generate document hash for integrity
    generateDocumentHash(file);
    
    // If we're on step 7, prepare document for signing
    const currentStep = window.appState.currentStep || 1;
    if (currentStep === 7 && typeof window.prepareDocumentForSigning === 'function') {
        console.log("Currently on step 7, re-preparing document for signing...");
        setTimeout(function() {
            window.prepareDocumentForSigning();
        }, 500);
    }
}

// Generate document hash for integrity checking
async function generateDocumentHash(file) {
    try {
        const fileBuffer = await file.arrayBuffer();
        const hashBuffer = await crypto.subtle.digest('SHA-256', fileBuffer);
        const hashArray = Array.from(new Uint8Array(hashBuffer));
        const hashHex = hashArray.map(b => b.toString(16).padStart(2, '0')).join('');
        
        console.log('Document hash generated:', hashHex);
        window.appState.documentHash = hashHex;
        
        // In a real implementation, we would send this hash to the server
        // Here we just log it and store it in appState
        return hashHex;
    } catch (error) {
        console.error('Error generating document hash:', error);
        // Fall back to random hash for demo purposes
        const fallbackHash = Array.from(crypto.getRandomValues(new Uint8Array(32)))
            .map(b => b.toString(16).padStart(2, '0')).join('');
        window.appState.documentHash = fallbackHash;
        return fallbackHash;
    }
}

// Render PDF with proper error handling
async function renderPDF(file) {
    console.log('Rendering PDF using PDF.js...');
    
    // Show loading indicator
    showLoadingIndicator();
    
    try {
        const fileReader = new FileReader();
        const fileData = await new Promise((resolve, reject) => {
            fileReader.onload = (e) => resolve(e.target.result);
            fileReader.onerror = (e) => reject(e);
            fileReader.readAsArrayBuffer(file);
        });
        
        const typedArray = new Uint8Array(fileData);
        
        // Make sure PDF.js is available
        if (typeof pdfjsLib === 'undefined') {
            throw new Error('PDF.js library not found!');
        }
        
        // Load the PDF using PDF.js
        const pdfDoc = await pdfjsLib.getDocument(typedArray).promise;
        console.log('PDF loaded, pages:', pdfDoc.numPages);
        
        // Initialize app state
        window.appState = window.appState || {};
        window.appState.totalPages = pdfDoc.numPages;
        window.appState.pageImages = [];
        window.appState.pageDimensions = [];
        window.appState.pdfDocument = pdfDoc;
        
        // Create a container for the pages
        const previewContainer = document.getElementById('file-preview-container');
        if (!previewContainer) {
            throw new Error('Preview container not found');
        }
        
        // Clear the container
        previewContainer.innerHTML = '';
        
        // Create a wrapper div for all pages
        const wrapper = document.createElement('div');
        wrapper.className = 'document-pages';
        wrapper.style.display = 'flex';
        wrapper.style.flexDirection = 'column';
        wrapper.style.gap = '20px';
        previewContainer.appendChild(wrapper);
        
        // Process each page
        for (let pageNum = 1; pageNum <= pdfDoc.numPages; pageNum++) {
            try {
                // Get the page
                const page = await pdfDoc.getPage(pageNum);
                
                // Get the viewport with a reasonable scale
                const viewport = page.getViewport({ scale: 1.5 });
                
                // Store page dimensions
                window.appState.pageDimensions[pageNum - 1] = {
                    width: viewport.width,
                    height: viewport.height
                };
                
                // Create a canvas for the page
                const canvas = document.createElement('canvas');
                canvas.className = 'preview-canvas';
                canvas.width = viewport.width;
                canvas.height = viewport.height;
                canvas.style.maxWidth = '100%';
                canvas.style.height = 'auto';
                canvas.style.display = 'block';
                
                // Create the page div
                const pageDiv = document.createElement('div');
                pageDiv.className = 'document-page';
                pageDiv.dataset.pageNum = pageNum;
                pageDiv.style.position = 'relative';
                pageDiv.style.marginBottom = '20px';
                pageDiv.style.border = '1px solid #ddd';
                pageDiv.style.backgroundColor = '#fff';
                pageDiv.style.boxShadow = '0 2px 4px rgba(0,0,0,0.1)';
                
                // Get the 2D context
                const context = canvas.getContext('2d');
                
                // Render the page
                await page.render({
                    canvasContext: context,
                    viewport: viewport
                }).promise;
                
                // Store the rendered page image
                window.appState.pageImages[pageNum - 1] = canvas.toDataURL();
                
                // Add the canvas to the page div
                pageDiv.appendChild(canvas);
                
                // Add page number indicator
                const pageNumIndicator = document.createElement('div');
                pageNumIndicator.className = 'page-number';
                pageNumIndicator.textContent = `Page ${pageNum}`;
                pageNumIndicator.style.position = 'absolute';
                pageNumIndicator.style.top = '10px';
                pageNumIndicator.style.right = '10px';
                pageNumIndicator.style.backgroundColor = 'rgba(0,0,0,0.5)';
                pageNumIndicator.style.color = '#fff';
                pageNumIndicator.style.padding = '4px 8px';
                pageNumIndicator.style.borderRadius = '4px';
                pageNumIndicator.style.fontSize = '12px';
                pageDiv.appendChild(pageNumIndicator);
                
                // Add the page div to the wrapper
                wrapper.appendChild(pageDiv);
                
                console.log(`Page ${pageNum} rendered`);
            } catch (error) {
                console.error(`Error rendering page ${pageNum}:`, error);
                throw error;
            }
        }
        
        // Add page navigation controls
        const navContainer = document.createElement('div');
        navContainer.className = 'page-navigation';
        navContainer.style.display = 'flex';
        navContainer.style.justifyContent = 'center';
        navContainer.style.gap = '10px';
        navContainer.style.marginTop = '20px';
        
        const prevBtn = document.createElement('button');
        prevBtn.className = 'btn btn-secondary';
        prevBtn.textContent = 'Previous';
        prevBtn.onclick = () => navigatePages('prev');
        
        const nextBtn = document.createElement('button');
        nextBtn.className = 'btn btn-secondary';
        nextBtn.textContent = 'Next';
        nextBtn.onclick = () => navigatePages('next');
        
        navContainer.appendChild(prevBtn);
        navContainer.appendChild(nextBtn);
        previewContainer.appendChild(navContainer);
        
        console.log('PDF rendering complete');
        
        // Update UI elements
        updateUIAfterRendering();
        
    } catch (error) {
        console.error('Error in PDF rendering:', error);
        alert('Error rendering PDF. The file may be corrupted or password protected.');
        renderPlaceholder(file); // Fallback
    } finally {
        // Always hide the loading indicator
        hideLoadingIndicator();
    }
}

// Render image with proper scaling
function renderImage(file) {
    console.log('Rendering image file...');
    
    // Show loading indicator
    showLoadingIndicator();
    
    const fileReader = new FileReader();
    fileReader.onload = function(e) {
        const img = new Image();
        img.onload = function() {
            console.log('Image loaded, dimensions:', img.width, 'x', img.height);
            
            // Store image dimensions
            window.appState.pageDimensions = [{
                width: img.width,
                height: img.height
            }];
            window.appState.totalPages = 1;
            
            // Create canvas for the image
            const previewContainer = document.getElementById('file-preview-container');
            if (previewContainer) {
                previewContainer.innerHTML = '';
                
                const canvasContainer = document.createElement('div');
                canvasContainer.className = 'canvas-container';
                canvasContainer.style.position = 'relative';
                canvasContainer.style.border = '1px solid #ddd';
                canvasContainer.style.margin = '10px 0';
                canvasContainer.style.overflow = 'auto';
                previewContainer.appendChild(canvasContainer);
                
                const canvas = document.createElement('canvas');
                canvas.className = 'image-preview';
                canvas.id = 'image-preview-canvas';
                
                // Set canvas dimensions with appropriate scaling
                const scale = Math.min(800 / img.width, 1100 / img.height);
                canvas.width = img.width * scale;
                canvas.height = img.height * scale;
                
                canvasContainer.appendChild(canvas);
                
                // Draw image on canvas
                const ctx = canvas.getContext('2d');
                ctx.drawImage(img, 0, 0, canvas.width, canvas.height);
                
                // Store the rendered image
                window.appState.pageImages = [canvas.toDataURL()];
                
                // Hide loading indicator
                hideLoadingIndicator();
                
                // Update UI elements
                updateUIAfterRendering();
            }
        };
        
        img.onerror = function(error) {
            console.error('Error loading image');
            hideLoadingIndicator();
            alert('Error loading image. The file may be corrupted or in an unsupported format.');
            renderPlaceholder(file); // Fallback
        };
        
        img.src = e.target.result;
    };
    
    fileReader.onerror = function(error) {
        console.error('Error reading file:', error);
        hideLoadingIndicator();
        alert('Error reading file. Please try again.');
    };
    
    fileReader.readAsDataURL(file);
}

// Render placeholder for unsupported files
function renderPlaceholder(file) {
    console.log('Rendering placeholder for file:', file.name);
    
    // Create a simple placeholder
    const previewContainer = document.getElementById('file-preview-container');
    if (previewContainer) {
        previewContainer.innerHTML = '';
        
        const placeholderContainer = document.createElement('div');
        placeholderContainer.className = 'placeholder-container';
        placeholderContainer.style.border = '1px solid #ddd';
        placeholderContainer.style.padding = '20px';
        placeholderContainer.style.textAlign = 'center';
        placeholderContainer.style.margin = '10px 0';
        
        const fileIcon = document.createElement('div');
        fileIcon.style.fontSize = '48px';
        fileIcon.style.marginBottom = '10px';
        fileIcon.innerHTML = '📄';
        
        const fileInfo = document.createElement('div');
        fileInfo.innerHTML = `
            <h4>${file.name}</h4>
            <p>File type: ${file.type || 'Unknown'}</p>
            <p>Size: ${Math.round(file.size / 1024)} KB</p>
            <p class="text-muted">Preview not available for this file type</p>
        `;
        
        placeholderContainer.appendChild(fileIcon);
        placeholderContainer.appendChild(fileInfo);
        previewContainer.appendChild(placeholderContainer);
        
        // Set default values for page dimensions
        window.appState.pageDimensions = [{
            width: 595, // A4 width in points
            height: 842  // A4 height in points
        }];
        window.appState.totalPages = 1;
        window.appState.pageImages = [];
        
        // Hide loading indicator
        hideLoadingIndicator();
        
        // Update UI elements
        updateUIAfterRendering();
    }
}

// Update UI elements after rendering
function updateUIAfterRendering() {
    // Enable next step button
    const step1NextBtn = document.getElementById('step1-next');
    if (step1NextBtn) {
        step1NextBtn.disabled = false;
    }
    
    // Enable page navigation if multiple pages
    if (window.appState.totalPages > 1) {
        const prevBtn = document.getElementById('prev-page');
        const nextBtn = document.getElementById('next-page');
        
        if (prevBtn) prevBtn.disabled = true; // First page
        if (nextBtn) nextBtn.disabled = window.appState.totalPages <= 1;
    }
}

// Initialize image upload for Step 5
function initImageUpload(elements) {
    if (!elements.documentImage) {
        return;
    }
    
    // Set up file input styles
    elements.documentImage.style.position = "absolute";
    elements.documentImage.style.width = "100%";
    elements.documentImage.style.height = "100%";
    elements.documentImage.style.top = "0";
    elements.documentImage.style.left = "0";
    elements.documentImage.style.opacity = "0";
    elements.documentImage.style.cursor = "pointer";
    elements.documentImage.style.zIndex = "10";
    
    // Handle image selection
    elements.documentImage.addEventListener('change', function() {
        if (this.files && this.files.length > 0) {
            handleImageSelection(this.files[0]);
        }
    });
    
    // Set up drag and drop for image area if it exists
    if (elements.imageDropArea) {
        elements.imageDropArea.style.position = "relative";
        elements.imageDropArea.style.cursor = "pointer";
        
        // Same fix for click handler
        elements.imageDropArea.onclick = function(e) {
            if (e.target !== elements.documentImage) {
                e.preventDefault();
                elements.documentImage.click();
            }
        };
        
        // Drag and drop handlers
        ['dragenter', 'dragover', 'dragleave', 'drop'].forEach(eventName => {
            elements.imageDropArea.addEventListener(eventName, function(e) {
                e.preventDefault();
                e.stopPropagation();
                
                if (eventName === 'dragenter' || eventName === 'dragover') {
                    this.classList.add('drag-active');
                } else {
                    this.classList.remove('drag-active');
                }
            });
        });
        
        elements.imageDropArea.addEventListener('drop', function(e) {
            this.classList.remove('drag-active');
            if (e.dataTransfer.files && e.dataTransfer.files.length > 0) {
                const file = e.dataTransfer.files[0];
                if (file.type.startsWith('image/')) {
                    handleImageSelection(file);
                } else {
                    alert('Please select an image file (JPEG, PNG, etc.)');
                }
            }
        });
    }
}

// Handle the selected image for Step 5
function handleImageSelection(file) {
    if (!file.type.startsWith('image/')) {
        alert('Please select an image file (JPEG, PNG, etc.)');
        return;
    }
    
    const elements = window.elements;
    
    // Store the image in app state
    window.appState.uploadedImage = file;
    
    // Display the file name and image preview in a visually appealing way
    if (elements.savedImage) {
        const reader = new FileReader();
        reader.onload = function(e) {
            elements.savedImage.innerHTML = `
                <div style="text-align: center;">
                    <img src="${e.target.result}" style="max-width: 100%; max-height: 150px; margin-bottom: 10px; border: 1px solid #ddd; padding: 5px;">
                    <div><strong>Selected image:</strong> ${file.name}</div>
                </div>
            `;
        };
        reader.readAsDataURL(file);
    }
    
    // Display image preview in the field template
    const previewReader = new FileReader();
    previewReader.onload = function(e) {
        if (elements.imagePreview) {
            elements.imagePreview.src = e.target.result;
            elements.imagePreview.style.display = 'block';
        }
        
        if (elements.imagePlaceholder) {
            elements.imagePlaceholder.style.display = 'none';
        }
    };
    previewReader.readAsDataURL(file);
    
    // Update field availability
    if (typeof window.updateFieldAvailability === 'function') {
        window.updateFieldAvailability();
    }
    
    // Show success message
    const successMsg = document.createElement('div');
    successMsg.className = 'alert alert-success';
    successMsg.textContent = 'Image uploaded successfully!';
    successMsg.style.marginTop = '10px';
    
    // Remove any existing success messages
    const existingMsg = elements.savedImage.parentNode.querySelector('.alert-success');
    if (existingMsg) {
        existingMsg.remove();
    }
    
    // Add success message after the image display
    elements.savedImage.parentNode.appendChild(successMsg);
    
    // Remove message after 3 seconds
    setTimeout(() => {
        successMsg.remove();
    }, 3000);
}

// Navigate between pages in a multi-page document
function navigatePages(direction) {
    const pageIndicator = document.getElementById('page-indicator');
    if (!pageIndicator) return;
    
    const pageInfo = pageIndicator.textContent.match(/Page (\d+) of (\d+)/);
    if (!pageInfo) return;
    
    let currentPage = parseInt(pageInfo[1]);
    const totalPages = parseInt(pageInfo[2]);
    
    if (direction === 'next' && currentPage < totalPages) {
        currentPage++;
    } else if (direction === 'prev' && currentPage > 1) {
        currentPage--;
    }
    
    // Update page indicator
    pageIndicator.textContent = `Page ${currentPage} of ${totalPages}`;
    
    // Update button states
    const prevButton = document.getElementById('prev-page');
    const nextButton = document.getElementById('next-page');
    
    if (prevButton) prevButton.disabled = (currentPage === 1);
    if (nextButton) nextButton.disabled = (currentPage === totalPages);
    
    // Show current page
    const allPages = document.querySelectorAll('.pdf-page');
    allPages.forEach(page => page.style.display = 'none');
    
    const currentPageElement = document.getElementById('pdf-page-' + currentPage);
    if (currentPageElement) {
        currentPageElement.style.display = 'block';
    }
}

// Show loading indicator
function showLoadingIndicator() {
    // Remove any existing loading indicators
    hideLoadingIndicator();
    
    // Create loading indicator
    const loader = document.createElement('div');
    loader.id = 'file-loading-indicator';
    loader.className = 'loading-indicator';
    loader.innerHTML = '<div class="spinner"></div><div class="loading-text">Processing document...</div>';
    
    // Add to document
    document.body.appendChild(loader);
}

// Hide loading indicator
function hideLoadingIndicator() {
    const loader = document.getElementById('file-loading-indicator');
    if (loader) {
        loader.remove();
    }
}

// Export functions
window.initFileHandling = initFileHandling;
window.handleFile = handleFile;
window.renderPDF = renderPDF;
window.renderImage = renderImage;
window.renderPlaceholder = renderPlaceholder;
window.navigatePages = navigatePages;
window.showLoadingIndicator = showLoadingIndicator;
window.hideLoadingIndicator = hideLoadingIndicator;
window.handleImageSelection = handleImageSelection;
window.generateDocumentHash = generateDocumentHash;