/**
 * GDPR Compliance Module
 * Handles data privacy concerns, storage, encryption, and data deletion requests
 */

// Initialize GDPR compliance module
function initGDPRCompliance() {
    console.log('Initializing GDPR compliance module...');
    
    // Add GDPR notice to the footer or appropriate location
    addGDPRNotice();
    
    // Add GDPR compliance to step 1
    addDataPrivacyNoticeToStep1();
}

/**
 * Add GDPR notice to the page
 */
function addGDPRNotice() {
    // Check if notice already exists
    if (document.getElementById('gdpr-notice')) return;
    
    // Create notice element
    const notice = document.createElement('div');
    notice.id = 'gdpr-notice';
    notice.className = 'gdpr-notice';
    notice.style.padding = '10px';
    notice.style.backgroundColor = '#f0f9ff';
    notice.style.borderTop = '1px solid #bae6fd';
    notice.style.marginTop = '30px';
    notice.style.fontSize = '12px';
    notice.style.color = '#666';
    notice.style.textAlign = 'center';
    
    notice.innerHTML = `
        <p>
            Your data is processed in accordance with GDPR regulations. 
            <a href="#" id="privacy-policy-link">Privacy Policy</a> | 
            <a href="#" id="data-deletion-link">Request Data Deletion</a>
        </p>
    `;
    
    // Append to document body or another appropriate container
    const container = document.querySelector('.container');
    if (container) {
        container.appendChild(notice);
    } else {
        document.body.appendChild(notice);
    }
    
    // Add event listeners
    document.getElementById('privacy-policy-link').addEventListener('click', function(e) {
        e.preventDefault();
        showPrivacyPolicy();
    });
    
    document.getElementById('data-deletion-link').addEventListener('click', function(e) {
        e.preventDefault();
        showDataDeletionForm();
    });
}

/**
 * Add data privacy notice to step 1
 */
function addDataPrivacyNoticeToStep1() {
    const step1 = document.getElementById('step1');
    if (!step1) return;
    
    // Check if notice already exists
    if (document.getElementById('data-privacy-notice')) return;
    
    // Create notice element
    const notice = document.createElement('div');
    notice.id = 'data-privacy-notice';
    notice.style.marginTop = '20px';
    notice.style.padding = '10px';
    notice.style.backgroundColor = '#f8fafc';
    notice.style.border = '1px solid #e2e8f0';
    notice.style.borderRadius = '4px';
    notice.style.fontSize = '14px';
    
    notice.innerHTML = `
        <h4 style="margin-top: 0;">Data Privacy Notice</h4>
        <p>The following personal data will be processed when using this e-signature service:</p>
        <ul>
            <li>Your name and email address</li>
            <li>Your IP address</li>
            <li>Your signature image</li>
            <li>Document hash and metadata</li>
            <li>Timestamps of your actions</li>
        </ul>
        <p>This data is necessary for the electronic signature process and is processed in accordance with GDPR Article 6(1)(b) - necessary for the performance of a contract.</p>
        <p>
            <label>
                <input type="checkbox" id="data-privacy-checkbox"> 
                I consent to the processing of my personal data for the purpose of electronic signing.
            </label>
        </p>
    `;
    
    // Add the notice to step 1
    const card = step1.querySelector('.card');
    if (card) {
        // Insert before the continue button
        const continueBtn = card.querySelector('#step1-next');
        if (continueBtn) {
            card.insertBefore(notice, continueBtn);
        } else {
            card.appendChild(notice);
        }
    }
    
    // Modify step 1 next button to check for consent
    const nextBtn = document.getElementById('step1-next');
    if (nextBtn) {
        const originalClickHandler = nextBtn.onclick;
        nextBtn.onclick = function(e) {
            const checkbox = document.getElementById('data-privacy-checkbox');
            if (!checkbox || !checkbox.checked) {
                e.preventDefault();
                alert('Please consent to data processing before continuing.');
                return false;
            }
            
            // Call the original click handler
            if (typeof originalClickHandler === 'function') {
                return originalClickHandler.call(this, e);
            }
        };
    }
}

/**
 * Show privacy policy modal
 */
function showPrivacyPolicy() {
    // Create modal container
    const modal = document.createElement('div');
    modal.className = 'modal';
    modal.style.position = 'fixed';
    modal.style.top = '0';
    modal.style.left = '0';
    modal.style.width = '100%';
    modal.style.height = '100%';
    modal.style.backgroundColor = 'rgba(0, 0, 0, 0.5)';
    modal.style.display = 'flex';
    modal.style.justifyContent = 'center';
    modal.style.alignItems = 'center';
    modal.style.zIndex = '1000';
    
    // Create modal content
    const modalContent = document.createElement('div');
    modalContent.className = 'modal-content';
    modalContent.style.backgroundColor = '#fff';
    modalContent.style.padding = '20px';
    modalContent.style.borderRadius = '8px';
    modalContent.style.maxWidth = '800px';
    modalContent.style.maxHeight = '80vh';
    modalContent.style.overflow = 'auto';
    
    modalContent.innerHTML = `
        <h2>Privacy Policy</h2>
        <p><strong>Last Updated:</strong> ${new Date().toLocaleDateString()}</p>
        
        <h3>1. Introduction</h3>
        <p>This Privacy Policy explains how we collect, use, and protect your personal data when you use our e-signature service.</p>
        
        <h3>2. Data Controller</h3>
        <p>We are the data controller for the personal data collected through this service.</p>
        
        <h3>3. Personal Data We Collect</h3>
        <p>We collect the following personal data:</p>
        <ul>
            <li>Name and email address</li>
            <li>IP address</li>
            <li>Your signature image</li>
            <li>Document hash and metadata</li>
            <li>Timestamps of your actions</li>
        </ul>
        
        <h3>4. Purpose and Legal Basis for Processing</h3>
        <p>We process your data for the following purposes:</p>
        <ul>
            <li>To provide the e-signature service (legal basis: performance of a contract)</li>
            <li>To comply with legal obligations regarding electronic signatures (legal basis: legal obligation)</li>
            <li>To maintain audit logs for security purposes (legal basis: legitimate interest)</li>
        </ul>
        
        <h3>5. Data Retention</h3>
        <p>We retain your personal data for as long as necessary to fulfill the purposes for which we collected it, including for the purposes of satisfying any legal, accounting, or reporting requirements.</p>
        
        <h3>6. Your Rights</h3>
        <p>Under the GDPR, you have the following rights:</p>
        <ul>
            <li>Right to access</li>
            <li>Right to rectification</li>
            <li>Right to erasure ('right to be forgotten')</li>
            <li>Right to restriction of processing</li>
            <li>Right to data portability</li>
            <li>Right to object</li>
        </ul>
        
        <h3>7. Data Security</h3>
        <p>We implement appropriate technical and organizational measures to protect your personal data.</p>
        
        <h3>8. Contact Information</h3>
        <p>For any questions regarding this Privacy Policy, please contact us at privacy@example.com.</p>
        
        <button id="close-privacy-policy" style="padding: 10px 20px; background-color: #4299e1; color: white; border: none; border-radius: 4px; cursor: pointer; margin-top: 20px;">Close</button>
    `;
    
    modal.appendChild(modalContent);
    document.body.appendChild(modal);
    
    // Add close event listener
    document.getElementById('close-privacy-policy').addEventListener('click', function() {
        document.body.removeChild(modal);
    });
}

/**
 * Show data deletion request form
 */
function showDataDeletionForm() {
    // Create modal container
    const modal = document.createElement('div');
    modal.className = 'modal';
    modal.style.position = 'fixed';
    modal.style.top = '0';
    modal.style.left = '0';
    modal.style.width = '100%';
    modal.style.height = '100%';
    modal.style.backgroundColor = 'rgba(0, 0, 0, 0.5)';
    modal.style.display = 'flex';
    modal.style.justifyContent = 'center';
    modal.style.alignItems = 'center';
    modal.style.zIndex = '1000';
    
    // Create modal content
    const modalContent = document.createElement('div');
    modalContent.className = 'modal-content';
    modalContent.style.backgroundColor = '#fff';
    modalContent.style.padding = '20px';
    modalContent.style.borderRadius = '8px';
    modalContent.style.maxWidth = '500px';
    
    modalContent.innerHTML = `
        <h2>Request Data Deletion</h2>
        <p>Please provide your email address to request the deletion of your personal data.</p>
        
        <div style="margin-bottom: 15px;">
            <label for="deletion-email" style="display: block; margin-bottom: 5px;">Email Address</label>
            <input type="email" id="deletion-email" style="width: 100%; padding: 8px; border: 1px solid #ddd; border-radius: 4px;">
        </div>
        
        <div style="margin-bottom: 15px;">
            <label for="deletion-reason" style="display: block; margin-bottom: 5px;">Reason for Deletion (Optional)</label>
            <textarea id="deletion-reason" style="width: 100%; padding: 8px; border: 1px solid #ddd; border-radius: 4px; height: 100px;"></textarea>
        </div>
        
        <div style="display: flex; justify-content: space-between;">
            <button id="cancel-deletion" style="padding: 10px 20px; background-color: #e2e8f0; border: none; border-radius: 4px; cursor: pointer;">Cancel</button>
            <button id="submit-deletion" style="padding: 10px 20px; background-color: #4299e1; color: white; border: none; border-radius: 4px; cursor: pointer;">Submit Request</button>
        </div>
    `;
    
    modal.appendChild(modalContent);
    document.body.appendChild(modal);
    
    // Add event listeners
    document.getElementById('cancel-deletion').addEventListener('click', function() {
        document.body.removeChild(modal);
    });
    
    document.getElementById('submit-deletion').addEventListener('click', function() {
        const email = document.getElementById('deletion-email').value;
        const reason = document.getElementById('deletion-reason').value;
        
        if (!email) {
            alert('Please enter your email address.');
            return;
        }
        
        // Process deletion request
        processDataDeletionRequest(email, reason);
        
        // Show confirmation
        modalContent.innerHTML = `
            <h2>Request Submitted</h2>
            <p>Your data deletion request has been submitted. We will process your request within 30 days.</p>
            <p>A confirmation email has been sent to ${email}.</p>
            <button id="close-deletion" style="padding: 10px 20px; background-color: #4299e1; color: white; border: none; border-radius: 4px; cursor: pointer; margin-top: 20px;">Close</button>
        `;
        
        document.getElementById('close-deletion').addEventListener('click', function() {
            document.body.removeChild(modal);
        });
    });
}

/**
 * Process data deletion request (mock implementation)
 * @param {string} email - User's email address
 * @param {string} reason - Reason for deletion request
 */
function processDataDeletionRequest(email, reason) {
    console.log(`Data deletion request for ${email}: ${reason}`);
    
    // Store deletion request in localStorage for demo purposes
    // In a real implementation, this would be sent to your server
    const deletionRequests = JSON.parse(localStorage.getItem('deletionRequests') || '[]');
    deletionRequests.push({
        email,
        reason,
        timestamp: Date.now(),
        status: 'pending'
    });
    localStorage.setItem('deletionRequests', JSON.stringify(deletionRequests));
    
    // Create audit log for the deletion request
    if (window.eidasCompliance && window.eidasCompliance.createAuditLog) {
        window.eidasCompliance.createAuditLog({
            userId: email,
            action: 'data_deletion_requested',
            timestamp: Math.floor(Date.now() / 1000)
        });
    }
}

/**
 * Encrypt sensitive data using AES-GCM
 * @param {string} data - Data to encrypt
 * @param {string} password - Password for encryption
 * @returns {Promise<string>} - Promise resolving to encrypted data
 */
async function encryptData(data, password) {
    try {
        // Convert data and password to ArrayBuffer
        const encoder = new TextEncoder();
        const dataBuffer = encoder.encode(data);
        const passwordBuffer = encoder.encode(password);
        
        // Derive key from password
        const salt = crypto.getRandomValues(new Uint8Array(16));
        const iv = crypto.getRandomValues(new Uint8Array(12));
        
        const keyMaterial = await crypto.subtle.importKey(
            'raw',
            passwordBuffer,
            { name: 'PBKDF2' },
            false,
            ['deriveBits', 'deriveKey']
        );
        
        const key = await crypto.subtle.deriveKey(
            {
                name: 'PBKDF2',
                salt,
                iterations: 100000,
                hash: 'SHA-256'
            },
            keyMaterial,
            { name: 'AES-GCM', length: 256 },
            false,
            ['encrypt']
        );
        
        // Encrypt the data
        const encryptedData = await crypto.subtle.encrypt(
            {
                name: 'AES-GCM',
                iv
            },
            key,
            dataBuffer
        );
        
        // Combine salt, iv, and encrypted data
        const result = new Uint8Array(salt.length + iv.length + encryptedData.byteLength);
        result.set(salt, 0);
        result.set(iv, salt.length);
        result.set(new Uint8Array(encryptedData), salt.length + iv.length);
        
        // Convert to base64
        return btoa(String.fromCharCode.apply(null, result));
    } catch (error) {
        console.error('Error encrypting data:', error);
        throw error;
    }
}

/**
 * Decrypt data encrypted with AES-GCM
 * @param {string} encryptedData - Base64 encoded encrypted data
 * @param {string} password - Password for decryption
 * @returns {Promise<string>} - Promise resolving to decrypted data
 */
async function decryptData(encryptedData, password) {
    try {
        // Convert from base64
        const encryptedBytes = new Uint8Array(atob(encryptedData).split('').map(char => char.charCodeAt(0)));
        
        // Extract salt, iv, and data
        const salt = encryptedBytes.slice(0, 16);
        const iv = encryptedBytes.slice(16, 16 + 12);
        const data = encryptedBytes.slice(16 + 12);
        
        // Convert password to ArrayBuffer
        const encoder = new TextEncoder();
        const passwordBuffer = encoder.encode(password);
        
        // Derive key from password
        const keyMaterial = await crypto.subtle.importKey(
            'raw',
            passwordBuffer,
            { name: 'PBKDF2' },
            false,
            ['deriveBits', 'deriveKey']
        );
        
        const key = await crypto.subtle.deriveKey(
            {
                name: 'PBKDF2',
                salt,
                iterations: 100000,
                hash: 'SHA-256'
            },
            keyMaterial,
            { name: 'AES-GCM', length: 256 },
            false,
            ['decrypt']
        );
        
        // Decrypt the data
        const decryptedBuffer = await crypto.subtle.decrypt(
            {
                name: 'AES-GCM',
                iv
            },
            key,
            data
        );
        
        // Convert to string
        const decoder = new TextDecoder();
        return decoder.decode(decryptedBuffer);
    } catch (error) {
        console.error('Error decrypting data:', error);
        throw error;
    }
}

// Export functions for use in other modules
window.gdprCompliance = {
    init: initGDPRCompliance,
    showPrivacyPolicy,
    showDataDeletionForm,
    processDataDeletionRequest,
    encryptData,
    decryptData
};