/**
 * Improved Signature Pad Module
 * Handles drawing and saving signatures with transparent background
 */

// Initialize signature pad
function initSignaturePad(elements) {
    if (!elements.signatureCanvas) {
        console.error('Signature canvas element not found');
        return;
    }
    
    console.log('Initializing improved signature pad...');
    
    const canvas = elements.signatureCanvas;
    const ctx = canvas.getContext('2d');
    let isDrawing = false;
    let lastX = 0;
    let lastY = 0;
    
    // Initialize signature canvas with transparent background
    ctx.clearRect(0, 0, canvas.width, canvas.height);
    
    // Set stroke style
    ctx.lineWidth = 2.5;
    ctx.strokeStyle = '#000000';
    ctx.lineJoin = 'round';
    ctx.lineCap = 'round';
    
    // Add a light grey grid to help users see the canvas area
    drawGrid();
    
    function drawGrid() {
        // Save the current state
        ctx.save();
        
        // Set grid properties
        ctx.strokeStyle = '#EEEEEE';
        ctx.lineWidth = 0.5;
        
        // Draw horizontal lines
        const gridStep = 20;
        for (let y = 0; y < canvas.height; y += gridStep) {
            ctx.beginPath();
            ctx.moveTo(0, y);
            ctx.lineTo(canvas.width, y);
            ctx.stroke();
        }
        
        // Draw vertical lines
        for (let x = 0; x < canvas.width; x += gridStep) {
            ctx.beginPath();
            ctx.moveTo(x, 0);
            ctx.lineTo(x, canvas.height);
            ctx.stroke();
        }
        
        // Restore context state
        ctx.restore();
    }
    
    // Mouse event listeners
    canvas.addEventListener('mousedown', startDrawing);
    canvas.addEventListener('mousemove', draw);
    canvas.addEventListener('mouseup', stopDrawing);
    canvas.addEventListener('mouseout', stopDrawing);
    
    // Touch event listeners for mobile
    canvas.addEventListener('touchstart', function(e) {
        e.preventDefault();
        const touch = e.touches[0];
        const rect = canvas.getBoundingClientRect();
        const x = touch.clientX - rect.left;
        const y = touch.clientY - rect.top;
        
        lastX = x;
        lastY = y;
        isDrawing = true;
    });
    
    canvas.addEventListener('touchmove', function(e) {
        if (!isDrawing) return;
        e.preventDefault();
        
        const touch = e.touches[0];
        const rect = canvas.getBoundingClientRect();
        const x = touch.clientX - rect.left;
        const y = touch.clientY - rect.top;
        
        ctx.beginPath();
        ctx.moveTo(lastX, lastY);
        ctx.lineTo(x, y);
        ctx.stroke();
        
        lastX = x;
        lastY = y;
    });
    
    canvas.addEventListener('touchend', function(e) {
        isDrawing = false;
    });
    
    // Button event listeners
    if (elements.clearSignatureBtn) {
        elements.clearSignatureBtn.addEventListener('click', clearSignature);
    }
    
    if (elements.saveSignatureBtn) {
        elements.saveSignatureBtn.addEventListener('click', saveSignature);
    }
    
    // Initialize date handling if the element exists
    initDateHandling(elements);
    
    function startDrawing(e) {
        isDrawing = true;
        const rect = canvas.getBoundingClientRect();
        lastX = e.clientX - rect.left;
        lastY = e.clientY - rect.top;
    }
    
    function draw(e) {
        if (!isDrawing) return;
        
        const rect = canvas.getBoundingClientRect();
        const x = e.clientX - rect.left;
        const y = e.clientY - rect.top;
        
        ctx.beginPath();
        ctx.moveTo(lastX, lastY);
        ctx.lineTo(x, y);
        ctx.stroke();
        
        lastX = x;
        lastY = y;
    }
    
    function stopDrawing() {
        isDrawing = false;
    }
    
    function clearSignature() {
        // Clear the canvas completely - transparent background
        ctx.clearRect(0, 0, canvas.width, canvas.height);
        
        // Draw the grid again
        drawGrid();
        
        if (elements.savedSignature) {
            elements.savedSignature.innerHTML = '<div>Your signature will appear here after saving</div>';
        }
        
        if (elements.signatureImagePreview) {
            elements.signatureImagePreview.style.display = 'none';
        }
        
        if (elements.signaturePlaceholder) {
            elements.signaturePlaceholder.style.display = 'block';
        }
        
        window.appState.signatureImage = null;
        
        if (typeof window.updateFieldAvailability === 'function') {
            window.updateFieldAvailability();
        }
    }
    
    function saveSignature() {
        if (isCanvasEmpty()) {
            alert('Please draw your signature first');
            return;
        }
        
        try {
            // Create a temporary canvas for the signature without the grid
            const tempCanvas = document.createElement('canvas');
            tempCanvas.width = canvas.width;
            tempCanvas.height = canvas.height;
            const tempCtx = tempCanvas.getContext('2d');
            
            // Get the signature data only (no background, no grid)
            const imageData = ctx.getImageData(0, 0, canvas.width, canvas.height);
            tempCtx.putImageData(imageData, 0, 0);
            
            // Remove the grid lines by setting a threshold for very light pixels
            const cleanedData = tempCtx.getImageData(0, 0, tempCanvas.width, tempCanvas.height);
            const data = cleanedData.data;
            
            // Filter out very light grey colors (grid)
            for (let i = 0; i < data.length; i += 4) {
                // If pixel is very light grey (grid lines)
                if (data[i] > 220 && data[i+1] > 220 && data[i+2] > 220) {
                    // Make it fully transparent
                    data[i+3] = 0;
                }
            }
            
            tempCtx.putImageData(cleanedData, 0, 0);
            
            // Get the transparent PNG
            window.appState.signatureImage = tempCanvas.toDataURL('image/png');
            
            if (elements.savedSignature) {
                // Display with checkered background to show transparency
                elements.savedSignature.innerHTML = `
                    <div style="background-image: linear-gradient(45deg, #f0f0f0 25%, transparent 25%), 
                                linear-gradient(-45deg, #f0f0f0 25%, transparent 25%), 
                                linear-gradient(45deg, transparent 75%, #f0f0f0 75%), 
                                linear-gradient(-45deg, transparent 75%, #f0f0f0 75%);
                        background-size: 20px 20px;
                        background-position: 0 0, 0 10px, 10px -10px, -10px 0px;">
                        <img src="${window.appState.signatureImage}" alt="Your signature" style="max-width: 100%; max-height: 80px;">
                    </div>
                `;
            }
            
            if (elements.signatureImagePreview) {
                elements.signatureImagePreview.src = window.appState.signatureImage;
                elements.signatureImagePreview.style.display = 'block';
            }
            
            if (elements.signaturePlaceholder) {
                elements.signaturePlaceholder.style.display = 'none';
            }
            
            // Show success message
            const successMsg = document.createElement('div');
            successMsg.className = 'alert alert-success';
            successMsg.textContent = 'Signature saved successfully!';
            successMsg.style.marginTop = '10px';
            
            // Remove any existing success messages
            const existingMsg = canvas.parentNode.querySelector('.alert-success');
            if (existingMsg) {
                existingMsg.remove();
            }
            
            // Add success message after canvas
            canvas.parentNode.insertBefore(successMsg, canvas.nextSibling);
            
            // Remove message after 3 seconds
            setTimeout(() => {
                successMsg.remove();
            }, 3000);
            
            if (typeof window.updateFieldAvailability === 'function') {
                window.updateFieldAvailability();
            }
        } catch (error) {
            console.error('Error saving signature:', error);
            alert('Error saving signature. Please try again.');
        }
    }
    
    function isCanvasEmpty() {
        // This function detects if the canvas is empty 
        // (other than grid lines which are very light)
        const pixelData = ctx.getImageData(0, 0, canvas.width, canvas.height).data;
        
        for (let i = 0; i < pixelData.length; i += 4) {
            const red = pixelData[i];
            const green = pixelData[i+1];
            const blue = pixelData[i+2];
            const alpha = pixelData[i+3];
            
            // If we find any non-transparent pixel that's not very light (not grid)
            if (alpha > 0 && (red < 220 || green < 220 || blue < 220)) {
                return false; // Canvas is not empty
            }
        }
        return true; // Canvas is empty or only has grid lines
    }
}

// Initialize date handling
function initDateHandling(elements) {
    if (!elements.documentDate) {
        return;
    }
    
    // Set min and max date values
    const today = new Date();
    elements.documentDate.min = "2024-01-01";
    elements.documentDate.max = "2100-12-31";
    
    elements.documentDate.addEventListener('change', function() {
        const selectedDate = new Date(this.value);
        const year = selectedDate.getFullYear();
        
        if (year < 2024 || year > 2100) {
            alert('Please select a date between 2024 and 2100');
            this.value = '';
            return;
        }
        
        window.appState.selectedDate = this.value;
        const formattedDate = selectedDate.toLocaleDateString();
        
        if (elements.savedDate) {
            elements.savedDate.innerHTML = `<div>${formattedDate}</div>`;
        }
        
        if (elements.dateValue) {
            elements.dateValue.textContent = formattedDate;
        }
        
        if (typeof window.updateFieldAvailability === 'function') {
            window.updateFieldAvailability();
        }
    });
}

// Handle custom text fields
function initCustomTextFields(elements) {
    if (!elements.addCustomFieldBtn || !elements.customFieldTitle || !elements.customFieldContent || !elements.customFieldList) {
        return;
    }
    
    elements.addCustomFieldBtn.addEventListener('click', function() {
        const title = elements.customFieldTitle.value.trim();
        const content = elements.customFieldContent.value.trim();
        
        if (!title || !content) {
            alert('Please enter both a title and content for the custom field');
            return;
        }
        
        // Add to app state
        if (!window.appState.customTextFields) {
            window.appState.customTextFields = [];
        }
        
        window.appState.customTextFields.push({ title, content });
        
        // Add to UI
        const fieldItem = document.createElement('div');
        fieldItem.className = 'custom-field-item';
        fieldItem.innerHTML = `
            <div>
                <strong>${title}:</strong> ${content}
            </div>
            <button class="btn btn-danger btn-sm remove-custom-field">Remove</button>
        `;
        
        elements.customFieldList.appendChild(fieldItem);
        
        // Add remove handler
        const removeBtn = fieldItem.querySelector('.remove-custom-field');
        if (removeBtn) {
            removeBtn.addEventListener('click', function() {
                // Remove from DOM
                fieldItem.remove();
                
                // Remove from app state
                window.appState.customTextFields = window.appState.customTextFields.filter(
                    field => field.title !== title || field.content !== content
                );
            });
        }
        
        // Clear input fields
        elements.customFieldTitle.value = '';
        elements.customFieldContent.value = '';
    });
}

// Initialize recipient handling
function initRecipientHandling(elements) {
    if (!elements.addRecipientBtn || !elements.recipientName || !elements.recipientEmail || !elements.recipientsList) {
        return;
    }
    
    elements.addRecipientBtn.addEventListener('click', function() {
        const name = elements.recipientName.value.trim();
        const email = elements.recipientEmail.value.trim();
        
        if (!name || !email) {
            alert('Please enter both a name and email for the recipient');
            return;
        }
        
        // Validate email format
        const emailRegex = /^[^\s@]+@[^\s@]+\.[^\s@]+$/;
        if (!emailRegex.test(email)) {
            alert('Please enter a valid email address');
            return;
        }
        
        // Add to app state
        if (!window.appState.recipients) {
            window.appState.recipients = [];
        }
        
        window.appState.recipients.push({ name, email });
        
        // Add to UI
        const recipientItem = document.createElement('div');
        recipientItem.className = 'custom-field-item';
        recipientItem.innerHTML = `
            <div>
                <strong>${name}</strong> (${email})
            </div>
            <button class="btn btn-danger btn-sm remove-recipient">Remove</button>
        `;
        
        elements.recipientsList.appendChild(recipientItem);
        
        // Add remove handler
        const removeBtn = recipientItem.querySelector('.remove-recipient');
        if (removeBtn) {
            removeBtn.addEventListener('click', function() {
                // Remove from DOM
                recipientItem.remove();
                
                // Remove from app state
                window.appState.recipients = window.appState.recipients.filter(
                    recipient => recipient.name !== name || recipient.email !== email
                );
                
                // Update field availability
                if (typeof window.updateFieldAvailability === 'function') {
                    window.updateFieldAvailability();
                }
            });
        }
        
        // Clear input fields
        elements.recipientName.value = '';
        elements.recipientEmail.value = '';
        
        // Update field availability
        if (typeof window.updateFieldAvailability === 'function') {
            window.updateFieldAvailability();
        }
    });
}

// Export functions for use in other modules
window.initSignaturePad = initSignaturePad;
window.initDateHandling = initDateHandling;
window.initCustomTextFields = initCustomTextFields;
window.initRecipientHandling = initRecipientHandling;