<?php
/**
 * OTP Generation and Email Sending
 * Handles OTP generation, storage, and email delivery
 */
// Enable error reporting and detailed logging
error_reporting(E_ALL);
ini_set('display_errors', 0); // Don't display errors to users
ini_set('log_errors', 1);
ini_set('error_log', __DIR__ . '/logs/otp_error.log');
error_log("====== " . date('Y-m-d H:i:s') . " - OTP Send Operation Started ======");

// Required files
require_once 'config.php';
require_once 'db-config.php';
require_once 'vendor/autoload.php';
use PHPMailer\PHPMailer\PHPMailer;
use PHPMailer\PHPMailer\Exception;

// Set headers for JSON response
header('Content-Type: application/json');

// Only allow POST requests
if ($_SERVER['REQUEST_METHOD'] !== 'POST') {
    http_response_code(405);
    echo json_encode(['success' => false, 'message' => 'Method Not Allowed']);
    exit;
}

// Create logs directory if it doesn't exist
if (!is_dir(LOG_DIR)) {
    mkdir(LOG_DIR, 0755, true);
}

// Function to log errors
function logError($message) {
    $log_file = LOG_DIR . '/otp_errors.log';
    $timestamp = date('Y-m-d H:i:s');
    $log_entry = "[$timestamp] $message\n";
    file_put_contents($log_file, $log_entry, FILE_APPEND);
}

try {
    // Get request data from both POST and JSON input
    $request_body = file_get_contents('php://input');
    $json_data = json_decode($request_body, true);
    
    // Use data from either POST or JSON body
    $email = isset($_POST['email']) ? filter_var($_POST['email'], FILTER_SANITIZE_EMAIL) : 
             (isset($json_data['email']) ? filter_var($json_data['email'], FILTER_SANITIZE_EMAIL) : null);
    
    $document_id = isset($_POST['document_id']) ? $_POST['document_id'] : 
                   (isset($json_data['document_id']) ? $json_data['document_id'] : null);
    
    error_log("Processing OTP request for email: $email, document: $document_id");
    
    // Validate email
    if (!$email || !filter_var($email, FILTER_VALIDATE_EMAIL)) {
        http_response_code(400);
        echo json_encode(['success' => false, 'message' => 'Invalid email address']);
        error_log("Invalid email provided: $email");
        exit;
    }
    
    // Check rate limiting
    $ip = getClientIP();
    $rate_key = md5($ip . $email);
    
    $stmt = $db_conn->prepare("SELECT COUNT(*) as count FROM otp_requests WHERE request_key = :key AND request_time > :time");
    $time_limit = time() - OTP_RATE_WINDOW;
    $stmt->bindParam(':key', $rate_key);
    $stmt->bindParam(':time', $time_limit);
    $stmt->execute();
    $result = $stmt->fetch();
    
    if ($result && $result['count'] >= OTP_RATE_LIMIT) {
        logError("Rate limit exceeded for $email from IP $ip");
        http_response_code(429);
        echo json_encode([
            'success' => false, 
            'message' => 'Too many attempts. Please try again later.'
        ]);
        exit;
    }
    
    // Generate OTP
    $otp = '';
    for ($i = 0; $i < OTP_LENGTH; $i++) {
        $otp .= mt_rand(0, 9);
    }
    
    // Store OTP in database
    $stmt = $db_conn->prepare("INSERT INTO otp_requests (email, otp, document_id, request_key, request_time, expires_at) VALUES (:email, :otp, :doc_id, :key, :time, :expires)");
    $now = time();
    $expires = $now + (OTP_EXPIRY_MINUTES * 60);
    
    $stmt->bindParam(':email', $email);
    $stmt->bindParam(':otp', $otp);
    $stmt->bindParam(':doc_id', $document_id);
    $stmt->bindParam(':key', $rate_key);
    $stmt->bindParam(':time', $now);
    $stmt->bindParam(':expires', $expires);
    $stmt->execute();
    
    error_log("OTP generated for $email: $otp (expires at " . date('Y-m-d H:i:s', $expires) . ")");
    
    // For development, save the OTP to a file
    if (APP_ENV === 'development') {
        $dev_log = LOG_DIR . '/development_otps.txt';
        $dev_entry = date('Y-m-d H:i:s') . " - Email: $email, OTP: $otp, Document: $document_id\n";
        file_put_contents($dev_log, $dev_entry, FILE_APPEND);
        error_log("Development mode: OTP saved to file");
    }
    
    // Send OTP via email
    $sent = sendOtpEmail($email, $otp, $document_id);
    
    if ($sent) {
        error_log("OTP email sent successfully to $email");
        echo json_encode(['success' => true, 'message' => 'OTP sent successfully']);
    } else {
        // In development mode, return the OTP for testing
        if (APP_ENV === 'development') {
            logError("Development mode: OTP generated but email not sent. OTP: $otp");
            echo json_encode([
                'success' => true,
                'message' => 'Development mode: OTP generated but email not sent',
                'dev_otp' => $otp
            ]);
        } else {
            logError("Failed to send OTP email to $email");
            http_response_code(500);
            echo json_encode(['success' => false, 'message' => 'Failed to send OTP email']);
        }
    }
} catch (Exception $e) {
    logError("Error in send-otp.php: " . $e->getMessage());
    error_log("Exception in send-otp.php: " . $e->getMessage());
    http_response_code(500);
    echo json_encode([
        'success' => false, 
        'message' => 'An error occurred while processing your request. Please try again later.'
    ]);
}

/**
 * Send OTP via email using PHPMailer
 */
function sendOtpEmail($email, $otp, $document_id = null) {
    try {
        error_log("Starting email send process to $email");
        
        // Get document information if document ID is provided
        $document_info = '';
        if ($document_id) {
            global $db_conn;
            $stmt = $db_conn->prepare("SELECT file_name FROM documents WHERE document_id = :doc_id LIMIT 1");
            $stmt->bindParam(':doc_id', $document_id);
            $stmt->execute();
            $result = $stmt->fetch();
            
            if ($result && isset($result['file_name'])) {
                $document_info = " pentru document: " . htmlspecialchars($result['file_name']);
                error_log("Found document info: " . $result['file_name']);
            }
        }
        
        $mail = new PHPMailer(true);
        
        // Server settings
        $mail->isSMTP();
        $mail->Host = SMTP_HOST;
        $mail->SMTPAuth = true;
        $mail->Username = SMTP_USERNAME;
        $mail->Password = SMTP_PASSWORD;
        $mail->SMTPSecure = SMTP_SECURE;
        $mail->Port = SMTP_PORT;
        $mail->CharSet = 'UTF-8';
        
        // Enable debug output in development mode
        if (APP_ENV === 'development') {
            $mail->SMTPDebug = 2;
            $mail->Debugoutput = function($str, $level) {
                error_log("SMTP Debug: $str");
            };
        } else {
            $mail->SMTPDebug = 0;
        }
        
        // Add SMTPOptions for better compatibility
        $mail->SMTPOptions = array(
            'ssl' => array(
                'verify_peer' => false,
                'verify_peer_name' => false,
                'allow_self_signed' => true
            )
        );
        
        // Log SMTP connection details
        error_log("SMTP Settings - Host: $mail->Host, Port: $mail->Port, Username: $mail->Username");
        
        // Recipients
        $mail->setFrom(SMTP_FROM_EMAIL, SMTP_FROM_NAME);
        $mail->addAddress($email);
        
        // Content
        $mail->isHTML(true);
        $mail->Subject = 'Inkless-Flow: Your Verification Code';
        
        // Email template
        $mail->Body = "
            <html>
            <head>
                <title>Your Verification Code for Inkless-Flow</title>
                <style>
                    body { font-family: Arial, sans-serif; line-height: 1.6; color: #333; }
                    .container { max-width: 600px; margin: 0 auto; padding: 20px; }
                    .header { text-align: center; padding: 20px 0; }
                    .otp-code { font-size: 24px; font-weight: bold; text-align: center; 
                                letter-spacing: 5px; padding: 20px; background-color: #f5f5f5; 
                                border-radius: 5px; margin: 20px 0; }
                    .footer { padding: 20px 0; font-size: 12px; color: #777; text-align: center; }
                </style>
            </head>
            <body>
                <div class='container'>
                    <div class='header'>
                        <h1>Verification Code for Inkless-Flow</h1>
                    </div>
                    <p>Hello,</p>
                    <p>You requested a verification code to electronically sign a document" . $document_info . ".</p>
                    <p>Your verification code is:</p>
                    <div class='otp-code'>" . $otp . "</div>
                    <p>This code will expire in 5 minutes. Do not share this code with anyone.</p>
                    <p>If you did not request this code, please ignore this email.</p>
                    <p>Best regards,<br>The Inkless-Flow Team</p>
                    <div class='footer'>
                        <p>This email was sent automatically. Please do not reply to this email.</p>
                        <p>&copy; " . date('Y') . " Inkless-Flow. All rights reserved.</p>
                    </div>
                </div>
            </body>
            </html>";
        
        $mail->AltBody = "Hello,\n\nYour verification code is: $otp\nThis code will expire in 5 minutes.\n\nBest regards,\nThe Inkless-Flow Team";
        
        // Send email
        $result = $mail->send();
        error_log("Email send result: " . ($result ? "Success" : "Failed"));
        return $result;
    } catch (Exception $e) {
        error_log("Failed to send OTP email to $email: " . $e->getMessage());
        
        // In development mode, log the OTP for testing
        if (APP_ENV === 'development') {
            $log_file = LOG_DIR . '/otp.log';
            $log_entry = date('Y-m-d H:i:s') . " - Development OTP for $email: $otp\n";
            file_put_contents($log_file, $log_entry, FILE_APPEND);
        }
        
        return false;
    }
}