<?php
/**
 * Document Signing API
 * Handles document signing with eIDAS compliance
 */

// Enable error reporting in development (disable in production)
error_reporting(E_ALL);
ini_set('display_errors', 0);
ini_set('log_errors', 1);
ini_set('error_log', __DIR__ . '/logs/signing_error.log');

// Required files
require_once 'config.php';
require_once 'db-config.php';
require_once 'eidas-compliance.php';
require_once 'document-storage.php'; // Include the new document storage file

// Set headers for JSON response
header('Content-Type: application/json');

// Only allow POST requests
if ($_SERVER['REQUEST_METHOD'] !== 'POST') {
    http_response_code(405);
    echo json_encode(['success' => false, 'message' => 'Method not allowed']);
    exit;
}

// Get POST data
$request_body = file_get_contents('php://input');
$data = json_decode($request_body, true);

// Validate input data
if (!$data || !isset($data['document_id']) || !isset($data['email']) || 
    !isset($data['signature_data']) || !isset($data['document_content'])) {
    http_response_code(400);
    echo json_encode(['success' => false, 'message' => 'Invalid input data']);
    exit;
}

$document_id = filter_var($data['document_id'], FILTER_SANITIZE_STRING);
$email = filter_var($data['email'], FILTER_SANITIZE_EMAIL);
$signature_data = $data['signature_data'];
$document_content = $data['document_content']; // Base64 encoded signed document

// Initialize eIDAS compliance module
$eidas = new EidasCompliance();

try {
    // 1. Verify user has been OTP verified - check otp_verification table
    $stmt = $eidas->db_conn->prepare("
        SELECT v.id
        FROM otp_verification v
        JOIN otp_requests r ON v.email = r.email
        WHERE v.email = :email 
        AND v.verified = 1
        AND r.document_id = :doc_id
        LIMIT 1
    ");
    
    $stmt->bindParam(':email', $email);
    $stmt->bindParam(':doc_id', $document_id);
    $stmt->execute();
    
    if (!$stmt->fetch()) {
        http_response_code(401);
        echo json_encode(['success' => false, 'message' => 'Identity not verified. Please complete OTP verification first.']);
        exit;
    }

    // 2. Verify document exists and is not already signed
    $stmt = $eidas->db_conn->prepare("
        SELECT status FROM documents 
        WHERE document_id = :doc_id AND status = 'uploaded'
    ");
    $stmt->bindParam(':doc_id', $document_id);
    $stmt->execute();
    
    if (!$stmt->fetch()) {
        http_response_code(400);
        echo json_encode(['success' => false, 'message' => 'Document not found or already signed']);
        exit;
    }

    // 3. Get trusted timestamp
    $timestamp_data = $eidas->getTrustedTimestamp();
    
    // 4. Add timestamp and IP to signature data
    $signature_data['timestamp'] = $timestamp_data;
    $signature_data['timestamp_iso'] = date('Y-m-d H:i:s', $timestamp_data);
    $signature_data['timestamp_source'] = 'worldtimeapi.org';
    $signature_data['ip_address'] = $_SERVER['REMOTE_ADDR'];

    // 5. Store the signed document with all eIDAS metadata
    $storage_result = storeSignedDocument(
        $document_id,
        $email,
        $signature_data,
        $document_content
    );
    
    if (!$storage_result['success']) {
        throw new Exception($storage_result['message']);
    }
    
    // 6. Return success response with eIDAS compliance details
    echo json_encode([
        'success' => true,
        'message' => 'Document signed successfully with eIDAS compliance',
        'document_id' => $document_id,
        'signature_id' => $storage_result['signature_id'],
        'timestamp' => $storage_result['timestamp'],
        'timestamp_iso' => $storage_result['timestamp_iso'],
        'eidas_level' => 'SES',  // Simple Electronic Signature
        'compliance_details' => [
            'identity_verified' => true,
            'timestamp_source' => 'worldtimeapi.org',
            'signature_algorithm' => 'SHA-256',
            'document_integrity_verified' => true
        ]
    ]);
} catch (Exception $e) {
    error_log("Error in sign-document.php: " . $e->getMessage());
    http_response_code(500);
    echo json_encode(['success' => false, 'message' => 'An error occurred while signing the document: ' . $e->getMessage()]);
}