<?php
/**
 * Document Upload API
 * Handles document uploads, generates SHA-256 hash, and stores document information
 */

// Enable error reporting in development (disable in production)
error_reporting(E_ALL);
ini_set('display_errors', 1);

// Required files
require_once 'db-config.php';
require_once 'eidas-compliance.php';

// Set headers for JSON response
header('Content-Type: application/json');

// Only allow POST requests
if ($_SERVER['REQUEST_METHOD'] !== 'POST') {
    http_response_code(405);
    echo json_encode(['success' => false, 'message' => 'Metodă nepermisă']);
    exit;
}

// Check if file was uploaded
if (!isset($_FILES['document']) || $_FILES['document']['error'] !== UPLOAD_ERR_OK) {
    $error_message = 'Eroare la încărcarea documentului';
    if (isset($_FILES['document']['error'])) {
        switch ($_FILES['document']['error']) {
            case UPLOAD_ERR_INI_SIZE:
                $error_message = 'Documentul depășește dimensiunea maximă permisă';
                break;
            case UPLOAD_ERR_FORM_SIZE:
                $error_message = 'Documentul depășește dimensiunea maximă permisă';
                break;
            case UPLOAD_ERR_PARTIAL:
                $error_message = 'Documentul a fost încărcat parțial';
                break;
            case UPLOAD_ERR_NO_FILE:
                $error_message = 'Niciun document nu a fost încărcat';
                break;
            case UPLOAD_ERR_NO_TMP_DIR:
                $error_message = 'Eroare de server: Lipsește directorul temporar';
                break;
            case UPLOAD_ERR_CANT_WRITE:
                $error_message = 'Eroare de server: Nu se poate scrie documentul pe disc';
                break;
            case UPLOAD_ERR_EXTENSION:
                $error_message = 'Eroare de server: Încărcarea a fost oprită de o extensie';
                break;
        }
    }
    
    http_response_code(400);
    echo json_encode(['success' => false, 'message' => $error_message]);
    exit;
}

// Validate user information
if (!isset($_POST['email']) || !filter_var($_POST['email'], FILTER_VALIDATE_EMAIL)) {
    http_response_code(400);
    echo json_encode(['success' => false, 'message' => 'Adresă de email invalidă']);
    exit;
}

$user_email = filter_var($_POST['email'], FILTER_SANITIZE_EMAIL);

// Initialize eIDAS compliance module
$eidas = new EidasCompliance();

// Create upload directory if it doesn't exist
$upload_dir = __DIR__ . '/uploads';
if (!is_dir($upload_dir)) {
    mkdir($upload_dir, 0755, true);
}

// Add index.html to prevent directory listing
if (!file_exists($upload_dir . '/index.html')) {
    file_put_contents($upload_dir . '/index.html', '<html><head><title>403 Forbidden</title></head><body><h1>Forbidden</h1><p>You don\'t have permission to access this resource.</p></body></html>');
}

// Add .htaccess for extra security
if (!file_exists($upload_dir . '/.htaccess')) {
    file_put_contents($upload_dir . '/.htaccess', "Options -Indexes\nDeny from all");
}

// Get file information
$file_name = $_FILES['document']['name'];
$file_tmp = $_FILES['document']['tmp_name'];
$file_size = $_FILES['document']['size'];
$file_type = $_FILES['document']['type'];

// Validate file type (only allow PDF files)
$allowed_types = ['application/pdf', 'application/x-pdf'];
if (!in_array($file_type, $allowed_types)) {
    http_response_code(400);
    echo json_encode(['success' => false, 'message' => 'Doar documentele PDF sunt acceptate']);
    exit;
}

try {
    // Generate document hash
    $document_hash = $eidas->generateDocumentHash($file_tmp);
    
    // Create a secure filename
    $secure_filename = $document_hash . '.pdf';
    $upload_path = $upload_dir . '/' . $secure_filename;
    
    // Move uploaded file
    if (move_uploaded_file($file_tmp, $upload_path)) {
        // Store document information in database
        $stored = $eidas->storeDocument($document_hash, $file_name, $file_size, $file_type, $user_email);
        
        if ($stored) {
            // Return success with document hash
            echo json_encode([
                'success' => true,
                'message' => 'Document încărcat cu succes',
                'document_id' => $document_hash,
                'file_name' => $file_name
            ]);
        } else {
            // Remove file if database storage failed
            @unlink($upload_path);
            
            http_response_code(500);
            echo json_encode(['success' => false, 'message' => 'Eroare la stocarea informațiilor documentului']);
        }
    } else {
        http_response_code(500);
        echo json_encode(['success' => false, 'message' => 'Eroare la mutarea documentului încărcat']);
    }
} catch (Exception $e) {
    http_response_code(500);
    echo json_encode(['success' => false, 'message' => 'Eroare: ' . $e->getMessage()]);
}