<?php
/**
 * Document Verification API
 * Validates document signatures and generates eIDAS compliance reports
 */

// Enable error reporting in development (disable in production)
error_reporting(E_ALL);
ini_set('display_errors', 0);
ini_set('log_errors', 1);
ini_set('error_log', __DIR__ . '/logs/verification_error.log');

// Required files
require_once 'config.php';
require_once 'db-config.php';
require_once 'eidas-compliance.php';
require_once 'document-storage.php';  // Include the new document storage file

// Set content type based on requested format
$format = isset($_GET['format']) ? strtolower($_GET['format']) : 'json';

switch ($format) {
    case 'html':
        header('Content-Type: text/html; charset=UTF-8');
        break;
    case 'pdf':
        header('Content-Type: application/pdf');
        break;
    default:
        header('Content-Type: application/json');
        $format = 'json';
}

// Get document ID from request
$document_id = isset($_GET['id']) ? filter_var($_GET['id'], FILTER_SANITIZE_STRING) : null;
$signature_id = isset($_GET['signature']) ? filter_var($_GET['signature'], FILTER_SANITIZE_STRING) : null;

if (!$document_id) {
    if ($format === 'json') {
        http_response_code(400);
        echo json_encode(['success' => false, 'message' => 'Document ID is required']);
    } else {
        showErrorPage('Document ID is required');
    }
    exit;
}

try {
    // Generate verification report
    $report = generateVerificationReport($document_id);
    
    if (!$report['success']) {
        if ($format === 'json') {
            http_response_code(404);
            echo json_encode($report);
        } else {
            showErrorPage($report['message']);
        }
        exit;
    }
    
    // Output based on requested format
    switch ($format) {
        case 'html':
            outputHtmlReport($report);
            break;
        case 'pdf':
            outputPdfReport($report);
            break;
        default:
            echo json_encode($report);
    }
} catch (Exception $e) {
    error_log("Error verifying document: " . $e->getMessage());
    
    if ($format === 'json') {
        http_response_code(500);
        echo json_encode(['success' => false, 'message' => 'Error verifying document: ' . $e->getMessage()]);
    } else {
        showErrorPage('Error verifying document: ' . $e->getMessage());
    }
}

/**
 * Show error page for HTML output
 */
function showErrorPage($message) {
    ?>
    <!DOCTYPE html>
    <html lang="en">
    <head>
        <meta charset="UTF-8">
        <meta name="viewport" content="width=device-width, initial-scale=1.0">
        <title>Document Verification Error</title>
        <style>
            body {
                font-family: Arial, sans-serif;
                line-height: 1.6;
                color: #333;
                max-width: 800px;
                margin: 0 auto;
                padding: 20px;
            }
            .error-container {
                background-color: #fef2f2;
                border: 1px solid #f87171;
                border-radius: 8px;
                padding: 20px;
                margin-top: 30px;
            }
            .error-icon {
                color: #ef4444;
                font-size: 48px;
                text-align: center;
                margin-bottom: 20px;
            }
            h1 {
                color: #b91c1c;
                text-align: center;
            }
            .back-link {
                display: block;
                text-align: center;
                margin-top: 30px;
                color: #2563eb;
                text-decoration: none;
            }
            .back-link:hover {
                text-decoration: underline;
            }
        </style>
    </head>
    <body>
        <div class="error-container">
            <div class="error-icon">⚠️</div>
            <h1>Verification Error</h1>
            <p><?php echo htmlspecialchars($message); ?></p>
        </div>
        <a href="/" class="back-link">Return to Homepage</a>
    </body>
    </html>
    <?php
}

/**
 * Output HTML verification report
 */
function outputHtmlReport($report) {
    ?>
    <!DOCTYPE html>
    <html lang="en">
    <head>
        <meta charset="UTF-8">
        <meta name="viewport" content="width=device-width, initial-scale=1.0">
        <title>eIDAS Document Verification Report</title>
        <style>
            body {
                font-family: Arial, sans-serif;
                line-height: 1.6;
                color: #333;
                max-width: 800px;
                margin: 0 auto;
                padding: 20px;
            }
            .report-header {
                text-align: center;
                margin-bottom: 30px;
            }
            .verification-status {
                text-align: center;
                padding: 20px;
                border-radius: 8px;
                margin: 20px 0;
            }
            .verification-success {
                background-color: #ecfdf5;
                border: 1px solid #6ee7b7;
            }
            .verification-warning {
                background-color: #fffbeb;
                border: 1px solid #fcd34d;
            }
            .verification-failed {
                background-color: #fef2f2;
                border: 1px solid #f87171;
            }
            .verification-icon {
                font-size: 48px;
                margin-bottom: 10px;
            }
            .info-section {
                margin: 20px 0;
                border: 1px solid #e2e8f0;
                border-radius: 8px;
                padding: 20px;
            }
            .section-title {
                margin-top: 0;
                padding-bottom: 10px;
                border-bottom: 1px solid #e2e8f0;
                color: #1e293b;
            }
            .info-row {
                display: flex;
                margin-bottom: 10px;
            }
            .info-label {
                width: 200px;
                font-weight: bold;
                color: #64748b;
            }
            .info-value {
                flex: 1;
            }
            .audit-trail {
                margin-top: 30px;
            }
            table {
                width: 100%;
                border-collapse: collapse;
                margin-top: 10px;
            }
            th, td {
                padding: 10px;
                border: 1px solid #e2e8f0;
                text-align: left;
            }
            th {
                background-color: #f8fafc;
                font-weight: bold;
            }
            tr:nth-child(even) {
                background-color: #f8fafc;
            }
            .footer {
                margin-top: 40px;
                text-align: center;
                font-size: 14px;
                color: #64748b;
                border-top: 1px solid #e2e8f0;
                padding-top: 20px;
            }
            .qr-code {
                text-align: center;
                margin: 20px 0;
            }
            .report-logo {
                max-width: 200px;
                margin-bottom: 20px;
            }
            .print-btn {
                display: block;
                text-align: center;
                margin: 20px auto;
                padding: 10px 20px;
                background-color: #2563eb;
                color: white;
                border: none;
                border-radius: 4px;
                cursor: pointer;
                font-size: 16px;
            }
            .print-btn:hover {
                background-color: #1d4ed8;
            }
            @media print {
                .print-btn {
                    display: none;
                }
                body {
                    padding: 0;
                    margin: 0;
                }
            }
        </style>
    </head>
    <body>
        <div class="report-header">
            <img src="/img/logo.png" alt="Inkless-Flow" class="report-logo">
            <h1>eIDAS Document Verification Report</h1>
            <p>This report details the verification status of an electronically signed document</p>
        </div>
        
        <button class="print-btn" onclick="window.print()">Print Report</button>
        
        <?php if ($report['is_signed'] && $report['integrity_verified']): ?>
        <div class="verification-status verification-success">
            <div class="verification-icon">✓</div>
            <h2>Document Verification Successful</h2>
            <p>This document has been successfully verified and is compliant with eIDAS regulations for Simple Electronic Signatures (SES).</p>
        </div>
        <?php elseif ($report['is_signed'] && !$report['integrity_verified']): ?>
        <div class="verification-status verification-warning">
            <div class="verification-icon">⚠️</div>
            <h2>Document Integrity Warning</h2>
            <p>This document has been signed, but its integrity could not be fully verified. The document may have been modified after signing.</p>
        </div>
        <?php else: ?>
        <div class="verification-status verification-failed">
            <div class="verification-icon">✕</div>
            <h2>Document Not Signed</h2>
            <p>This document has not been electronically signed or the signature could not be verified.</p>
        </div>
        <?php endif; ?>
        
        <div class="info-section">
            <h3 class="section-title">Document Information</h3>
            <div class="info-row">
                <div class="info-label">Document ID:</div>
                <div class="info-value"><?php echo htmlspecialchars($report['document_id']); ?></div>
            </div>
            <div class="info-row">
                <div class="info-label">File Name:</div>
                <div class="info-value"><?php echo htmlspecialchars($report['file_name']); ?></div>
            </div>
            <div class="info-row">
                <div class="info-label">Status:</div>
                <div class="info-value"><?php echo htmlspecialchars($report['status']); ?></div>
            </div>
            <div class="info-row">
                <div class="info-label">Uploaded By:</div>
                <div class="info-value"><?php echo htmlspecialchars($report['uploaded_by']); ?></div>
            </div>
            <div class="info-row">
                <div class="info-label">Upload Time:</div>
                <div class="info-value"><?php echo htmlspecialchars($report['upload_time']); ?></div>
            </div>
        </div>
        
        <?php if ($report['is_signed']): ?>
        <div class="info-section">
            <h3 class="section-title">Signature Information</h3>
            <div class="info-row">
                <div class="info-label">Signed By:</div>
                <div class="info-value"><?php echo htmlspecialchars($report['signer_email']); ?></div>
            </div>
            <div class="info-row">
                <div class="info-label">Signature Time:</div>
                <div class="info-value"><?php echo htmlspecialchars($report['signature_time']); ?></div>
            </div>
            <div class="info-row">
                <div class="info-label">Timestamp Source:</div>
                <div class="info-value"><?php echo htmlspecialchars($report['signature_source']); ?></div>
            </div>
            <div class="info-row">
                <div class="info-label">eIDAS Level:</div>
                <div class="info-value"><?php echo htmlspecialchars($report['eidas_compliance_level']); ?> (Simple Electronic Signature)</div>
            </div>
            <div class="info-row">
                <div class="info-label">Integrity Verified:</div>
                <div class="info-value"><?php echo $report['integrity_verified'] ? 'Yes' : 'No'; ?></div>
            </div>
        </div>
        <?php endif; ?>
        
        <div class="info-section audit-trail">
            <h3 class="section-title">Audit Trail</h3>
            <?php if (empty($report['audit_trail'])): ?>
                <p>No audit trail available for this document.</p>
            <?php else: ?>
                <table>
                    <thead>
                        <tr>
                            <th>Date & Time</th>
                            <th>User</th>
                            <th>Action</th>
                            <th>IP Address</th>
                        </tr>
                    </thead>
                    <tbody>
                        <?php foreach ($report['audit_trail'] as $entry): ?>
                        <tr>
                            <td><?php echo htmlspecialchars($entry['timestamp_iso']); ?></td>
                            <td><?php echo htmlspecialchars($entry['user_email']); ?></td>
                            <td><?php echo htmlspecialchars($entry['action']); ?></td>
                            <td><?php echo htmlspecialchars($entry['ip_address']); ?></td>
                        </tr>
                        <?php endforeach; ?>
                    </tbody>
                </table>
            <?php endif; ?>
        </div>
        
        <div class="qr-code">
            <div>Scan to verify document online:</div>
            <img src="https://api.qrserver.com/v1/create-qr-code/?size=150x150&data=<?php echo urlencode($report['verification_url']); ?>" alt="Verification QR Code" width="150" height="150">
        </div>
        
        <div class="footer">
            <p>This verification report was generated on <?php echo date('Y-m-d H:i:s'); ?></p>
            <p>Inkless-Flow eIDAS Compliance Report - <?php echo date('Y'); ?></p>
        </div>
    </body>
    </html>
    <?php
}

/**
 * Output PDF verification report
 */
function outputPdfReport($report) {
    // If you have installed a PDF library like TCPDF or FPDF, you can use it here
    // For this example, we'll include the necessary library
    
    // If TCPDF is available
    if (class_exists('TCPDF')) {
        generatePdfWithTCPDF($report);
    } else {
        // If no PDF library is available, fall back to HTML output
        header('Content-Type: text/html; charset=UTF-8');
        echo '<div style="background-color: #fff3cd; padding: 15px; margin-bottom: 20px; border: 1px solid #ffeeba; border-radius: 4px;">';
        echo '<h3 style="color: #856404; margin-top: 0;">PDF Generation Not Available</h3>';
        echo '<p>PDF generation requires the TCPDF library. Displaying HTML report instead.</p>';
        echo '</div>';
        
        outputHtmlReport($report);
    }
}

/**
 * Generate PDF report with TCPDF
 */
function generatePdfWithTCPDF($report) {
    // Create new PDF document
    $pdf = new TCPDF(PDF_PAGE_ORIENTATION, PDF_UNIT, PDF_PAGE_FORMAT, true, 'UTF-8', false);
    
    // Set document information
    $pdf->SetCreator('Inkless-Flow');
    $pdf->SetAuthor('Inkless-Flow eIDAS System');
    $pdf->SetTitle('Document Verification Report');
    $pdf->SetSubject('eIDAS Compliance Report');
    $pdf->SetKeywords('eIDAS, electronic signature, document verification');
    
    // Set default header data
    $pdf->SetHeaderData('', 0, 'Inkless-Flow eIDAS Compliance Report', 'Document Verification Report - Generated on ' . date('Y-m-d H:i:s'));
    
    // Set header and footer fonts
    $pdf->setHeaderFont(array('helvetica', '', 10));
    $pdf->setFooterFont(array('helvetica', '', 8));
    
    // Set default monospaced font
    $pdf->SetDefaultMonospacedFont('courier');
    
    // Set margins
    $pdf->SetMargins(15, 20, 15);
    $pdf->SetHeaderMargin(5);
    $pdf->SetFooterMargin(10);
    
    // Set auto page breaks
    $pdf->SetAutoPageBreak(true, 15);
    
    // Set image scale factor
    $pdf->setImageScale(1.25);
    
    // Add a page
    $pdf->AddPage();
    
    // Start with title
    $pdf->SetFont('helvetica', 'B', 16);
    $pdf->Cell(0, 10, 'eIDAS Document Verification Report', 0, 1, 'C');
    $pdf->SetFont('helvetica', '', 10);
    $pdf->Cell(0, 5, 'Generated on ' . date('Y-m-d H:i:s'), 0, 1, 'C');
    $pdf->Ln(10);
    
    // Add verification status
    $pdf->SetFont('helvetica', 'B', 14);
    
    if ($report['is_signed'] && $report['integrity_verified']) {
        $pdf->SetFillColor(236, 253, 245);
        $pdf->SetTextColor(22, 101, 52);
        $pdf->Cell(0, 10, 'Document Verification Successful', 0, 1, 'C', true);
        $pdf->SetTextColor(0, 0, 0);
        $pdf->SetFont('helvetica', '', 10);
        $pdf->Cell(0, 10, 'This document has been successfully verified and is compliant with eIDAS regulations.', 0, 1, 'C');
    } elseif ($report['is_signed'] && !$report['integrity_verified']) {
        $pdf->SetFillColor(255, 251, 235);
        $pdf->SetTextColor(133, 77, 14);
        $pdf->Cell(0, 10, 'Document Integrity Warning', 0, 1, 'C', true);
        $pdf->SetTextColor(0, 0, 0);
        $pdf->SetFont('helvetica', '', 10);
        $pdf->Cell(0, 10, 'Document has been signed, but its integrity could not be fully verified.', 0, 1, 'C');
    } else {
        $pdf->SetFillColor(254, 242, 242);
        $pdf->SetTextColor(153, 27, 27);
        $pdf->Cell(0, 10, 'Document Not Signed', 0, 1, 'C', true);
        $pdf->SetTextColor(0, 0, 0);
        $pdf->SetFont('helvetica', '', 10);
        $pdf->Cell(0, 10, 'This document has not been electronically signed or the signature could not be verified.', 0, 1, 'C');
    }
    
    $pdf->Ln(5);
    
    // Document Information section
    $pdf->SetFont('helvetica', 'B', 14);
    $pdf->Cell(0, 10, 'Document Information', 0, 1, 'L');
    
    $pdf->SetFont('helvetica', 'B', 10);
    $pdf->Cell(60, 7, 'Document ID:', 0, 0, 'L');
    $pdf->SetFont('helvetica', '', 10);
    $pdf->Cell(0, 7, $report['document_id'], 0, 1, 'L');
    
    $pdf->SetFont('helvetica', 'B', 10);
    $pdf->Cell(60, 7, 'File Name:', 0, 0, 'L');
    $pdf->SetFont('helvetica', '', 10);
    $pdf->Cell(0, 7, $report['file_name'], 0, 1, 'L');
    
    $pdf->SetFont('helvetica', 'B', 10);
    $pdf->Cell(60, 7, 'Status:', 0, 0, 'L');
    $pdf->SetFont('helvetica', '', 10);
    $pdf->Cell(0, 7, $report['status'], 0, 1, 'L');
    
    $pdf->SetFont('helvetica', 'B', 10);
    $pdf->Cell(60, 7, 'Uploaded By:', 0, 0, 'L');
    $pdf->SetFont('helvetica', '', 10);
    $pdf->Cell(0, 7, $report['uploaded_by'], 0, 1, 'L');
    
    $pdf->SetFont('helvetica', 'B', 10);
    $pdf->Cell(60, 7, 'Upload Time:', 0, 0, 'L');
    $pdf->SetFont('helvetica', '', 10);
    $pdf->Cell(0, 7, $report['upload_time'], 0, 1, 'L');
    
    $pdf->Ln(5);
    
    // Signature Information section if signed
    if ($report['is_signed']) {
        $pdf->SetFont('helvetica', 'B', 14);
        $pdf->Cell(0, 10, 'Signature Information', 0, 1, 'L');
        
        $pdf->SetFont('helvetica', 'B', 10);
        $pdf->Cell(60, 7, 'Signed By:', 0, 0, 'L');
        $pdf->SetFont('helvetica', '', 10);
        $pdf->Cell(0, 7, $report['signer_email'], 0, 1, 'L');
        
        $pdf->SetFont('helvetica', 'B', 10);
        $pdf->Cell(60, 7, 'Signature Time:', 0, 0, 'L');
        $pdf->SetFont('helvetica', '', 10);
        $pdf->Cell(0, 7, $report['signature_time'], 0, 1, 'L');
        
        $pdf->SetFont('helvetica', 'B', 10);
        $pdf->Cell(60, 7, 'Timestamp Source:', 0, 0, 'L');
        $pdf->SetFont('helvetica', '', 10);
        $pdf->Cell(0, 7, $report['signature_source'], 0, 1, 'L');
        
        $pdf->SetFont('helvetica', 'B', 10);
        $pdf->Cell(60, 7, 'eIDAS Level:', 0, 0, 'L');
        $pdf->SetFont('helvetica', '', 10);
        $pdf->Cell(0, 7, $report['eidas_compliance_level'] . ' (Simple Electronic Signature)', 0, 1, 'L');
        
        $pdf->SetFont('helvetica', 'B', 10);
        $pdf->Cell(60, 7, 'Integrity Verified:', 0, 0, 'L');
        $pdf->SetFont('helvetica', '', 10);
        $pdf->Cell(0, 7, $report['integrity_verified'] ? 'Yes' : 'No', 0, 1, 'L');
        
        $pdf->Ln(5);
    }
    
    // Audit Trail section
    $pdf->SetFont('helvetica', 'B', 14);
    $pdf->Cell(0, 10, 'Audit Trail', 0, 1, 'L');
    
    if (empty($report['audit_trail'])) {
        $pdf->SetFont('helvetica', '', 10);
        $pdf->Cell(0, 7, 'No audit trail available for this document.', 0, 1, 'L');
    } else {
        // Create table headers
        $pdf->SetFont('helvetica', 'B', 9);
        $pdf->Cell(50, 7, 'Date & Time', 1, 0, 'C', true);
        $pdf->Cell(60, 7, 'User', 1, 0, 'C', true);
        $pdf->Cell(40, 7, 'Action', 1, 0, 'C', true);
        $pdf->Cell(40, 7, 'IP Address', 1, 1, 'C', true);
        
        // Fill table rows
        $pdf->SetFont('helvetica', '', 8);
        
        foreach ($report['audit_trail'] as $entry) {
            $pdf->Cell(50, 6, $entry['timestamp_iso'], 1, 0, 'L');
            $pdf->Cell(60, 6, $entry['user_email'], 1, 0, 'L');
            $pdf->Cell(40, 6, $entry['action'], 1, 0, 'L');
            $pdf->Cell(40, 6, $entry['ip_address'], 1, 1, 'L');
        }
    }
    
    // Add verification URL
    $pdf->Ln(10);
    $pdf->SetFont('helvetica', 'B', 10);
    $pdf->Cell(0, 7, 'Verification URL:', 0, 1, 'L');
    $pdf->SetFont('helvetica', '', 10);
    $pdf->Cell(0, 7, $report['verification_url'], 0, 1, 'L');
    
    // Add legal footer
    $pdf->Ln(10);
    $pdf->SetFont('helvetica', 'I', 8);
    $pdf->Cell(0, 5, 'This document was verified according to eIDAS Regulation (EU) No 910/2014 standards.', 0, 1, 'C');
    $pdf->Cell(0, 5, 'Inkless-Flow eIDAS Compliance Service - ' . date('Y'), 0, 1, 'C');
    
    // Output the PDF
    $pdf->Output('eIDAS_Verification_Report_' . substr($report['document_id'], 0, 8) . '.pdf', 'D');
    
    exit; // Important to prevent further output
}
?>