<?php
/**
 * OTP Verification Endpoint
 * Validates OTP codes sent via email
 */

// Enable error reporting and detailed logging
error_reporting(E_ALL);
ini_set('display_errors', 0); // Don't display errors to users
ini_set('log_errors', 1);
ini_set('error_log', __DIR__ . '/logs/otp_error.log');
error_log("====== " . date('Y-m-d H:i:s') . " - OTP Verification Operation Started ======");

// Set headers for JSON response
header('Content-Type: application/json');

try {
    // Required files - wrapped in try/catch to handle any include errors
    require_once 'config.php';
    require_once 'db-config.php';
    require_once 'eidas-compliance.php';

    // Only allow POST requests
    if ($_SERVER['REQUEST_METHOD'] !== 'POST') {
        http_response_code(405);
        echo json_encode(['success' => false, 'message' => 'Method Not Allowed']);
        exit;
    }

    // Get POST data - handle both form data and JSON input
    $request_body = file_get_contents('php://input');
    $json_data = json_decode($request_body, true);
    
    // Use data from either POST or JSON body
    $email = isset($_POST['email']) ? filter_var($_POST['email'], FILTER_SANITIZE_EMAIL) : 
             (isset($json_data['email']) ? filter_var($json_data['email'], FILTER_SANITIZE_EMAIL) : null);
    
    $otp = isset($_POST['otp']) ? filter_var($_POST['otp'], FILTER_SANITIZE_STRING) : 
           (isset($json_data['otp']) ? filter_var($json_data['otp'], FILTER_SANITIZE_STRING) : null);
    
    $document_id = isset($_POST['document_id']) ? filter_var($_POST['document_id'], FILTER_SANITIZE_STRING) : 
                   (isset($json_data['document_id']) ? filter_var($json_data['document_id'], FILTER_SANITIZE_STRING) : null);
    
    error_log("Verifying OTP for email: $email, OTP: $otp, document: $document_id");

    // Validate input data
    if (!$email || !filter_var($email, FILTER_VALIDATE_EMAIL) || !$otp) {
        error_log("Invalid verification data - Email: $email, OTP provided: " . ($otp ? 'Yes' : 'No'));
        http_response_code(400);
        echo json_encode(['success' => false, 'message' => 'Invalid input data']);
        exit;
    }

    // For development environment, check manual OTPs file first
    $is_dev = (defined('APP_ENV') && APP_ENV === 'development');

    if ($is_dev) {
        error_log("Development mode: checking alternative OTP verification methods");
        
        // Check in development_otps.txt file
        $dev_log = __DIR__ . '/logs/development_otps.txt';
        if (file_exists($dev_log)) {
            $log_content = file_get_contents($dev_log);
            if (strpos($log_content, "Email: $email") !== false && 
                strpos($log_content, "OTP: $otp") !== false) {
                
                error_log("Development OTP verified from file for: $email");
                echo json_encode([
                    'success' => true, 
                    'message' => 'OTP verified successfully (Development mode)',
                    'verified' => true,
                    'mode' => 'development'
                ]);
                exit;
            }
        }
        
        // Check in otp.log file
        $otp_log = __DIR__ . '/logs/otp.log';
        if (file_exists($otp_log)) {
            $log_content = file_get_contents($otp_log);
            if (strpos($log_content, "Development OTP for $email: $otp") !== false) {
                error_log("Development OTP verified from otp.log for: $email");
                echo json_encode([
                    'success' => true, 
                    'message' => 'OTP verified successfully (Development mode)',
                    'verified' => true,
                    'mode' => 'development'
                ]);
                exit;
            }
        }
    }

    // Check the database for the OTP
    try {
        $stmt = $db_conn->prepare("
            SELECT * FROM otp_requests 
            WHERE email = :email 
            AND otp = :otp 
            AND expires_at > :now
            ORDER BY request_time DESC 
            LIMIT 1
        ");
        
        $now = time();
        $stmt->bindParam(':email', $email);
        $stmt->bindParam(':otp', $otp);
        $stmt->bindParam(':now', $now);
        $stmt->execute();
        
        $result = $stmt->fetch(PDO::FETCH_ASSOC);
        
        if ($result) {
            // Mark this OTP as verified
            $update_stmt = $db_conn->prepare("
                UPDATE otp_requests 
                SET verified = 1, 
                    verification_ip = :ip, 
                    verification_time = :time 
                WHERE email = :email 
                AND otp = :otp
            ");
            
            $ip = $_SERVER['REMOTE_ADDR'];
            $verification_time = time();
            
            $update_stmt->bindParam(':ip', $ip);
            $update_stmt->bindParam(':time', $verification_time);
            $update_stmt->bindParam(':email', $email);
            $update_stmt->bindParam(':otp', $otp);
            $update_stmt->execute();
            
            error_log("OTP verified successfully for: $email");
            
            // Create audit log if document_id is provided
            if ($document_id && function_exists('createAuditLog')) {
                createAuditLog($document_id, $email, 'otp_verified', [
                    'verification_time' => $verification_time,
                    'verification_ip' => $ip
                ]);
            }
            
            echo json_encode([
                'success' => true, 
                'message' => 'OTP verified successfully',
                'verified' => true
            ]);
        } else {
            // Check if OTP exists but is expired
            $stmt = $db_conn->prepare("
                SELECT expires_at FROM otp_requests 
                WHERE email = :email 
                AND otp = :otp 
                ORDER BY request_time DESC 
                LIMIT 1
            ");
            
            $stmt->bindParam(':email', $email);
            $stmt->bindParam(':otp', $otp);
            $stmt->execute();
            
            $expired_result = $stmt->fetch(PDO::FETCH_ASSOC);
            
            if ($expired_result) {
                $expires_at = $expired_result['expires_at'];
                $current_time = time();
                
                if ($current_time > $expires_at) {
                    error_log("Expired OTP for $email. Expired at: " . date('Y-m-d H:i:s', $expires_at));
                    http_response_code(401);
                    echo json_encode([
                        'success' => false, 
                        'message' => 'OTP has expired. Please request a new one.',
                        'verified' => false,
                        'expired' => true
                    ]);
                } else {
                    error_log("Invalid OTP for $email but not expired");
                    http_response_code(401);
                    echo json_encode([
                        'success' => false, 
                        'message' => 'Invalid OTP. Please check and try again.',
                        'verified' => false
                    ]);
                }
            } else {
                error_log("No OTP found for $email with code: $otp");
                http_response_code(401);
                echo json_encode([
                    'success' => false, 
                    'message' => 'Invalid OTP or email. Please request a new OTP.',
                    'verified' => false
                ]);
            }
        }
    } catch (PDOException $e) {
        error_log("Database error checking OTP: " . $e->getMessage());
        http_response_code(500);
        echo json_encode([
            'success' => false, 
            'message' => 'Error verifying OTP. Please try again later.',
            'verified' => false
        ]);
    }
} catch (Exception $e) {
    // Log the error but don't expose details to the user
    error_log("Error in verify-otp.php: " . $e->getMessage());
    
    http_response_code(500);
    echo json_encode([
        'success' => false, 
        'message' => 'An error occurred while verifying the code. Please try again.'
    ]);
}

/**
 * Create audit log entry if eidas-compliance.php is not loaded
 */
function createAuditLog($document_id, $user_email, $action, $additional_data = null) {
    try {
        global $db_conn;
        
        $stmt = $db_conn->prepare("
            INSERT INTO audit_logs (
                document_id, user_email, action, timestamp,
                ip_address, user_agent, additional_data
            ) VALUES (
                :doc_id, :email, :action, :timestamp,
                :ip, :user_agent, :data
            )
        ");
        
        $timestamp = time();
        $ip = $_SERVER['REMOTE_ADDR'];
        $user_agent = isset($_SERVER['HTTP_USER_AGENT']) ? $_SERVER['HTTP_USER_AGENT'] : null;
        $data_json = $additional_data ? json_encode($additional_data) : null;
        
        $stmt->bindParam(':doc_id', $document_id);
        $stmt->bindParam(':email', $user_email);
        $stmt->bindParam(':action', $action);
        $stmt->bindParam(':timestamp', $timestamp);
        $stmt->bindParam(':ip', $ip);
        $stmt->bindParam(':user_agent', $user_agent);
        $stmt->bindParam(':data', $data_json);
        
        $stmt->execute();
        
        return true;
    } catch (Exception $e) {
        error_log("Error creating audit log: " . $e->getMessage());
        return false;
    }
}